@@ -2,11 +2,11 @@
 
                           Firewall Builder
 
-                 Copyright (C) 2003 NetCitadel, LLC
+                 Copyright (C) 2007 NetCitadel, LLC
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: definitions.h,v 1.2 2006/02/18 22:33:19 vkurland Exp $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,7 +30,9 @@
 #ifndef __DEFINITIONS_
 #define __DEFINITIONS_
 
+// Various #defines that are needed both in the GUI and compilers
 
+// definitions for ipfw classify methods
 #define DIVERTSOCKET 0
 #define DUMMYNETPIPE 1
 #define DUMMYNETQUEUE 2
