@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_cisco.cpp,v 1.1 2008/03/06 06:48:56 vkurland Exp $
+  $Id: PolicyCompiler_cisco.cpp 407 2008-07-22 15:20:34Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -51,7 +51,7 @@
 #include <iomanip>
 #include <algorithm>
 #include <functional>
-
+#include <cstring>
 #include <assert.h>
 
 using namespace libfwbuilder;
@@ -62,8 +62,9 @@ string PolicyCompiler_cisco::myPlatformName() { return ""; }
 
 PolicyCompiler_cisco::PolicyCompiler_cisco(FWObjectDatabase *_db,
                                            const std::string &fwname,
+                                           bool ipv6_policy,
                                            OSConfigurator *_oscnf) :
-    PolicyCompiler(_db,fwname,_oscnf) , helper(this)
+    PolicyCompiler(_db, fwname, ipv6_policy, _oscnf) , helper(this)
 {
 }
 
@@ -106,15 +107,19 @@ void PolicyCompiler_cisco::addDefaultPolicyRule()
          !getCachedFwOpt()->getStr("mgmt_addr").empty() )
     {
         PolicyRule *r;
-        TCPService *ssh=TCPService::cast(dbcopy->create(TCPService::TYPENAME) );
-        ssh->setInt("dst_range_start",22);
-        ssh->setInt("dst_range_end",22);
+        TCPService *ssh = TCPService::cast(
+            dbcopy->create(TCPService::TYPENAME) );
+        ssh->setDstRangeStart(22);
+        ssh->setDstRangeEnd(22);
         dbcopy->add(ssh,false);
         cacheObj(ssh); // to keep cache consistent
 
-        Network *mgmt_workstation = Network::cast(dbcopy->create(Network::TYPENAME));
-        *mgmt_workstation = getCachedFwOpt()->getStr("mgmt_addr");
-        dbcopy->add(mgmt_workstation,false);
+        Network *mgmt_workstation = Network::cast(
+            dbcopy->create(Network::TYPENAME));
+        mgmt_workstation->setAddressNetmask(
+            getCachedFwOpt()->getStr("mgmt_addr"));
+
+        dbcopy->add(mgmt_workstation, false);
         cacheObj(mgmt_workstation); // to keep cache consistent
 
         r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME) );
@@ -221,11 +226,11 @@ bool PolicyCompiler_cisco::splitIfDstAny::processNext()
         if (ICMPService::isA(s))  cl.push_back(s);
 
         if (TCPService::isA(s) && 
-            s->getInt("dst_range_start")==22 && 
-            s->getInt("dst_range_end")==22) cl.push_back(s);
+            TCPUDPService::cast(s)->getDstRangeStart()==22 && 
+            TCPUDPService::cast(s)->getDstRangeEnd()==22) cl.push_back(s);
         if (TCPService::isA(s) && 
-            s->getInt("dst_range_start")==23 && 
-            s->getInt("dst_range_end")==23) cl.push_back(s);
+            TCPUDPService::cast(s)->getDstRangeStart()==23 && 
+            TCPUDPService::cast(s)->getDstRangeEnd()==23) cl.push_back(s);
     }
 
 
@@ -409,7 +414,7 @@ bool PolicyCompiler_cisco::splitIfRuleElementMatchesFW::processNext()
         Address *a=Address::cast(obj);
         assert(a!=NULL);
 
-//        IPAddress obj_addr=a->getAddress();
+//        InetAddr obj_addr=a->getAddress();
 
         if (cisco_comp->complexMatch(a,cisco_comp->fw)) {
 
@@ -535,8 +540,8 @@ bool PolicyCompiler_cisco::tcpServiceToFW::processNext()
             assert(s!=NULL);
 
             if (TCPService::isA(s) && 
-                s->getInt("dst_range_start")==port && 
-                s->getInt("dst_range_end")==port) cl.push_back(o);
+                TCPUDPService::cast(s)->getDstRangeStart()==port && 
+                TCPUDPService::cast(s)->getDstRangeEnd()==port) cl.push_back(o);
         }
         if (!cl.empty()) 
         {
@@ -631,7 +636,8 @@ bool PolicyCompiler_cisco::replaceFWinDSTPolicy::processNext()
         {
             try
             {
-                string iface_id=helper.findInterfaceByNetzone(compiler->getFirstSrc(rule));
+                int iface_id = helper.findInterfaceByNetzone(
+                    compiler->getFirstSrc(rule));
                 Interface *iface = compiler->getCachedFwInterface(iface_id);
 
                 dst->clearChildren();
@@ -653,17 +659,16 @@ bool PolicyCompiler_cisco::replaceFWinDSTPolicy::processNext()
 }
 
 void PolicyCompiler_cisco::splitByNetworkZonesForRE::AddToInterface(
-    const std::string &interface_id,
-    libfwbuilder::Address *addr,
-    PolicyRule *rule)
+    int interface_id, Address *addr, PolicyRule *rule)
 {
     PolicyRule  *new_rule;
     RuleElement *new_re;
 
-    new_rule=rules[interface_id];
+    new_rule = rules[interface_id];
     if (new_rule==NULL) 
     {
-        new_rule= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+        new_rule = PolicyRule::cast(compiler->dbcopy->create(
+                                        PolicyRule::TYPENAME) );
         compiler->temp_ruleset->add(new_rule);
         new_rule->duplicate(rule);
         rules[interface_id]=new_rule;
@@ -697,11 +702,11 @@ bool PolicyCompiler_cisco::splitByNetworkZonesForRE::processNext()
         Address *a=Address::cast(obj);
         assert(a!=NULL);
 
-//        IPAddress obj_addr=a->getAddress();
+//        InetAddr obj_addr=a->getAddress();
 
         try
         {
-            string interface_id=helper.findInterfaceByNetzone(a);
+            int interface_id = helper.findInterfaceByNetzone(a);
             AddToInterface(interface_id, a, rule);
         } catch (string err)
         {
@@ -727,7 +732,7 @@ bool PolicyCompiler_cisco::splitByNetworkZonesForRE::processNext()
             }
         }
     }
-    for (std::map<std::string,PolicyRule*>::iterator i=rules.begin();
+    for (std::map<int,PolicyRule*>::iterator i=rules.begin();
          i!=rules.end(); ++i)
     {
         tmp_queue.push_back((*i).second);        
