@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: AddressRangeDialog.h,v 1.7 2006/05/13 06:53:05 vkurland Exp $
+  $Id: AddressRangeDialog.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,21 +27,27 @@
 #ifndef __ADDRESSRANGEDIALOG_H_
 #define __ADDRESSRANGEDIALOG_H_
 
-#include "config.h"
-#include <addressrangedialog_q.h>
+#include "../../config.h"
+#include <ui_addressrangedialog_q.h>
+#include <QWidget>
 
 #include "fwbuilder/FWObject.h"
 
 
-class AddressRangeDialog : public AddressRangeDialog_q
+class ProjectPanel;
+class AddressRangeDialog : public QWidget
 {
     Q_OBJECT
 
     libfwbuilder::FWObject *obj;
     bool init;
+    
+    Ui::AddressRangeDialog_q *m_dialog;
 
+    ProjectPanel *m_project;
  public:
-    AddressRangeDialog(QWidget *parent) : AddressRangeDialog_q(parent) { obj=NULL; }
+    AddressRangeDialog(ProjectPanel *project, QWidget *parent);
+    ~AddressRangeDialog();
     
 public slots:
     virtual void changed();
