@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ColorLabelMenuItem.cpp,v 1.2 2006/10/22 04:39:36 vkurland Exp $
+  $Id: ColorLabelMenuItem.cpp 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,14 +17,14 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -43,28 +43,31 @@ using namespace std;
 
 
 ColorLabelMenuItem::ColorLabelMenuItem(QWidget *parent) :
-    colorLabelMenuItem_q(parent)
+    QWidget(parent)
 {
-    setup( noneBtn,   "#FFFFFF", tr("no color") );
-    setup( redBtn,
+    m_widget = new Ui::colorLabelMenuItem_q;
+    m_widget->setupUi(this);
+
+    setup( m_widget->noneBtn,   "#FFFFFF", tr("no color") );
+    setup( m_widget->redBtn,
            st->getLabelColor(FWBSettings::RED   ),
            st->getLabelText(FWBSettings::RED   ));
-    setup( orangeBtn,
+    setup( m_widget->orangeBtn,
            st->getLabelColor(FWBSettings::ORANGE),
            st->getLabelText(FWBSettings::ORANGE));
-    setup( yellowBtn,
+    setup( m_widget->yellowBtn,
            st->getLabelColor(FWBSettings::YELLOW),
            st->getLabelText(FWBSettings::YELLOW));
-    setup( greenBtn,
+    setup( m_widget->greenBtn,
            st->getLabelColor(FWBSettings::GREEN ),
            st->getLabelText(FWBSettings::GREEN ));
-    setup( blueBtn,
+    setup( m_widget->blueBtn,
            st->getLabelColor(FWBSettings::BLUE  ),
            st->getLabelText(FWBSettings::BLUE  ));
-    setup( purpleBtn,
+    setup( m_widget->purpleBtn,
            st->getLabelColor(FWBSettings::PURPLE),
            st->getLabelText(FWBSettings::PURPLE));
-    setup( grayBtn,
+    setup( m_widget->grayBtn,
            st->getLabelColor(FWBSettings::GRAY  ),
            st->getLabelText(FWBSettings::GRAY  ));
 }
@@ -76,8 +79,8 @@ void ColorLabelMenuItem::setup(QToolButton *btn,
     pm.fill( QColor(c) );
     QPainter p( &pm );
     p.drawRect( pm.rect() );
-    btn->setPixmap(pm);
-    QToolTip::add(btn, t);
+    btn->setIcon(QIcon(pm));
+    btn->setToolTip(t);
 }
 
 void ColorLabelMenuItem::colorClicked()
