@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: CustomServiceDialog.cpp,v 1.21 2007/04/14 00:18:43 vkurland Exp $
+  $Id: CustomServiceDialog.cpp 565 2008-10-04 14:34:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,20 +17,20 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
+#include "ProjectPanel.h"
 
 #include "FWBTree.h"
 #include "CustomServiceDialog.h"
-#include "ObjectManipulator.h"
 #include "FWBSettings.h"
 
 #include "fwbuilder/Library.h"
@@ -43,10 +43,24 @@
 #include <qpushbutton.h>
 
 #include <iostream>
+#include "FWBSettings.h"
 
+#include "FWWindow.h"
 using namespace libfwbuilder;
 using namespace std;
 
+CustomServiceDialog::CustomServiceDialog(ProjectPanel *project, QWidget *parent) : QWidget(parent), m_project(project)
+{
+    m_dialog = new Ui::CustomServiceDialog_q;
+    m_dialog->setupUi(this);
+    obj=NULL;
+}
+
+CustomServiceDialog::~CustomServiceDialog()
+{
+    delete m_dialog;
+}
+
 void CustomServiceDialog::loadFWObject(FWObject *o)
 {
     obj=o;
@@ -55,61 +69,58 @@ void CustomServiceDialog::loadFWObject(FWObject *o)
 
     init=true;
 
-    fillLibraries(libs,obj);
-
-    obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
-    comment->setText( QString::fromUtf8(s->getComment().c_str()) );
-
-/* fill in platform */
-    platform->clear();
+    m_dialog->protocol_string_label->setVisible (false); //!REM 
+    m_dialog->protocol_string->setVisible (false); //!REM 
+    m_dialog->obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
+    m_dialog->comment->setText( QString::fromUtf8(s->getComment().c_str()) );
+    //!NEW m_dialog->protocl_string->setText(QString::fromUtf8(s->getProtocolStringForPlatform().c_str()));
+/* fill in m_dialog->platform */
+    m_dialog->platform->clear();
 
     int cp=0;
-    showPlatform=st->readEntry("/FirewallBuilder2/CustomService/Platform");
+    showPlatform=st->value(SETTINGS_PATH_PREFIX"/CustomService/Platform").toString();
     QMap<QString,QString> platforms = getAllPlatforms();
     QMap<QString,QString>::iterator i;
     for (i=platforms.begin(); i!=platforms.end(); i++,cp++)
     {
-//        cerr << "platform: key=" << i.key() << "  data=" << i.data() << endl;
+//        cerr << "m_dialog->platform: key=" << i.key() << "  data=" << i.data() << endl;
 
-/* here i.key is platform code ( "ipf", "ipfw", "iptables", "pf")
+/* here i.key is m_dialog->platform m_dialog->code ( "ipf", "ipfw", "iptables", "pf")
  * while i.data is human readable name ("ipfilter", "PF" )
  */
-        platformReverseMap[i.data()]=i.key();
+        platformReverseMap[i.value()]=i.key();
 
-        platform->insertItem( i.data() );
+        m_dialog->platform->addItem( i.value() );
         if (showPlatform=="") showPlatform = i.key();
-        if (showPlatform==i.key()) platform->setCurrentItem( cp );
-        allCodes[ i.key() ]=s->getCodeForPlatform( i.key().latin1() ).c_str();
+        if (showPlatform==i.key()) m_dialog->platform->setCurrentIndex( cp );
+        allCodes[ i.key() ]=s->getCodeForPlatform( i.key().toLatin1().constData() ).c_str();
     }
 
-    code->setText( QString::fromUtf8(allCodes[showPlatform] ));
+    m_dialog->code->setText( allCodes[showPlatform] ); //fromUtf8
 
     //apply->setEnabled( false );
 
-    obj_name->setEnabled(!o->isReadOnly());
-    setDisabledPalette(obj_name);
+    m_dialog->obj_name->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->obj_name);
 
-    libs->setEnabled(!o->isReadOnly());
-    setDisabledPalette(libs);
+    m_dialog->comment->setReadOnly(o->isReadOnly());
+    setDisabledPalette(m_dialog->comment);
 
-    comment->setReadOnly(o->isReadOnly());
-    setDisabledPalette(comment);
+//    m_dialog->platform->setEnabled(!o->isReadOnly());
 
-//    platform->setEnabled(!o->isReadOnly());
-
-    code->setEnabled(!o->isReadOnly());
-    setDisabledPalette(code);
+    m_dialog->code->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->code);
 
 
     init=false;
 }
-    
+
 void CustomServiceDialog::changed()
 {
     if (!init)
     {
-        QString  pl   = platformReverseMap[platform->currentText()];
-        allCodes[pl] = code->text().utf8();
+        QString  pl   = platformReverseMap[m_dialog->platform->currentText()];
+        allCodes[pl] = m_dialog->code->text().toUtf8().constData();
 
         //apply->setEnabled( true );
         emit changed_sign();
@@ -121,10 +132,10 @@ void CustomServiceDialog::validate(bool *res)
 {
     *res=true;
     if (!isTreeReadWrite(this,obj)) { *res=false; return; }
-    if (!validateName(this,obj,obj_name->text())) { *res=false; return; }
+    if (!validateName(this,obj,m_dialog->obj_name->text())) { *res=false; return; }
 }
 
-void CustomServiceDialog::isChanged(bool *res)
+void CustomServiceDialog::isChanged(bool*)
 {
     //*res=(!init && apply->isEnabled());
 }
@@ -137,10 +148,10 @@ void CustomServiceDialog::libChanged()
 void CustomServiceDialog::platformChanged()
 {
     init=true;
-    QString npl = platformReverseMap[platform->currentText()];
-    code->setText( QString::fromUtf8(allCodes[ npl ]) );
+    QString npl = platformReverseMap[m_dialog->platform->currentText()];
+    m_dialog->code->setText( allCodes[ npl ] ); //fromUtf8
     showPlatform = npl;
-    st->writeEntry("/FirewallBuilder2/CustomService/Platform",showPlatform);
+    st->setValue(SETTINGS_PATH_PREFIX"/CustomService/Platform",showPlatform);
     init=false;
 //    changed();
 }
@@ -151,31 +162,23 @@ void CustomServiceDialog::applyChanges()
     assert(s!=NULL);
 
     string oldname=obj->getName();
-    obj->setName( string(obj_name->text().utf8()) );
-    string commText = string(comment->text().utf8());
+    obj->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
+    string commText = string(m_dialog->comment->toPlainText().toUtf8().constData());
     obj->setComment( commText );
-
+    //!NEW s->setProtocolStringForPlatform(string(m_dialog->protocol_string->text().toUtf8().constData()));
     QMap<QString,QString>::iterator i;
     for (i=allCodes.begin(); i!=allCodes.end(); ++i)
     {
-        string code = string(i.data().utf8());
+        string code = string(i.value().toUtf8().constData());
         if (fwbdebug)
-            qDebug("Storing custom service code %s :: %s",
-                   i.key().latin1(),code.c_str());
-        s->setCodeForPlatform( i.key().latin1(), code );
+            qDebug("Storing custom service m_dialog->code %s :: %s",
+                   i.key().toLatin1().constData(),code.c_str());
+        s->setCodeForPlatform( i.key().toLatin1().constData(), code );
     }
-    om->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
- 
-    init=true;
-
-/* move to another lib if we have to */
-    if (! FWBTree::isSystem(obj) && libs->currentText() != QString(obj->getLibrary()->getName().c_str()))
-        om->moveObject(libs->currentText(), obj);
-
-    init=false;
+    mw->updateObjName(obj,QString::fromUtf8(oldname.c_str()));
 
     //apply->setEnabled( false );
-    om->updateLastModifiedTimestampForAllFirewalls(obj);
+    mw->updateLastModifiedTimestampForAllFirewalls(obj);
 }
 
 void CustomServiceDialog::discardChanges()
