@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: CustomServiceDialog.h,v 1.7 2006/05/13 06:53:05 vkurland Exp $
+  $Id: CustomServiceDialog.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,8 +27,9 @@
 #ifndef __CUSTOMSERVICEDIALOG_H_
 #define __CUSTOMSERVICEDIALOG_H_
 
-#include "config.h"
-#include <customservicedialog_q.h>
+#include "../../config.h"
+#include <ui_customservicedialog_q.h>
+#include <QDialog>
 
 #include <qstring.h>
 #include <qmap.h>
@@ -37,7 +38,8 @@ namespace libfwbuilder {
     class FWObject;
 };
 
-class CustomServiceDialog : public CustomServiceDialog_q
+class ProjectPanel;
+class CustomServiceDialog : public QWidget
 {
     Q_OBJECT
 
@@ -46,9 +48,12 @@ class CustomServiceDialog : public CustomServiceDialog_q
     QMap<QString, QString>     platformReverseMap;
     QMap<QString, QString>     allCodes;
     QString                    showPlatform;
+    Ui::CustomServiceDialog_q *m_dialog;
+    ProjectPanel *m_project;
     
  public:
-    CustomServiceDialog(QWidget *parent) : CustomServiceDialog_q(parent) { obj=NULL; }
+    CustomServiceDialog(ProjectPanel *project, QWidget *parent);
+    ~CustomServiceDialog();
     
 public slots:
     virtual void changed();
