@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: DialogFactory.h,v 1.3 2005/11/19 05:27:26 vkurland Exp $
+  $Id: DialogFactory.h 304 2008-07-03 02:19:16Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,12 +29,12 @@ namespace libfwbuilder {
     class FWObject;
     class FWException;
 };
-
+class ProjectPanel;
 class DialogFactory {
 
  public:
 
-    static QWidget *createDialog(QWidget *parent,const QString &objType);
+    static QWidget *createDialog(ProjectPanel *project, QWidget *parent,const QString &objType);
     static QWidget *createFWDialog(QWidget *parent,libfwbuilder::FWObject *o)
         throw(libfwbuilder::FWException);
     static QWidget *createOSDialog(QWidget *parent,libfwbuilder::FWObject *o)
