@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWBSettings.h,v 1.20 2006/06/26 03:21:00 vkurland Exp $
+  $Id: FWBSettings.h 565 2008-10-04 14:34:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -29,6 +29,9 @@
 #include <qsettings.h>
 #include <qrect.h>
 #include <qprinter.h>
+#include <qfont.h>
+
+#include <set>
 
 class QWidget;
 
@@ -40,13 +43,12 @@ class QWidget;
 #define LoadStandardObjects 0
 #define LoadLastEditedFile  1
     
-#define SETTINGS_PATH_PREFIX  "/FirewallBuilder2_1"
-
 class FWBSettings : public QSettings {
 
  public:
 
     enum LabelColors { RED, ORANGE, YELLOW, GREEN, BLUE, PURPLE, GRAY };
+    enum IconSize{ SIZE25X25, SIZE16X16};
 
  private:
     QString getLabelColorStr(enum LabelColors c);
@@ -134,6 +136,58 @@ class FWBSettings : public QSettings {
 
     void    getPrinterOptions(QPrinter *printer,int &pageWidth,int &pageHeight);
     void    setPrinterOptions(QPrinter *printer,int pageWidth,int pageHeight);
+
+    enum IconSize getIconsInRulesSize();
+    void setIconsInRulesSize(enum IconSize size);
+
+    bool getShowIconsInRules();
+    void setShowIconsInRules(bool showIcons);
+
+    bool getShowDirectionText();
+    void setShowDirectionText(bool showText);
+
+    QFont getRulesFont();
+    void setRulesFont(const QFont &font);
+
+    QFont getTreeFont();
+    void setTreeFont(const QFont &font);
+
+    QFont getUiFont();
+    void setUiFont(const QFont &font);
+
+    bool getClipComment();
+    void setClipComment(bool);
+
+    bool getCheckUpdates();
+    void setCheckUpdates(bool);
+
+    QString getCheckUpdatesProxy();
+    void setCheckUpdatesProxy(const QString &proxy_line);
+
+    void getExpandedObjectIds(const QString &filename,
+                              const QString &lib,
+                              std::set<int> &ids);
+    void setExpandedObjectIds(const QString &filename,
+                              const QString &lib,
+                              const std::set<int> &ids);
+
+    int getVisibleRuleSetId(const QString &filename,
+                            const QString &lib);
+    void setVisibleRuleSet(const QString &filename,
+                           const QString &lib,
+                           libfwbuilder::FWObject *ruleset);
+
+    void getCollapsedRuleGroups(const QString &filename,
+                                const QString &firewall,
+                                const QString &ruleset,
+                                QStringList &collapsed_groups);
+    void setCollapsedRuleGroups(const QString &filename,
+                                const QString &firewall,
+                                const QString &ruleset,
+                                const QStringList &collapsed_groups);
+                                
+private:
+    QFont getFontByType(const char*type);
 };
 
 #endif
