@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: FindObjectWidget.cpp,v 1.9 2007/04/24 05:00:47 vkurland Exp $
+  $Id: FindObjectWidget.cpp 565 2008-10-04 14:34:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,23 +17,22 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
+#include "../../definitions.h"
 #include "global.h"
 #include "utils.h"
 #include "platforms.h"
-#include "definitions.h"
 
+#include <QLineEdit>
 #include "FindObjectWidget.h"
-#include "ObjectManipulator.h"
 #include "FWWindow.h"
 #include "FWObjectDropArea.h"
-#include "ObjectManipulator.h"
 #include "FWBTree.h"
 #include "FWBSettings.h"
 #include "ObjectTreeView.h"
@@ -51,7 +50,7 @@
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 
-#include <qwidgetstack.h>
+#include <QStackedWidget>
 #include <qradiobutton.h>
 #include <qcheckbox.h>
 #include <qcombobox.h>
@@ -69,47 +68,52 @@ using namespace libfwbuilder;
 
 #define MAX_SEARCH_ITEMS_COUNT 10
 
-FindObjectWidget::FindObjectWidget(QWidget*p, const char * n, WFlags f) : findObjectWidget_q(p,n,f) 
+FindObjectWidget::FindObjectWidget(QWidget*p, const char * n, Qt::WindowFlags f) : QWidget(p,f)
 {
+    m_widget = new Ui::findObjectWidget_q;
+    m_widget->setupUi(this);
+
+    setObjectName(n);
+
     replaceDisable();
-    srScope->setCurrentItem(st->readNumEntry("/FirewallBuilder2/Search/Scope"));
+    m_widget->srScope->setCurrentIndex(st->getInt(SETTINGS_PATH_PREFIX"/Search/Scope"));
 }
 void FindObjectWidget::findObject(FWObject *o)
 {
     if (fwbdebug) qDebug("FindObjectWidget::findObject");
-    show(); 
-    findDropArea->insertObject(o); 
+    show();
+    m_widget->findDropArea->insertObject(o);
 
 }
 
 
 void FindObjectWidget::enableAll()
 {
-    useRegexp->setEnabled (true);
-    findAttr->setEnabled  (true);
-    attribute->setEnabled (true);
+    m_widget->useRegexp->setEnabled (true);
+    m_widget->findAttr->setEnabled  (true);
+    m_widget->attribute->setEnabled (true);
 }
 
 void FindObjectWidget::disableAll()
 {
-    useRegexp->setEnabled (false);
-    findAttr->setEnabled  (false);
-    attribute->setEnabled (false);
+    m_widget->useRegexp->setEnabled (false);
+    m_widget->findAttr->setEnabled  (false);
+    m_widget->attribute->setEnabled (false);
 }
 
 void FindObjectWidget::objectInserted()
 {
-    FWObject *o=findDropArea->getObject();
+    FWObject *o=m_widget->findDropArea->getObject();
     if (o==NULL) return;
     disableAll();
-    
+
     QString n=QString::fromUtf8(o->getName().c_str());
-    
-    if (findAttr->count()>=MAX_SEARCH_ITEMS_COUNT)
-        findAttr->removeItem(MAX_SEARCH_ITEMS_COUNT-1);
-    
-    findAttr->setCurrentText (n);
-    
+
+    if (m_widget->findAttr->count()>=MAX_SEARCH_ITEMS_COUNT)
+        m_widget->findAttr->removeItem(MAX_SEARCH_ITEMS_COUNT-1);
+
+    m_widget->findAttr->lineEdit()->setText (n);
+
      reset();
 }
 
@@ -130,61 +134,66 @@ void FindObjectWidget::findAttrChanged(const QString &ns)
 
 void FindObjectWidget::find()
 {
-    if (findAttr->currentText().isEmpty() &&
-        findDropArea->isEmpty()) return;
+    if (m_widget->findAttr->currentText().isEmpty() &&
+        m_widget->findDropArea->isEmpty()) return;
 
-    if (findAttr->currentText() != findAttr->text(0))
+    if (m_widget->findAttr->currentText() != m_widget->findAttr->itemText(0))
     {
-        if (findAttr->count()>=MAX_SEARCH_ITEMS_COUNT)
-            findAttr->removeItem(MAX_SEARCH_ITEMS_COUNT-1);
+        if (m_widget->findAttr->count()>=MAX_SEARCH_ITEMS_COUNT)
+            m_widget->findAttr->removeItem(MAX_SEARCH_ITEMS_COUNT-1);
 
-        findAttr->insertItem( findAttr->currentText() , 0 );
+        m_widget->findAttr->insertItem( 0, m_widget->findAttr->lineEdit()->text() );
 
         if (fwbdebug)
-            qDebug("FindObjectWidget::find() : findAttr->text(0)=%s",
-                   findAttr->text(0).latin1());
+            qDebug("FindObjectWidget::find() : m_widget->findAttr->text(0)=%s",
+                   m_widget->findAttr->itemText(0).toLatin1().constData());
     }
     findNext();
 }
 
-bool FindObjectWidget::matchID(const QString &id)
+bool FindObjectWidget::matchID(int id)
 {
-    if (findDropArea->isEmpty()) return true;
-    QString s=QString::fromUtf8(findDropArea->getObject()->getId().c_str() );
-    
-    return s==id;
+    if (m_widget->findDropArea->isEmpty()) return true;
+    int s_id = m_widget->findDropArea->getObject()->getId();
+    return s_id==id;
 }
+
 bool FindObjectWidget::matchAttr(libfwbuilder::FWObject *obj)
 {
-    if (!findDropArea->isEmpty()) return true;
-    QString s=findAttr->currentText();
+    if (!m_widget->findDropArea->isEmpty()) return true;
+    QString s=m_widget->findAttr->currentText();
     if (s.isEmpty()) return true;
 
     bool res=false;
-    int  attrN = attribute->currentItem();
+    int  attrN = m_widget->attribute->currentIndex();
 
     switch (attrN) {
     case 0:   // Name
     {
         QString name=QString::fromUtf8( obj->getName().c_str() );
-        
-        if (useRegexp->isChecked()) res= ( name.find( QRegExp(s) )!=-1 );
+
+        if (m_widget->useRegexp->isChecked()) res= ( name.indexOf( QRegExp(s) )!=-1 );
         else                        res= ( name == s );
-        
+
         //res= ( name == s );
         break;
-        
+
     }
     case 1:   // Address
     {
         Address *a = Address::cast(obj);
         if (a!=NULL)
         {
-            QString addr = a->getAddress().toString().c_str();
-            
-            if (useRegexp->isChecked()) res= ( addr.find( QRegExp(s) )!=-1 );
-            else                        res= ( addr == s );
-            
+            const InetAddr *inet_addr = a->getAddressPtr();
+            if (inet_addr)
+            {
+                QString addr = inet_addr->toString().c_str();
+
+                if (m_widget->useRegexp->isChecked())
+                    res = ( addr.indexOf( QRegExp(s) )!=-1 );
+                else
+                    res= ( addr == s );
+            }
             //res= ( addr == s );
         }
         break;
@@ -192,25 +201,29 @@ bool FindObjectWidget::matchAttr(libfwbuilder::FWObject *obj)
     case 2:   // port
         if (TCPService::cast(obj)!=NULL || UDPService::cast(obj)!=NULL)
         {
-            if (useRegexp->isChecked()) 
+            if (m_widget->useRegexp->isChecked())
             {
                 QString port;
-                port.setNum(obj->getInt("src_range_start"));
-                res |= ( port.find( QRegExp(s) )!=-1 );
-                port.setNum(obj->getInt("src_range_end"));
-                res |= ( port.find( QRegExp(s) )!=-1 );
-                port.setNum(obj->getInt("dst_range_start"));
-                res |= ( port.find( QRegExp(s) )!=-1 );
-                port.setNum(obj->getInt("dst_range_end"));
-                res |= ( port.find( QRegExp(s) )!=-1 );
+                port.setNum(TCPUDPService::cast(obj)->getSrcRangeStart());
+                res |= ( port.indexOf( QRegExp(s) )!=-1 );
+                port.setNum(TCPUDPService::cast(obj)->getSrcRangeEnd());
+                res |= ( port.indexOf( QRegExp(s) )!=-1 );
+                port.setNum(TCPUDPService::cast(obj)->getDstRangeStart());
+                res |= ( port.indexOf( QRegExp(s) )!=-1 );
+                port.setNum(TCPUDPService::cast(obj)->getDstRangeEnd());
+                res |= ( port.indexOf( QRegExp(s) )!=-1 );
             } else
             {
                 bool conversion_status = false;
                 int port = s.toInt(&conversion_status);
-                res |= (conversion_status && (port == obj->getInt("src_range_start")));
-                res |= (conversion_status && (port == obj->getInt("src_range_end")));
-                res |= (conversion_status && (port == obj->getInt("dst_range_start")));
-                res |= (conversion_status && (port == obj->getInt("dst_range_end")));
+                res |= (conversion_status &&
+                        (port == TCPUDPService::cast(obj)->getSrcRangeStart()));
+                res |= (conversion_status &&
+                        (port == TCPUDPService::cast(obj)->getSrcRangeEnd()));
+                res |= (conversion_status &&
+                        (port == TCPUDPService::cast(obj)->getDstRangeStart()));
+                res |= (conversion_status &&
+                        (port == TCPUDPService::cast(obj)->getDstRangeEnd()));
             }
             break;
         }
@@ -218,11 +231,11 @@ bool FindObjectWidget::matchAttr(libfwbuilder::FWObject *obj)
     case 3:   // protocol num.
         if (IPService::cast(obj)!=NULL)
         {
-            if (useRegexp->isChecked()) 
+            if (m_widget->useRegexp->isChecked())
             {
                 QString proto;
                 proto.setNum(obj->getInt("protocol_num"));
-                res |= ( proto.find( QRegExp(s) )!=-1 );
+                res |= ( proto.indexOf( QRegExp(s) )!=-1 );
             } else
             {
                 bool conversion_status = false;
@@ -235,11 +248,11 @@ bool FindObjectWidget::matchAttr(libfwbuilder::FWObject *obj)
     case 4:   // icmp type
         if (ICMPService::cast(obj)!=NULL)
         {
-            if (useRegexp->isChecked()) 
+            if (m_widget->useRegexp->isChecked())
             {
                 QString icmptype;
                 icmptype.setNum(obj->getInt("type"));
-                res |= ( icmptype.find( QRegExp(s) )!=-1 );
+                res |= ( icmptype.indexOf( QRegExp(s) )!=-1 );
             } else
             {
                 bool conversion_status = false;
@@ -258,11 +271,11 @@ void FindObjectWidget::findNext()
 {
     if (fwbdebug) qDebug("FindObjectWidget::findNext");
     if (
-        findAttr->currentText().isEmpty() &&
-        findDropArea->isEmpty()) return;
+        m_widget->findAttr->currentText().isEmpty() &&
+        m_widget->findDropArea->isEmpty()) return;
 
-    if (findAttr->count()>MAX_SEARCH_ITEMS_COUNT)  
-        findAttr->removeItem(0);
+    if (m_widget->findAttr->count()>MAX_SEARCH_ITEMS_COUNT)
+        m_widget->findAttr->removeItem(0);
 
     FWObject *o=NULL;
 
@@ -276,31 +289,29 @@ loop:
 
         if( RuleElement::cast(o->getParent())!=NULL)
         {
-            if (srScope->currentItem()==3) // scope == selected firewalls
+            if (m_widget->srScope->currentIndex()==3) // scope == selected firewalls
             {
                 if ( !inSelectedFirewall(RuleElement::cast(o->getParent())) )
                 {
                     continue;
-                    
+
                 }
-            } else if (srScope->currentItem()==0) continue ; // scope == tree only
+            } else if (m_widget->srScope->currentIndex()==0) continue ; // scope == tree only
         } else
         {
 /* if not in rules, then in the tree. */
-            if (srScope->currentItem()>1) continue; // scope in (firewalls only , selected firewalls)
+            if (m_widget->srScope->currentIndex()>1) continue; // scope in (firewalls only , selected firewalls)
         }
 
         if (FWReference::cast(o)!=NULL)
         {
             FWReference *r=FWReference::cast(o);
-            if ( 
-                matchAttr( r->getPointer() ) &&
-                matchID( QString::fromUtf8(r->getPointer()->getId().c_str()) )) break;
+            if (matchAttr( r->getPointer() ) &&
+                matchID( r->getPointer()->getId() )) break;
         } else
         {
-            if (
-                matchAttr( o ) &&
-                matchID( QString::fromUtf8(o->getId().c_str()) )) break;
+            if (matchAttr( o ) &&
+                matchID( o->getId() )) break;
         }
     }
 
@@ -309,10 +320,10 @@ loop:
     if (treeSeeker==mw->db()->tree_end())
     {
         reset();
-        if (srScope->currentItem()==3) // scope ==selected firewalls
+        if (m_widget->srScope->currentIndex()==3) // scope ==selected firewalls
         {
             if ( QMessageBox::warning(
-                     this,"Firewall Builder", 
+                     this,"Firewall Builder",
                      tr("Search hit the end of the policy rules."),
                      tr("&Continue at top"), tr("&Stop"), QString::null, 0, 1 )==0 ) goto loop;
         }
@@ -320,7 +331,7 @@ loop:
         {
             if (fwbdebug) qDebug("widget that has focus: %p",mw->focusWidget());
             bool r= ( QMessageBox::warning(
-                     this,"Firewall Builder", 
+                     this,"Firewall Builder",
                      tr("Search hit the end of the object tree."),
                      tr("&Continue at top"), tr("&Stop"), QString::null, 0, 1 )==0 );
             if (fwbdebug) qDebug("widget that has focus: %p",mw->focusWidget());
@@ -333,34 +344,35 @@ loop:
 /* found object. Shift iterator so it does not return the same object
  * when user hits 'find next'
  */
-   
+
     ++treeSeeker;
-    
-    showObject(o);
-    
+
     if (fwbdebug)
     {
         qDebug("Found object: o=%p  id=%s  name=%s  type=%s",
-               o, o->getId().c_str(),o->getName().c_str(),o->getTypeName().c_str());
+               o,
+               FWObjectDatabase::getStringId(o->getId()).c_str(),
+               o->getName().c_str(),o->getTypeName().c_str());
     }
+    showObject(o);
 }
 
 bool FindObjectWidget::validateReplaceObject()
 {
-    if (findDropArea->isEmpty() || replaceDropArea->isEmpty()) 
+    if (m_widget->findDropArea->isEmpty() || m_widget->replaceDropArea->isEmpty())
     {
         QMessageBox::warning(
-              this,"Firewall Builder", 
+              this,"Firewall Builder",
               tr("Search or Replace object ind't specified."));
         return false;
     }
     FWObject *findObj, *replObj;
-    findObj=findDropArea->getObject();
-    replObj=replaceDropArea->getObject();
+    findObj=m_widget->findDropArea->getObject();
+    replObj=m_widget->replaceDropArea->getObject();
     if (findObj==replObj || findObj->getId() == replObj->getId())
     {
         QMessageBox::warning(
-              this,"Firewall Builder", 
+              this,"Firewall Builder",
               tr("Cannot replace object by itself."));
         return false;
     }
@@ -368,9 +380,9 @@ bool FindObjectWidget::validateReplaceObject()
             (Service::cast(findObj)!=NULL && Service::cast(replObj))))
     {
         QMessageBox::warning(
-              this,"Firewall Builder", 
+              this,"Firewall Builder",
               tr("Search and Replace objects are incompatible."));
-        
+
         return false;
     }
     return true;
@@ -378,19 +390,19 @@ bool FindObjectWidget::validateReplaceObject()
 void FindObjectWidget::replace()
 {
     if(!validateReplaceObject())  return;
-    
 
-    if (lastFound==NULL) 
+
+    if (lastFound==NULL)
     {
         find();
         return;
     }
-    
+
     QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
     FWObject *res=_replaceCurrent();
     mw->updateRuleSetView();
-    om->info();
-    if (res) 
+    mw->info();
+    if (res)
     {
         showObject(res);
     }
@@ -399,10 +411,10 @@ void FindObjectWidget::replace()
         // object isn't inserted
         qDebug("object isn't inserted");
     }
-    
+
     QApplication::restoreOverrideCursor();
 }
-    
+
 void FindObjectWidget::replaceAll()
 {
     if(!validateReplaceObject()) return;
@@ -410,7 +422,7 @@ void FindObjectWidget::replaceAll()
     FWObject *o=NULL;
     int count=0;
     bool f=true;
-    
+
     QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
     while (f)
     {
@@ -419,37 +431,35 @@ void FindObjectWidget::replaceAll()
             o = *treeSeeker;
             if( RuleElement::cast(o->getParent())!=NULL)
             {
-                if (srScope->currentItem()==3) // scope == selected firewalls
+                if (m_widget->srScope->currentIndex()==3) // scope == selected firewalls
                 {
                     if ( !inSelectedFirewall(RuleElement::cast(o->getParent())) )
                     {
                         continue;
-                        
+
                     }
-                } else if (srScope->currentItem()==0) continue ; // scope == tree only
+                } else if (m_widget->srScope->currentIndex()==0) continue ; // scope == tree only
             } else
             {
     /* if not in rules, then in the tree. */
-                if (srScope->currentItem()>1) continue; // scope in (firewalls only , selected firewalls)
+                if (m_widget->srScope->currentIndex()>1) continue; // scope in (firewalls only , selected firewalls)
             }
 
             if (FWReference::cast(o)!=NULL)
             {
                 FWReference *r=FWReference::cast(o);
-                if ( 
-                     matchAttr( r->getPointer() ) &&
-                     matchID( QString::fromUtf8(r->getPointer()->getId().c_str()) )) break;
+                if (matchAttr( r->getPointer() ) &&
+                    matchID( r->getPointer()->getId() )) break;
             } else
             {
-                if (
-                    matchAttr( o ) &&
-                    matchID( QString::fromUtf8(o->getId().c_str()) )) break;
+                if (matchAttr( o ) &&
+                    matchID( o->getId() )) break;
             }
         }
         if (treeSeeker==mw->db()->tree_end())
-        { 
+        {
             f=false;
-            
+
         } else
         {
             lastFound=o;
@@ -459,10 +469,10 @@ void FindObjectWidget::replaceAll()
         }
     }
     mw->updateRuleSetView();
-    om->info();
+    mw->info();
     QApplication::restoreOverrideCursor();
     QMessageBox::information(
-              this,"Firewall Builder", 
+              this,"Firewall Builder",
               tr("Replaced %1 objects.").arg(count));
 
 }
@@ -473,20 +483,20 @@ FWObject* FindObjectWidget::_replaceCurrent()
 
     if (p==NULL || o==NULL) return NULL;
     if (FWReference::cast(o)==NULL) return NULL;
-    
+
     p->removeRef(FWReference::cast(o)->getPointer());
     //chack for duplicates --------
-    
-    FWObject *ro=replaceDropArea->getObject();
+
+    FWObject *ro=m_widget->replaceDropArea->getObject();
     if (RuleElement::cast(p)==NULL || !RuleElement::cast(p)->isAny())
     {
 /* avoid duplicates */
-        string cp_id=ro->getId();
+        int cp_id = ro->getId();
         FWObject *oo;
         FWReference *ref;
-        
+
         list<FWObject*>::iterator j;
-        for(j=p->begin(); j!=p->end(); ++j)     
+        for(j=p->begin(); j!=p->end(); ++j)
         {
             oo=*j;
             if(cp_id==oo->getId()) return NULL;
@@ -495,12 +505,12 @@ FWObject* FindObjectWidget::_replaceCurrent()
                 cp_id==ref->getPointerId()) return NULL;
         }
     }
-    
+
     p->addRef(ro);
     FWObject *to;
     FWReference *ref;
     list<FWObject*>::iterator i;
-    string id=replaceDropArea->getObject()->getId();
+    int id = m_widget->replaceDropArea->getObject()->getId();
     for (i=p->begin();i!=p->end();++i)
     {
         to=*i;
@@ -520,61 +530,59 @@ bool FindObjectWidget::inSelectedFirewall( RuleElement* r)
     FWObject *f=r;
     while (f!=NULL && !Firewall::isA(f)) f=f->getParent();
     if (f==NULL) return false;
-    
+
     return selectedFirewall==(Firewall::cast(f));
 }
 
 void FindObjectWidget::replaceEnable()
 {
-    replaceButton->setEnabled   (true);
-    repNextButton->setEnabled   (true);
-    replaceAllButton->setEnabled(true);
-    
+    m_widget->replaceButton->setEnabled   (true);
+    m_widget->repNextButton->setEnabled   (true);
+    m_widget->replaceAllButton->setEnabled(true);
+
 }
 
 void FindObjectWidget::replaceDisable()
 {
-    replaceButton->setEnabled   (false);
-    repNextButton->setEnabled   (false);
-    replaceAllButton->setEnabled(false);
-    
+    m_widget->replaceButton->setEnabled   (false);
+    m_widget->repNextButton->setEnabled   (false);
+    m_widget->replaceAllButton->setEnabled(false);
+
 }
 
 void FindObjectWidget::showObject(FWObject* o)
 {
     if (fwbdebug) qDebug("FindObjectWidget::showObject");
-    
+
     FWReference* ref=FWReference::cast(o);
     if (ref!=NULL && RuleElement::cast(o->getParent())!=NULL)
     {
-        oe->close();
-        om->clearFocus();
+        mw->closeEditor();
+        mw->clearManipulatorFocus();
         mw->ensureObjectVisibleInRules( ref );
-        mw->selectRules();
         return;
     }
 
     mw->unselectRules();
 
-    if (Group::cast(o->getParent())!=NULL && 
-        !FWBTree::isSystem(o->getParent()))
+    if (Group::cast(o->getParent())!=NULL &&
+        !mw->isSystem(o->getParent()))
     {
-        om->openObject( o->getParent() );
-        om->editObject( o->getParent() );
-        oe->selectObject( (ref) ? ref->getPointer() : o );
-        //oe->setFocus();
+        mw->openObject( o->getParent() );
+        mw->editObject( o->getParent() );
+        mw->selectObjectInEditor( (ref) ? ref->getPointer() : o );
         return;
     }
 
-    oe->close();
-    om->openObject( o );
-    om->select();  // selects an item in the tree and assigns kbd focus to it
+    mw->closeEditor();
+    mw->openObject( o );
+    mw->select();  // selects an item in the tree and assigns kbd focus to it
 }
 
 void FindObjectWidget::init()
 {
-    findDropArea->deleteObject();
-    replaceDropArea->deleteObject();
+    m_widget->findDropArea->deleteObject();
+    m_widget->replaceDropArea->deleteObject();
     hidePanel();
 }
 
@@ -582,7 +590,7 @@ void FindObjectWidget::firewallOpened(Firewall *f)
 {
     if (f==NULL) return;
     selectedFirewall=f;
-    srScope->changeItem(tr("Policy of firewall '")+f->getName().c_str()+"'",3);
+    m_widget->srScope->setItemText( 3, tr("Policy of firewall '")+f->getName().c_str()+"'" );
 }
 
 void FindObjectWidget::findPrev()
@@ -597,6 +605,6 @@ void FindObjectWidget::replaceNext()
 }
 void FindObjectWidget::scopeChanged()
 {
-    st->writeEntry("/FirewallBuilder2/Search/Scope",srScope->currentItem ());
-    
+    st->setValue(SETTINGS_PATH_PREFIX"/Search/Scope",m_widget->srScope->currentIndex ());
+
 }
