@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IOSImporter.cpp,v 1.11 2007/08/06 07:07:22 vkurland Exp $
+  $Id: IOSImporter.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,13 +17,13 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils_no_qt.h"
 #include "platforms.h"
@@ -38,7 +38,7 @@
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/Address.h"
-#include "fwbuilder/IPAddress.h"
+#include "fwbuilder/InetAddr.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
@@ -193,17 +193,14 @@ FWObject* IOSImporter::createAddress(const std::string &addr,
     // invert netmask (this is IOS)
     try
     {
-        IPAddress orig_nm(netmask);
-        long nm = orig_nm.to32BitInt();
-        struct in_addr na;
-        na.s_addr = ~nm;
-        correct_nm = IPAddress(&na).toString();
+        InetAddr orig_nm(netmask);
+        correct_nm = (~orig_nm).toString();
         return Importer::createAddress(addr, correct_nm);
     } catch (FWException &ex)
     {
         markCurrentRuleBad(
             std::string("Error converting netmask '") + netmask + "' (address " + addr + ")");
-        return Importer::createAddress(addr, "255.255.255.255");
+        return Importer::createAddress(addr, InetAddr::getAllOnes().toString());
     }
 
 }
@@ -263,7 +260,7 @@ int IOSImporter::convertPort(const std::string &port_str,
             str1 >> port;
         } catch (const std::exception &ex) {
             // could not convert port_spec to an integer
-            markCurrentRuleBad(std::string("Port spec '") + port_str + 
+            markCurrentRuleBad(std::string("Port spec '") + port_str +
                                "' unknown. Error " + ex.what());
         }
     }
@@ -282,7 +279,7 @@ std::pair<int,int> IOSImporter::convertPortSpec(const std::string &port_op,
 
     if (fwbdebug)
         qDebug(QString("Convert TCP/UDP port spec: port_op=%1 port_spec=%2").
-               arg(port_op).arg(port_spec));
+               arg(port_op.c_str()).arg(port_spec.c_str()).toAscii().constData());
 
     if (portop=="" && portspec=="") return std::pair<int,int>(0, 0);
 
@@ -299,7 +296,7 @@ std::pair<int,int> IOSImporter::convertPortSpec(const std::string &port_op,
     range_start = convertPort(s1, port_map);
     range_end   = convertPort(s2, port_map);
 
-    if (portop=="lt")  range_start = 0; 
+    if (portop=="lt")  range_start = 0;
     if (portop=="gt")  range_end = 65535;
     if (portop=="eq")
     {
@@ -401,7 +398,7 @@ Firewall* IOSImporter::finalize()
             {
                 qDebug("Setting interface and direction for all rules");
                 qDebug(QString("all_rulesets.size()=%1").
-                       arg(all_rulesets.size()));
+                       arg(all_rulesets.size()).toAscii().constData());
             }
 
             std::map<const std::string,UnidirectionalRuleSet*>::iterator i;
@@ -412,11 +409,11 @@ Firewall* IOSImporter::finalize()
                 if (fwbdebug)
                 {
                     qDebug(QString("  irs->name=%1").
-                           arg(irs->name.c_str()));
+                           arg(irs->name.c_str()).toAscii().constData());
                     qDebug(QString("  irs->intf_dir.size()=%1").
-                           arg(irs->intf_dir.size()));
+                           arg(irs->intf_dir.size()).toAscii().constData());
                     qDebug(QString("  irs->ruleset->size()=%1").
-                           arg(irs->ruleset->size()));
+                           arg(irs->ruleset->size()).toAscii().constData());
                 }
 
                 // optimization: If we have several interfaces for
@@ -452,7 +449,7 @@ Firewall* IOSImporter::finalize()
                     {
                         og = createGroupOfInterfaces(irs->name, all_in);
                         std::for_each(irs->ruleset->begin(),
-                                      irs->ruleset->end(), 
+                                      irs->ruleset->end(),
                                       merge_rule(irs->name,
                                                  og,
                                                  PolicyRule::Inbound,
@@ -464,7 +461,7 @@ Firewall* IOSImporter::finalize()
                     {
                         og = createGroupOfInterfaces(irs->name, all_out);
                         std::for_each(irs->ruleset->begin(),
-                                      irs->ruleset->end(), 
+                                      irs->ruleset->end(),
                                       merge_rule(irs->name,
                                                  og,
                                                  PolicyRule::Outbound,
@@ -476,7 +473,7 @@ Firewall* IOSImporter::finalize()
                     {
                         og = createGroupOfInterfaces(irs->name, all_both);
                         std::for_each(irs->ruleset->begin(),
-                                      irs->ruleset->end(), 
+                                      irs->ruleset->end(),
                                       merge_rule(irs->name,
                                                  og,
                                                  PolicyRule::Both,
@@ -485,7 +482,7 @@ Firewall* IOSImporter::finalize()
                     }
 
                 }
-                else 
+                else
                 {
                     std::map<std::string,std::string>::iterator j;
                     for (j=irs->intf_dir.begin(); j!=irs->intf_dir.end(); ++j)
@@ -501,9 +498,9 @@ Firewall* IOSImporter::finalize()
                         {
                             if (fwbdebug)
                                 qDebug(QString("    interface=%1").
-                                       arg(intf->getName().c_str()));
+                                       arg(intf->getName().c_str()).toAscii().constData());
                             std::for_each(irs->ruleset->begin(),
-                                          irs->ruleset->end(), 
+                                          irs->ruleset->end(),
                                           merge_rule(irs->name,
                                                      intf,
                                                      direction,
