@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPServiceDialog.h,v 1.10 2006/05/13 06:53:05 vkurland Exp $
+  $Id: IPServiceDialog.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,22 +27,30 @@
 #ifndef __IPSERVICEDIALOG_H_
 #define __IPSERVICEDIALOG_H_
 
-#include "config.h"
+#include "../../config.h"
 
-#include <ipservicedialog_q.h>
+#include <ui_ipservicedialog_q.h>
+#include <QWidget>
 
 #include "fwbuilder/FWObject.h"
 
 
-class IPServiceDialog : public IPServiceDialog_q
+class ProjectPanel;
+class IPServiceDialog : public QWidget
 {
     Q_OBJECT
 
     libfwbuilder::FWObject *obj;
     bool init;
+    Ui::IPServiceDialog_q *m_dialog;
 
- public:
-    IPServiceDialog(QWidget *parent) : IPServiceDialog_q(parent) { obj=NULL; }
+    ProjectPanel *m_project;
+
+    void setCodeLabel();
+
+public:
+    IPServiceDialog(ProjectPanel *project, QWidget *parent);
+    ~IPServiceDialog();
     
 public slots:
     virtual void changed();
