@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPTImporter.cpp,v 1.11 2008/05/05 16:09:38 vkurland Exp $
+  $Id: IPTImporter.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,13 +17,13 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils_no_qt.h"
 #include "platforms.h"
@@ -46,7 +46,7 @@
 #include "fwbuilder/Network.h"
 #include "fwbuilder/Address.h"
 #include "fwbuilder/AddressRange.h"
-#include "fwbuilder/IPAddress.h"
+#include "fwbuilder/InetAddr.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
@@ -257,7 +257,7 @@ int IPTImporter::convertPort(const std::string &port_spec,
         str1 >> port;
     } catch (const std::exception &ex) {
         // could not convert port_spec to an integer
-        markCurrentRuleBad(std::string("Port spec '") + port_spec + 
+        markCurrentRuleBad(std::string("Port spec '") + port_spec +
                            "' unknown. Error " + ex.what());
     }
     return port;
@@ -269,13 +269,13 @@ FWObject* IPTImporter::createTCPUDPService(str_tuple &src_range,
 {
     if (fwbdebug)
     {
-        qDebug(QString("Creating %1 service").arg(proto.c_str()));
+        qDebug(QString("Creating %1 service").arg(proto.c_str()).toAscii().constData());
         qDebug(QString("src range: %1 - %2")
                .arg(src_range.first.c_str())
-               .arg(src_range.second.c_str()));
+               .arg(src_range.second.c_str()).toAscii().constData());
         qDebug(QString("dst range: %1 - %2")
                .arg(dst_range.first.c_str())
-               .arg(dst_range.second.c_str()));
+               .arg(dst_range.second.c_str()).toAscii().constData());
     }
 
 //     std::string name = proto + " "
@@ -348,7 +348,7 @@ FWObject* IPTImporter::createTCPUDPService(const std::string &proto)
         std::string name = proto + " group " + s.str();
 
         if (fwbdebug)
-            qDebug(QString("Group of %1 services with name '%2', sig '%3'").arg(proto.c_str()).arg(name.c_str()).arg(sig.c_str()));
+            qDebug(QString("Group of %1 services with name '%2', sig '%3'").arg(proto.c_str()).arg(name.c_str()).arg(sig.c_str()).toAscii().constData());
 
         ServiceGroup *sg = ServiceGroup::cast(createObject(ServiceGroup::TYPENAME, name));
         for (FWObject::iterator j=olist.begin(); j!=olist.end(); ++j)
@@ -383,7 +383,7 @@ void IPTImporter::addSrv()
     RuleElementSrv* srv = rule->getSrv();
     assert(srv!=NULL);
 
-    if (match_mark.empty()) 
+    if (match_mark.empty())
     {
         Importer::addSrv();
         return;
@@ -391,7 +391,7 @@ void IPTImporter::addSrv()
 
     srv->addRef( getTagService(match_mark) );
 }
-    
+
 void IPTImporter::pushRule()
 {
     assert(current_ruleset!=NULL);
@@ -545,10 +545,10 @@ void IPTImporter::pushPolicyRule()
         ropt->setStr("branch_name", branch_ruleset_name);
         getFirewallObject()->remove(rs->ruleset, false);
         branch_rulesets[branch_ruleset_name] = rs;
-        
+
         if (duplicate_branch)
             markCurrentRuleBad(
-                std::string("Rule passes control to branch ") + target + 
+                std::string("Rule passes control to branch ") + target +
                 std::string(
                     " which \n"
                     "is already used by some rule prior to this one. \n"
@@ -616,7 +616,7 @@ void IPTImporter::pushPolicyRule()
             << "\n";
     }
 
-    if (target=="CONNMARK" && 
+    if (target=="CONNMARK" &&
         last_mark_rule != NULL &&
         !action_params["connmark_save_mark"].empty())
     {
@@ -630,7 +630,7 @@ void IPTImporter::pushPolicyRule()
             << "\n";
     }
 
-    if (target=="CONNMARK" && 
+    if (target=="CONNMARK" &&
         !action_params["connmark_restore_mark"].empty())
     {
         // this rule is added automatically in
@@ -706,7 +706,7 @@ void IPTImporter::pushPolicyRule()
         }
 
         current_rule->setComment(rule_comment);
-      
+
     }
 
 //     *Importer::logger << "Rule: " << rule->getActionAsString() << " "
@@ -738,9 +738,9 @@ void IPTImporter::pushNATRule()
     addODst();
     addOSrv();
 
-    if (src_nm.empty()) src_nm = "255.255.255.255";
-    if (dst_nm.empty()) dst_nm = "255.255.255.255";
-    if (nat_nm.empty()) nat_nm = "255.255.255.255";
+    if (src_nm.empty()) src_nm = InetAddr::getAllOnes().toString();
+    if (dst_nm.empty()) dst_nm = InetAddr::getAllOnes().toString();
+    if (nat_nm.empty()) nat_nm = InetAddr::getAllOnes().toString();
 
     if (target=="ACCEPT")
     {
