@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPTImporterRun.cpp,v 1.2 2007/08/06 07:07:22 vkurland Exp $
+  $Id: IPTImporterRun.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,13 +17,13 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 
 #include "IPTImporter.h"
@@ -37,7 +37,7 @@
 #ifdef HAVE_ANTLR_RUNTIME
 #include <antlr/ANTLRException.hpp>
 
-// parser and lexer for files produced by iptables-save 
+// parser and lexer for files produced by iptables-save
 #include "../parsers/IPTCfgLexer.hpp"
 #include "../parsers/IPTCfgParser.hpp"
 
