@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Importer.cpp,v 1.17 2007/08/06 07:07:22 vkurland Exp $
+  $Id: Importer.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,13 +17,13 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils_no_qt.h"
 #include "platforms.h"
@@ -52,6 +52,8 @@
 #include "fwbuilder/NAT.h"
 #include "fwbuilder/RuleElement.h"
 
+#include "FWWindow.h"
+#include "ProjectPanel.h"
 #include "FWBTree.h"
 
 using namespace libfwbuilder;
@@ -60,8 +62,8 @@ FWObject* Importer::createObject(const std::string &objType,
                                  const std::string &objName)
 {
     assert(library!=NULL);
-    FWObject *slot = FWBTree::getStandardSlotForObject(library,
-                                                       objType.c_str());
+    FWBTree tree ;
+    FWObject *slot = tree.getStandardSlotForObject(library,objType.c_str());
     return createObject(slot, objType, objName);
 }
 
@@ -73,7 +75,7 @@ FWObject* Importer::createObject(FWObject *parent,
     FWObject*  o = library->getRoot()->create(objType);
     if (parent != NULL)
     {
-        parent->add(o);    
+        parent->add(o);
     }
     o->setName(objName);
     return o;
@@ -124,22 +126,22 @@ Importer::Importer(FWObject *_lib,
     current_ruleset = NULL;
     current_rule = NULL;
 
-    tcp_flag_names[libfwbuilder::TCPService::URG]="u"; 
-    tcp_flag_names[libfwbuilder::TCPService::ACK]="a"; 
-    tcp_flag_names[libfwbuilder::TCPService::PSH]="p"; 
-    tcp_flag_names[libfwbuilder::TCPService::RST]="r"; 
-    tcp_flag_names[libfwbuilder::TCPService::SYN]="s"; 
-    tcp_flag_names[libfwbuilder::TCPService::FIN]="f"; 
+    tcp_flag_names[libfwbuilder::TCPService::URG]="u";
+    tcp_flag_names[libfwbuilder::TCPService::ACK]="a";
+    tcp_flag_names[libfwbuilder::TCPService::PSH]="p";
+    tcp_flag_names[libfwbuilder::TCPService::RST]="r";
+    tcp_flag_names[libfwbuilder::TCPService::SYN]="s";
+    tcp_flag_names[libfwbuilder::TCPService::FIN]="f";
     tcp_flag_names[98]="N";   // NONE
     tcp_flag_names[99]="A";   // ALL
-    
+
 }
 
 void Importer::run()
 {
     // create and run parsers in derived classes
 }
-    
+
 
 Importer::~Importer()
 {
@@ -244,8 +246,8 @@ void Importer::addInterfaceAddress(const std::string &a,
                                       IPv4::TYPENAME,
                                       aname);
         current_interface->setUnnumbered(false);
-        IPv4::cast(nobj)->setAddress( a );
-        IPv4::cast(nobj)->setNetmask( nm );
+        IPv4::cast(nobj)->setAddress( InetAddr(a) );
+        IPv4::cast(nobj)->setNetmask( InetAddr(nm) );
 
         *logger << "Interface address: " << a << "/" << nm << "\n";
     }
@@ -393,17 +395,21 @@ void Importer::pushRule()
 
 FWObject* Importer::makeSrcObj()
 {
-    if ( (src_a=="" && src_nm=="") || (src_a=="0.0.0.0" && src_nm=="0.0.0.0"))
+    if ( (src_a=="" && src_nm=="") || 
+         (src_a==InetAddr::getAny().toString() &&
+          src_nm==InetAddr::getAny().toString()))
         return NULL;  // this is 'any'
-    if (src_nm=="") src_nm="255.255.255.255";
+    if (src_nm=="") src_nm=InetAddr::getAllOnes().toString();
     return createAddress(src_a, src_nm);
 }
 
 FWObject* Importer::makeDstObj()
 {
-    if ( (dst_a=="" && dst_nm=="") || (dst_a=="0.0.0.0" && dst_nm=="0.0.0.0"))
+    if ( (dst_a=="" && dst_nm=="") || 
+         (dst_a==InetAddr::getAny().toString() &&
+          dst_nm==InetAddr::getAny().toString()))
         return NULL;  // this is 'any'
-    if (dst_nm=="") dst_nm="255.255.255.255";
+    if (dst_nm=="") dst_nm=InetAddr::getAllOnes().toString();
     return createAddress(dst_a, dst_nm);
 }
 
@@ -448,7 +454,7 @@ void Importer::addSrv()
     FWObject *s = makeSrvObj();
     if (s) srv->addRef( s );
 }
-    
+
 void Importer::addOSrc()
 {
     NATRule *rule = NATRule::cast(current_rule);
@@ -475,7 +481,7 @@ void Importer::addOSrv()
     FWObject *s= makeSrvObj();
     if (s) srv->addRef( s );
 }
-    
+
 Firewall* Importer::finalize()
 {
     return fw;
@@ -499,7 +505,7 @@ FWObject* Importer::getIPService(int proto)
 
     if ( proto==0 && !fragments)
         return NULL;  // any
-    
+
     IPService *s = IPService::cast(createObject(IPService::TYPENAME, nstr.str()));
     s->setInt("protocol_num", proto);
     s->setBool("fragm", fragments);
@@ -579,10 +585,11 @@ FWObject* Importer::getTCPService(int srs, int sre,
 
     TCPService* s = TCPService::cast(
         createObject(TCPService::TYPENAME, nstr.str()));
-    s->setInt("src_range_start", srs);
-    s->setInt("src_range_end",   sre);
-    s->setInt("dst_range_start", drs);
-    s->setInt("dst_range_end",   dre);
+    s->setSrcRangeStart(srs);
+    s->setSrcRangeEnd(sre);
+    s->setDstRangeStart(drs);
+    s->setDstRangeEnd(dre);
+
 
     if (!flags_mask.empty() && !flags_comp.empty())
     {
@@ -640,14 +647,14 @@ FWObject* Importer::getUDPService(int srs, int sre, int drs, int dre)
          << srs << "-" << sre << ":" << drs << "-" << dre;
 
     std::ostringstream nstr;
-    nstr << "udp " 
+    nstr << "udp "
          << srs << "-" << sre << ":" << drs << "-" << dre;
 
     UDPService* s = UDPService::cast(createObject(UDPService::TYPENAME, nstr.str()));
-    s->setInt("src_range_start", srs);
-    s->setInt("src_range_end",   sre);
-    s->setInt("dst_range_start", drs);
-    s->setInt("dst_range_end",   dre);
+    s->setSrcRangeStart(srs);
+    s->setSrcRangeEnd(sre);
+    s->setDstRangeStart(drs);
+    s->setDstRangeEnd(dre);
 
     s->setComment(cstr.str());
     all_objects[sstr.str()] = s;
@@ -691,7 +698,7 @@ FWObject* Importer::createICMPService()
             s1 >> type;
         } catch (std::exception& e)
         {
-            // could not convert 
+            // could not convert
             type = -1;
             markCurrentRuleBad(std::string("ICMP type '") + icmp_type + "' unknown");
         }
@@ -705,7 +712,7 @@ FWObject* Importer::createICMPService()
             s2 >> code;
         } catch (std::exception& e)
         {
-            // could not convert 
+            // could not convert
             code = -1;
             markCurrentRuleBad(std::string("ICMP code '") + icmp_code + "' unknown");
         }
@@ -792,7 +799,7 @@ FWObject* Importer::createGroupOfInterfaces(
     std::for_each(interfaces.begin(), interfaces.end(), join(&sig, "_"));
 
     if (fwbdebug)
-        qDebug(QString("Interface group with name '%1', sig '%2'").arg(name).arg(sig));
+        qDebug(QString("Interface group with name '%1', sig '%2'").arg(name.c_str()).arg(sig.c_str()).toAscii().constData());
 
     if (all_objects.count(sig)!=0) return all_objects[sig];
 
@@ -814,13 +821,13 @@ FWObject* Importer::createAddress(const std::string &addr,
     std::string sig = std::string("addr-") + addr + "/" + netmask;
     if (all_objects.count(sig)!=0) return all_objects[sig];
 
-    if ( netmask == "255.255.255.255" )
+    if ( netmask == InetAddr::getAllOnes().toString() )
     {
         Address *a;
         std::string name = std::string("h-") + addr;
         a = Address::cast(createObject(IPv4::TYPENAME, name));
-        a->setAddress( addr );
-        a->setNetmask( "255.255.255.255" );
+        a->setAddress(InetAddr(addr));
+        a->setNetmask(InetAddr(InetAddr::getAllOnes()));
         a->setComment(comment);
         all_objects[sig] = a;
         *logger << "Address object: " << name << "\n";
@@ -832,7 +839,7 @@ FWObject* Importer::createAddress(const std::string &addr,
         net = Network::cast(createObject(Network::TYPENAME, name));
         try
         {
-            net->setAddress( addr );
+            net->setAddress( InetAddr(addr) );
         } catch (FWException &ex)
         {
             markCurrentRuleBad(
@@ -841,7 +848,7 @@ FWObject* Importer::createAddress(const std::string &addr,
 
         try
         {
-            net->setNetmask( netmask );
+            net->setNetmask( InetAddr(netmask) );
         } catch (FWException &ex)
         {
             if (netmask.find('.')!=std::string::npos)
@@ -858,9 +865,9 @@ FWObject* Importer::createAddress(const std::string &addr,
                 str.exceptions(std::ios::failbit);
                 int nm_len;
                 try
-                { 
+                {
                     str >> nm_len;
-                    net->setNetmask( Netmask(nm_len) );
+                    net->setNetmask( InetAddr(nm_len) );
                 } catch (std::exception& e)
                 {
                     // could not convert netmask as simple integer
@@ -892,7 +899,7 @@ FWObject* Importer::createAddressRange(const std::string &addr1,
 
     try
     {
-        ar->setRangeStart( IPAddress(addr1) );
+        ar->setRangeStart( InetAddr(addr1) );
     } catch (FWException &ex)
     {
         markCurrentRuleBad(
@@ -901,7 +908,7 @@ FWObject* Importer::createAddressRange(const std::string &addr1,
 
     try
     {
-        ar->setRangeEnd( IPAddress(addr2) );
+        ar->setRangeEnd( InetAddr(addr2) );
     } catch (FWException &ex)
     {
         markCurrentRuleBad(
@@ -923,9 +930,9 @@ void Importer::markCurrentRuleBad(const std::string &comment)
     FWOptions  *ropt = current_rule->getOptionsObject();
     assert(ropt!=NULL);
 
-    QString color = getBadRuleColor();
+    QString color = getBadRuleColor().c_str();
 
-    ropt->setStr("color", color.latin1());
+    ropt->setStr("color", color.toLatin1().constData());
 
     if (!rule_comment.empty()) rule_comment += "\n";
     rule_comment += comment;
@@ -936,4 +943,4 @@ void Importer::markCurrentRuleBad(const std::string &comment)
     error_counter++;
 }
 
-    
+
