@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: InterfaceDialog.cpp,v 1.30 2007/05/10 01:35:52 vkurland Exp $
+  $Id: InterfaceDialog.cpp 398 2008-07-21 01:47:01Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,18 +17,18 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
 #include "InterfaceDialog.h"
-#include "ObjectManipulator.h"
+#include "ProjectPanel.h"
 #include "FWWindow.h"
 
 #include "fwbuilder/Library.h"
@@ -48,17 +48,20 @@
 #include <qcombobox.h>
 #include <qmessagebox.h>
 #include <qcheckbox.h>
-#include <qbuttongroup.h>
+#include <qgroupbox.h>
 #include <qradiobutton.h>
 #include <qpushbutton.h>
 #include <qlabel.h>
+#include "FWBSettings.h"
 
 using namespace std;
 using namespace libfwbuilder;
 
-InterfaceDialog::InterfaceDialog(QWidget *parent) :
-    InterfaceDialog_q(parent)
+InterfaceDialog::InterfaceDialog(ProjectPanel *project, QWidget *parent) :
+        QWidget(parent), m_project(project)
 {
+    m_dialog = new Ui::InterfaceDialog_q;
+    m_dialog->setupUi(this);
 /*
     seclevel->hide();    seclevelLabel->hide();
     netzone->hide();     netzoneLabel->hide();
@@ -66,45 +69,83 @@ InterfaceDialog::InterfaceDialog(QWidget *parent) :
     obj=NULL;
 }
 
+InterfaceDialog::~InterfaceDialog()
+{
+    delete m_dialog;
+}
+
 void InterfaceDialog::loadFWObject(FWObject *o)
 {
     obj=o;
     Interface *s = dynamic_cast<Interface*>(obj);
     assert(s!=NULL);
 
-    init=true;
+    init = true;
 
-    fillLibraries(libs,obj);
+    m_dialog->obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
+    m_dialog->label->setText( QString::fromUtf8(s->getLabel().c_str()) );
 
-    obj_name->setText( QString::fromUtf8(s->getName().c_str()) );
-    label->setText( QString::fromUtf8(s->getLabel().c_str()) );
-
-    regular->setChecked( ! s->isDyn() &&
+    m_dialog->regular->setChecked( ! s->isDyn() &&
                          ! s->isUnnumbered() &&
                          ! s->isBridgePort() );
-    dynamic->setChecked( s->isDyn() );
-    unnumbered->setChecked( s->isUnnumbered() );
-    bridgeport->setChecked( s->isBridgePort() );
+    m_dialog->dynamic->setChecked( s->isDyn() );
+    m_dialog->unnumbered->setChecked( s->isUnnumbered() );
+    m_dialog->bridgeport->setChecked( s->isBridgePort() );
 
-    management->setChecked( s->isManagement() );
+    m_dialog->management->setChecked( s->isManagement() );
+
+    m_dialog->comment->setText( QString::fromUtf8(s->getComment().c_str()) );
+
+    m_dialog->obj_name->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->obj_name);
+
+    m_dialog->comment->setReadOnly(o->isReadOnly());
+    setDisabledPalette(m_dialog->comment);
+
+    m_dialog->label->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->label);
+
+    m_dialog->regular->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->regular);
+
+    m_dialog->dynamic->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->dynamic);
+
+    m_dialog->unnumbered->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->unnumbered);
+
+    m_dialog->bridgeport->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->bridgeport);
+
+    m_dialog->management->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->management);
+
+    m_dialog->unprotected->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->unprotected);
+
+    m_dialog->ext->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->ext);
+
+    m_dialog->seclevel->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->seclevel);
+
+    m_dialog->netzone->setEnabled(!o->isReadOnly());
+    setDisabledPalette(m_dialog->netzone);
 
-    comment->setText( QString::fromUtf8(s->getComment().c_str()) );
 
-/* interface should always belong to the host or firewall so we can't
- * move them from library to library */
-    libs->setEnabled( false );
 
     FWObject *f=obj->getParent();
 
 /* if parent is a host, hide checkbox 'external', security level and netzone */
     if (Host::isA( f ))
     {
-        ext->setEnabled( false );
-        ext->hide();
-        seclevel->hide();
-        seclevelLabel->hide();
-        netzone->hide();
-        netzoneLabel->hide();
+        m_dialog->ext->setEnabled(false);
+        m_dialog->management->setEnabled(false);
+        m_dialog->unprotected->setEnabled(false);
+        m_dialog->seclevel->setEnabled(false);
+        m_dialog->seclevelLabel->setEnabled(false);
+        m_dialog->netzone->setEnabled(false);
+        m_dialog->netzoneLabel->setEnabled(false);
     }
 
     bool supports_security_levels = false;
@@ -125,31 +166,31 @@ void InterfaceDialog::loadFWObject(FWObject *o)
     {
         if (supports_security_levels)
         {
-            seclevel->show();
-            seclevelLabel->show();
-            ext->hide();
-            seclevel->setValue( obj->getInt("security_level") );
+            m_dialog->seclevel->setEnabled(true);
+            m_dialog->seclevelLabel->setEnabled(true);
+            m_dialog->ext->setEnabled(false);
+            m_dialog->seclevel->setValue( obj->getInt("security_level") );
         } else
         {
-            seclevel->hide();
-            seclevelLabel->hide();
-            ext->show();
-            ext->setChecked( obj->getInt("security_level")==0 );
+            m_dialog->seclevel->setEnabled(false);
+            m_dialog->seclevelLabel->setEnabled(false);
+            m_dialog->ext->setEnabled(true);
+            m_dialog->ext->setChecked( obj->getInt("security_level")==0 );
         }
 
         if (supports_unprotected)
         {
-            unprotected->show();
-            unprotected->setChecked( obj->getBool("unprotected") );
+            m_dialog->unprotected->setEnabled(true);
+            m_dialog->unprotected->setChecked( obj->getBool("unprotected") );
         } else
         {
-            unprotected->hide();
+            m_dialog->unprotected->setEnabled(false);
         }
 
         if (supports_network_zones)
         {
-            netzone->show();
-            netzoneLabel->show();
+            m_dialog->netzone->setEnabled(true);
+            m_dialog->netzoneLabel->setEnabled(true);
 
             netzoneObjectIDs.clear();
             netzoneObjectNos.clear();
@@ -158,8 +199,8 @@ void InterfaceDialog::loadFWObject(FWObject *o)
 
             int n=0;
 
-            netzoneObjectIDs["sysid0"]=n;
-            netzoneObjectNos[n]="sysid0";
+            netzoneObjectIDs[FWObjectDatabase::ANY_ADDRESS_ID] = n;
+            netzoneObjectNos[n] = FWObjectDatabase::ANY_ADDRESS_ID;
             netzoneObjectNames.push_back(" Any ");
 
             ++n;
@@ -172,7 +213,7 @@ void InterfaceDialog::loadFWObject(FWObject *o)
                 FWObject *library= *l;
                 FWObject *o1,*o2;
 
-                if ( library->getId()==DELETED_LIB ) continue;
+                if ( library->getId()==FWObjectDatabase::DELETED_OBJECTS_ID ) continue;
 
                 o1=library->findObjectByName(ObjectGroup::TYPENAME,"Objects");
                 assert(o1!=NULL);
@@ -180,15 +221,15 @@ void InterfaceDialog::loadFWObject(FWObject *o)
                 if (o2==NULL)
                 {
                     if (fwbdebug)
-                        qDebug("InterfaceDialog::loadFWObject  missing Groups group in %s",o1->getId().c_str());
+                        qDebug("InterfaceDialog::loadFWObject  missing Groups group in %s", FWObjectDatabase::getStringId(o1->getId()).c_str());
                     continue;
                 }
 //                assert(o2!=NULL);
 
                 for (FWObject::iterator i=o2->begin(); i!=o2->end(); ++i)
                 {
-                    netzoneObjectIDs[ (*i)->getId().c_str() ]=n;
-                    netzoneObjectNos[n]= (*i)->getId().c_str();
+                    netzoneObjectIDs[(*i)->getId()] = n;
+                    netzoneObjectNos[n] =(*i)->getId();
                     netzoneObjectNames.push_back(
                         tr("Group: ")+ (*i)->getName().c_str() );
                     ++n;
@@ -198,80 +239,41 @@ void InterfaceDialog::loadFWObject(FWObject *o)
                 if (o2==NULL)
                 {
                     if (fwbdebug)
-                        qDebug("InterfaceDialog::loadFWObject  missing Networks group in %s",o1->getId().c_str());
+                        qDebug("InterfaceDialog::loadFWObject  missing Networks group in %s",
+                               FWObjectDatabase::getStringId(o1->getId()).c_str());
                     continue;
                 }
 //                assert(o2!=NULL);
 
                 for (FWObject::iterator i1=o2->begin(); i1!=o2->end(); ++i1)
                 {
-                    netzoneObjectIDs[ (*i1)->getId().c_str() ]=n;
-                    netzoneObjectNos[n]= (*i1)->getId().c_str();
+                    netzoneObjectIDs[(*i1)->getId()] = n;
+                    netzoneObjectNos[n] = (*i1)->getId();
                     netzoneObjectNames.push_back(
                         tr("Network: ")+ (*i1)->getName().c_str() );
                     ++n;
                 }
             }
 
-            netzone->clear();            
-            netzone->insertStringList( netzoneObjectNames );
+            m_dialog->netzone->clear();
+            m_dialog->netzone->addItems( netzoneObjectNames );
 
-            QString id=obj->getStr("network_zone").c_str();
-            if (id=="") id="sysid0";  // any network
-            netzone->setCurrentItem( netzoneObjectIDs[id] );
+            int id = FWObjectDatabase::getIntId(obj->getStr("network_zone"));
+            if (id==-1) id = FWObjectDatabase::ANY_ADDRESS_ID;  // any network
+            m_dialog->netzone->setCurrentIndex( netzoneObjectIDs[id] );
         }
         else
         {
-            netzone->hide();
-            netzoneLabel->hide();
+            m_dialog->netzone->setEnabled(false);
+            m_dialog->netzoneLabel->setEnabled(false);
         }
     }
 
     //apply->setEnabled( false );
 
-    obj_name->setEnabled(!o->isReadOnly());
-    setDisabledPalette(obj_name);
-
-    libs->setEnabled(!o->isReadOnly());
-    setDisabledPalette(libs);
-
-    comment->setReadOnly(o->isReadOnly());
-    setDisabledPalette(comment);
-
-    label->setEnabled(!o->isReadOnly());
-    setDisabledPalette(label);
-
-    regular->setEnabled(!o->isReadOnly());
-    setDisabledPalette(regular);
-
-    dynamic->setEnabled(!o->isReadOnly());
-    setDisabledPalette(dynamic);
-
-    unnumbered->setEnabled(!o->isReadOnly());
-    setDisabledPalette(unnumbered);
-
-    bridgeport->setEnabled(!o->isReadOnly());
-    setDisabledPalette(bridgeport);
-
-    management->setEnabled(!o->isReadOnly());
-    setDisabledPalette(management);
-
-    unprotected->setEnabled(!o->isReadOnly());
-    setDisabledPalette(unprotected);
-
-    ext->setEnabled(!o->isReadOnly());
-    setDisabledPalette(ext);
-
-    seclevel->setEnabled(!o->isReadOnly());
-    setDisabledPalette(seclevel);
-
-    netzone->setEnabled(!o->isReadOnly());
-    setDisabledPalette(netzone);
-
-
     init=false;
 }
-    
+
 void InterfaceDialog::changed()
 {
     if (fwbdebug)
@@ -285,10 +287,10 @@ void InterfaceDialog::validate(bool *res)
 {
     *res=true;
     if (!isTreeReadWrite(this,obj)) *res=false;
-    if (!validateName(this,obj,obj_name->text())) *res=false;
+    if (!validateName(this,obj,m_dialog->obj_name->text())) *res=false;
 }
 
-void InterfaceDialog::isChanged(bool *res)
+void InterfaceDialog::isChanged(bool*)
 {
     //*res=(!init && apply->isEnabled());
 }
@@ -305,13 +307,13 @@ void InterfaceDialog::applyChanges()
 
     string oldname=obj->getName();
     string oldlabel=s->getLabel();
-    obj->setName( string(obj_name->text().utf8()) );
-    obj->setComment( string(comment->text().utf8()) );
+    obj->setName( string(m_dialog->obj_name->text().toUtf8().constData()) );
+    obj->setComment( string(m_dialog->comment->toPlainText().toUtf8().constData()) );
 
-    s->setLabel( string(label->text().utf8()) );
-    s->setDyn( dynamic->isChecked() );
-    s->setUnnumbered( unnumbered->isChecked() );
-    s->setBridgePort( bridgeport->isChecked() );
+    s->setLabel( string(m_dialog->label->text().toUtf8().constData()) );
+    s->setDyn( m_dialog->dynamic->isChecked() );
+    s->setUnnumbered( m_dialog->unnumbered->isChecked() );
+    s->setBridgePort( m_dialog->bridgeport->isChecked() );
 
     FWObject *f = obj->getParent();
     bool supports_security_levels = false;
@@ -331,27 +333,28 @@ void InterfaceDialog::applyChanges()
     if (Firewall::isA( f ))
     {
         if (supports_security_levels)
-            obj->setInt("security_level", seclevel->value() );
+            obj->setInt("security_level", m_dialog->seclevel->value() );
         else
-            obj->setInt("security_level", (ext->isChecked()) ? 0 : 100 );
+            obj->setInt("security_level", (m_dialog->ext->isChecked()) ? 0 : 100 );
 
         if (supports_unprotected)
-            obj->setBool("unprotected",  unprotected->isChecked() );
+            obj->setBool("unprotected",  m_dialog->unprotected->isChecked() );
 
         if (supports_network_zones)
             obj->setStr("network_zone",
-                        netzoneObjectNos[ netzone->currentItem() ].latin1() );
-
-        s->setManagement( management->isChecked() );
+                        FWObjectDatabase::getStringId(
+                            netzoneObjectNos[
+                                m_dialog->netzone->currentIndex() ]));
 
+        s->setManagement( m_dialog->management->isChecked() );
     }
 
-    om->updateObjName(obj,
+    mw->updateObjName(obj,
                       QString::fromUtf8(oldname.c_str()),
                       QString::fromUtf8(oldlabel.c_str()));
 
     //apply->setEnabled( false );
-    om->updateLastModifiedTimestampForAllFirewalls(obj);
+    mw->updateLastModifiedTimestampForAllFirewalls(obj);
 }
 
 void InterfaceDialog::discardChanges()
