@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: LibExportDialog.cpp,v 1.18 2006/10/22 04:39:36 vkurland Exp $
+  $Id: LibExportDialog.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,13 +17,13 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -45,7 +45,6 @@
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Interface.h"
 
-#include <qlistbox.h>
 #include <qcheckbox.h>
 #include <qmessagebox.h>
 #include <qapplication.h>
@@ -68,41 +67,53 @@
 using namespace std;
 using namespace libfwbuilder;
 
-
-LibExportDialog::LibExportDialog( QWidget* parent, const char* name, bool modal )
-    : LibExport_q(parent, name, modal)
+LibExportDialog::~LibExportDialog()
 {
+    delete m_dialog;
+}
+
+LibExportDialog::LibExportDialog( QWidget* parent, const char* name, bool  )
+    : QDialog(parent)
+{
+    setWindowTitle(name);
+    m_dialog = new Ui::LibExport_q;
+    m_dialog->setupUi(this);
+
     init();
 }
 
 void LibExportDialog::init()
 {
 //    resize( QSize(500, 450) );
-    exportRO->setChecked(true);
+    m_dialog->exportRO->setChecked(true);
 
-    libs->clear();
+    m_dialog->libs->clear();
     list<FWObject*> ll = mw->db()->getRoot()->getByType( Library::TYPENAME );
     int n=0;
     string libicn;
 
     for (FWObject::iterator i=ll.begin(); i!=ll.end(); i++,n++)
     {
-        if (libicn.empty())
-            libicn=Resources::global_res->getObjResourceStr(*i,"icon-tree").c_str();
+        /*if (libicn.empty())
+            libicn=Resources::global_res->getObjResourceStr(*i,"icon-tree").c_str();*/
 
         mapOfLibs[n]= (*i);
 
         QPixmap pm;
-        QString icn = libicn.c_str();
+        QString icn = (":/Icons/" + (*i)->getTypeName() + "/icon-tree").c_str();
+        //QString icn = libicn.c_str();
         if ( ! QPixmapCache::find( icn, pm) )
         {
-            pm = QPixmap::fromMimeSource( icn );
+            pm.load( icn );
             QPixmapCache::insert( icn, pm);
         }
 
-        libs->insertItem(pm,
+        QListWidgetItem *item = new QListWidgetItem(QString::fromUtf8((*i)->getName().c_str()));
+        item->setIcon(QIcon(pm));
+        /*m_dialog->libs->addItem(QIcon(pm),
                          QString::fromUtf8((*i)->getName().c_str()),
-                         n);
+                         n);*/
+        m_dialog->libs->addItem(item);
     }
 
 }
