@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjConflictResolutionDialog.h,v 1.8 2007/02/04 03:58:59 vkurland Exp $
+  $Id: ObjConflictResolutionDialog.h 351 2008-07-13 00:29:05Z svn $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,7 +27,7 @@
 #ifndef __OBJCONFLICTRESOLUTIONDIALOG_H_
 #define __OBJCONFLICTRESOLUTIONDIALOG_H_
 
-#include "objconflictresolutiondialog_q.h"
+#include <ui_objconflictresolutiondialog_q.h>
 
 #include "fwbuilder/FWObjectDatabase.h"
 
@@ -37,7 +37,7 @@ namespace libfwbuilder {
     class FWObject;
 };
 
-class ObjConflictResolutionDialog : public ObjConflictResolutionDialog_q
+class ObjConflictResolutionDialog : public QDialog
 {
     Q_OBJECT
 
@@ -62,6 +62,8 @@ protected:
     QString  makeBold(const QString &str);
 
 public:
+    Ui::ObjConflictResolutionDialog_q *m_dialog;
+    
     ObjConflictResolutionDialog(QWidget *parent);
     virtual ~ObjConflictResolutionDialog();
 
@@ -100,7 +102,7 @@ class CompareObjectsDialog :
     std::list<QString> report;
     QStringList        report_attributes;
     int                num_conflicts;
-    int                column_width[];
+    int                column_width[4];
     
     void writeColumn(std::ostringstream &sstr,
                      int column_num,
