@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectEditor.cpp,v 1.44 2007/04/14 00:18:43 vkurland Exp $
+  $Id: ObjectEditor.cpp 495 2008-08-30 06:51:30Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,7 +17,7 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
@@ -25,7 +25,7 @@
 
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -34,30 +34,34 @@
 #include <qobject.h>
 #include <qpixmap.h>
 #include <qmessagebox.h>
-#include <qpopupmenu.h>
-#include <qobjectlist.h>
+#include <qmenu.h>
 #include <qlayout.h>
-#include <qwidgetstack.h>
+#include <qstackedwidget.h>
 #include <qpushbutton.h>
 
 #include "DialogFactory.h"
 #include "FWBTree.h"
-#include "FWWindow.h"
+#include "ProjectPanel.h"
 #include "FWBSettings.h"
 #include "GroupObjectDialog.h"
 #include "ActionsDialog.h"
 #include "MetricEditorPanel.h"
 #include "CommentEditorPanel.h"
-
+#include "ObjectManipulator.h"
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Host.h"
 #include "fwbuilder/Network.h"
+#include "fwbuilder/NetworkIPv6.h"
 #include "fwbuilder/IPv4.h"
+#include "fwbuilder/IPv6.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/AddressTable.h"
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/ObjectGroup.h"
+#include "fwbuilder/Policy.h"
+#include "fwbuilder/NAT.h"
+#include "fwbuilder/Routing.h"
 
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWReference.h"
@@ -68,10 +72,12 @@
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
+#include "fwbuilder/ICMP6Service.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/ServiceGroup.h"
 #include "fwbuilder/TagService.h"
+#include "fwbuilder/UserService.h"
 
 #include "fwbuilder/Interval.h"
 #include "fwbuilder/IntervalGroup.h"
@@ -85,11 +91,14 @@ using namespace libfwbuilder;
 
 
 
-ObjectEditor::ObjectEditor( QWidget *parent )
+ObjectEditor::ObjectEditor( QWidget *parent, ProjectPanel *project):
+    QObject(parent), opened(0), visible(-1),
+    parentWidget(dynamic_cast<QStackedWidget*>(parent)),
+    closeButton(0),
+    applyButton(0),
+    m_project(project),
+    openedOpt(optNone)
 {
-    opened       = NULL;
-    openedOpt    = optNone;
-    visible      = -1;
 
 #if defined(Q_WS_X11)
 /* do something that makes sense only on X11 */
@@ -101,105 +110,132 @@ ObjectEditor::ObjectEditor( QWidget *parent )
 
 #endif
 
-    parentWidget=(QWidgetStack*)parent; 
     QWidget *w;
-    w= DialogFactory::createDialog(parent,Library::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,UserService::TYPENAME);
+    stackIds[UserService::TYPENAME]  = parentWidget->addWidget(w);
+    dialogs[stackIds[UserService::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,Policy::TYPENAME);
+    stackIds[Policy::TYPENAME]  = parentWidget->addWidget(w);
+    dialogs[stackIds[Policy::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,NAT::TYPENAME);
+    stackIds[NAT::TYPENAME]  = parentWidget->addWidget(w);
+    dialogs[stackIds[NAT::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,Routing::TYPENAME);
+    stackIds[Routing::TYPENAME]  = parentWidget->addWidget(w);
+    dialogs[stackIds[Routing::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,Library::TYPENAME);
     stackIds[Library::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Library::TYPENAME]] = w;
-    
-    w= DialogFactory::createDialog(parent,IPv4::TYPENAME);
+
+    w= DialogFactory::createDialog(m_project, parent,IPv4::TYPENAME);
     stackIds[IPv4::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[IPv4::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,physAddress::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,IPv6::TYPENAME);
+    stackIds[IPv6::TYPENAME]  = parentWidget->addWidget(w);
+    dialogs[stackIds[IPv6::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,physAddress::TYPENAME);
     stackIds[physAddress::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[physAddress::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,AddressRange::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,AddressRange::TYPENAME);
     stackIds[AddressRange::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[AddressRange::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,Firewall::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,Firewall::TYPENAME);
     stackIds[Firewall::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Firewall::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,Host::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,Host::TYPENAME);
     stackIds[Host::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Host::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,Interface::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,Interface::TYPENAME);
     stackIds[Interface::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Interface::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,Network::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,Network::TYPENAME);
     stackIds[Network::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Network::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,CustomService::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,NetworkIPv6::TYPENAME);
+    stackIds[NetworkIPv6::TYPENAME]  = parentWidget->addWidget(w);
+    dialogs[stackIds[NetworkIPv6::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,CustomService::TYPENAME);
     stackIds[CustomService::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[CustomService::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,IPService::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,IPService::TYPENAME);
     stackIds[IPService::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[IPService::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,ICMPService::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,ICMPService::TYPENAME);
     stackIds[ICMPService::TYPENAME] = parentWidget->addWidget(w);
     dialogs[stackIds[ICMPService::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,TCPService::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,ICMP6Service::TYPENAME);
+    stackIds[ICMP6Service::TYPENAME] = parentWidget->addWidget(w);
+    dialogs[stackIds[ICMP6Service::TYPENAME]] = w;
+
+    w= DialogFactory::createDialog(m_project, parent,TCPService::TYPENAME);
     stackIds[TCPService::TYPENAME] = parentWidget->addWidget(w);
     dialogs[stackIds[TCPService::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,UDPService::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,UDPService::TYPENAME);
     stackIds[UDPService::TYPENAME] = parentWidget->addWidget(w);
     dialogs[stackIds[UDPService::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,ObjectGroup::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,ObjectGroup::TYPENAME);
     stackIds[ObjectGroup::TYPENAME] = parentWidget->addWidget(w);
     dialogs[stackIds[ObjectGroup::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,ServiceGroup::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,ServiceGroup::TYPENAME);
     stackIds[ServiceGroup::TYPENAME] = parentWidget->addWidget(w);
     dialogs[stackIds[ServiceGroup::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,IntervalGroup::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,IntervalGroup::TYPENAME);
     stackIds[IntervalGroup::TYPENAME] = parentWidget->addWidget(w);
     dialogs[stackIds[IntervalGroup::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,Interval::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,Interval::TYPENAME);
     stackIds[Interval::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Interval::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,Rule::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,Rule::TYPENAME);
     stackIds[Rule::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[Rule::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,RoutingRule::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,RoutingRule::TYPENAME);
     stackIds[RoutingRule::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[RoutingRule::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,PolicyRule::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,PolicyRule::TYPENAME);
     stackIds[PolicyRule::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[PolicyRule::TYPENAME]] = w;
 
-    w= DialogFactory::createDialog(parent,NATRule::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,NATRule::TYPENAME);
     stackIds[NATRule::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[NATRule::TYPENAME]] = w;
 
 
-    w= DialogFactory::createDialog(parent,DNSName::TYPENAME);
+    w= DialogFactory::createDialog(m_project, parent,DNSName::TYPENAME);
     stackIds[DNSName::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[DNSName::TYPENAME]] = w;
-    
-    w= DialogFactory::createDialog(parent,AddressTable::TYPENAME);
+
+    w= DialogFactory::createDialog(m_project, parent,AddressTable::TYPENAME);
     stackIds[AddressTable::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[AddressTable::TYPENAME]] = w;
-    
-    w= DialogFactory::createDialog(parent,TagService::TYPENAME);
+
+    w= DialogFactory::createDialog(m_project, parent,TagService::TYPENAME);
     stackIds[TagService::TYPENAME]  = parentWidget->addWidget(w);
     dialogs[stackIds[TagService::TYPENAME]] = w;
-    
+
 
     w= new ActionsDialog(parent);
     stackIds[getOptDialogName(optAction)]  = parentWidget->addWidget(w);
@@ -209,7 +245,7 @@ ObjectEditor::ObjectEditor( QWidget *parent )
     stackIds[getOptDialogName(optComment)]  = parentWidget->addWidget(w);
     dialogs[stackIds[getOptDialogName(optComment)]] = w;
 
-   
+
     w= new MetricEditorPanel(parent);
     stackIds[getOptDialogName(optMetric)]  = parentWidget->addWidget(w);
     dialogs[stackIds[getOptDialogName(optMetric)]] = w;
@@ -217,7 +253,7 @@ ObjectEditor::ObjectEditor( QWidget *parent )
     w=new QWidget(parent);
     stackIds["BLANK"]  = parentWidget->addWidget(w);
     dialogs[stackIds["BLANK"]] = w;
-    
+
 
 }
 
@@ -229,8 +265,8 @@ void ObjectEditor::show()
     //if (st->haveScreenPosition("Object Editor"))
     //    dialogs[ visible ]->move(st->getScreenPosition("Object Editor"));
     //dialogs[ visible ]->show();
-    parentWidget->raiseWidget(visible);
-    mw->openEditorPanel();
+    parentWidget->setCurrentIndex(visible);
+    m_project->openEditorPanel();
 }
 
 void ObjectEditor::hide()
@@ -252,7 +288,7 @@ void ObjectEditor::hide()
 //
 //       dialogs[ visible ]->hide();
 //    }
-    mw->closeEditorPanel();
+    m_project->closeEditorPanel();
     visible=-1;
 }
 
@@ -275,7 +311,7 @@ QString ObjectEditor::getOptDialogName(OptType t)
 void ObjectEditor::openOpt(FWObject *obj,OptType t)
 {
     if (Rule::cast(obj)==NULL) return;
-    
+
     disconnectSignals();
 
     int wid= stackIds[getOptDialogName(t)];
@@ -291,7 +327,7 @@ void ObjectEditor::openOpt(FWObject *obj,OptType t)
     connect(this, SIGNAL(validate_sign(bool*)),
             dialogs[ wid ],
             SLOT(validate(bool*)));
-            
+
     connect(this, SIGNAL(applyChanges_sign()),
             dialogs[ wid ],
             SLOT(applyChanges()));
@@ -308,11 +344,11 @@ void ObjectEditor::openOpt(FWObject *obj,OptType t)
             SLOT(changed()));
 
     emit loadObject_sign(obj);
-    
+
     opened = obj;
     openedOpt = t;
     applyButton->setEnabled(false);
-    
+
 }
 
 void ObjectEditor::open(FWObject *obj)
@@ -332,8 +368,8 @@ void ObjectEditor::open(FWObject *obj)
 //        disconnect( SIGNAL(close_sign(QCloseEvent*)) );
 
         //hide();
-        
-        
+
+
         visible=wid;
 
         show();
@@ -345,11 +381,11 @@ void ObjectEditor::open(FWObject *obj)
         connect(this, SIGNAL(validate_sign(bool*)),
                 dialogs[ wid ],
                 SLOT(validate(bool*)));
-                
+
         //connect(this, SIGNAL(isChanged_sign(bool*)),
         //        dialogs[ wid ],
         //        SLOT(isChanged(bool*)));
-                
+
         connect(this, SIGNAL(applyChanges_sign()),
                 dialogs[ wid ],
                 SLOT(applyChanges()));
@@ -404,7 +440,7 @@ void ObjectEditor::validateAndClose(QCloseEvent *e)
     {
         if (e) e->accept();
         //disconnectSignals();  // all signals will be disconnected
-                                // in next open(...)       
+                                // in next open(...)
         hide();
         return;
     }
@@ -418,12 +454,16 @@ bool ObjectEditor::validateAndSave()
     bool ischanged=false;
     ischanged = isModified();
     //emit isChanged_sign(&ischanged);
-    
+
+    if (fwbdebug)
+        qDebug("ObjectEditor::validateAndSave  "
+               "ischanged=%d db->isReadOnly()=%d",
+               ischanged, m_project->db()->isReadOnly());
+
 /* if nothing changed or tree is read-only, just close dialog */
-    if (!ischanged || !isTreeReadWrite(dialogs[ visible ],mw->db()))
+    if (!ischanged || !isTreeReadWrite(dialogs[visible], m_project->db()))
     {
-        if (fwbdebug)
-            qDebug("ObjectEditor::validateAndSave: no changes");
+        if (fwbdebug) qDebug("ObjectEditor::validateAndSave: no changes");
         return true;
     }
 
@@ -432,22 +472,27 @@ bool ObjectEditor::validateAndSave()
     emit validate_sign( &isgood );
     if (!isgood)
     {
-        switch ( QMessageBox::warning(dialogs[ visible ],
-                                      "Firewall Builder",
-                                      tr("Modifications done to this object can not be saved.\nDo you want to continue editing it ?"),
-                                      tr("&Continue editing"),
-                                      tr("&Discard changes"),
-                                      QString::null,
-                                      0, 1 ) )
+        switch (
+            QMessageBox::warning(
+                dialogs[ visible ],
+                "Firewall Builder",
+                tr("Modifications done to this object can not be saved.\n"
+                   "Do you want to continue editing it ?"),
+                tr("&Continue editing"),
+                tr("&Discard changes"),
+                QString::null,
+                0, 1 ) )
         {
         case 0:
             if (fwbdebug)
-                qDebug("ObjectEditor::validateAndSave:  return false, can not switch to another object");
+                qDebug("ObjectEditor::validateAndSave:  return false, "
+                       "can not switch to another object");
             return false;
 
         default:
             if (fwbdebug)
-                qDebug("ObjectEditor::validateAndSave  return true, discard changes, can switch to another object");
+                qDebug("ObjectEditor::validateAndSave  return true, "
+                       "discard changes, can switch to another object");
             discard();
             return true;
         }
@@ -460,11 +505,14 @@ bool ObjectEditor::validateAndSave()
         emit applyChanges_sign();
     } else
     {
-        switch ( QMessageBox::warning(dialogs[ visible ],
-                                      "Firewall Builder",
-                                      tr("This object has been modified but not saved.\nDo you want to save it ?"),
-                                      tr("&Save"), tr("&Discard"), tr("&Continue editing"),
-                                      0, 2 ) )
+        switch (
+            QMessageBox::warning(
+                dialogs[ visible ],
+                "Firewall Builder",
+                tr("This object has been modified but not saved.\n"
+                   "Do you want to save it ?"),
+                tr("&Save"), tr("&Discard"), tr("&Continue editing"),
+                0, 2 ) )
         {
         case 0:
             apply();
@@ -492,7 +540,7 @@ void ObjectEditor::setApplyButton(QPushButton * b)
     applyButton=b;
     applyButton->setEnabled(false);
     connect((QWidget*)applyButton,SIGNAL(clicked()),this,SLOT(apply()));
-    
+
 }
 
 void ObjectEditor::close()
@@ -504,13 +552,17 @@ void ObjectEditor::close()
 
 void ObjectEditor::apply()
 {
+    if (fwbdebug) qDebug("ObjectEditor::apply");
+
     bool isgood=true;
     emit validate_sign( &isgood );
+
     if (isgood)
     {
         emit applyChanges_sign();
         applyButton->setEnabled(false);
-        mw->updateRuleSetView( );
+        m_project->updateRuleSetView( );
+        m_project->updateTreeViewItemOrder();
     }
 }
 
@@ -532,7 +584,7 @@ void ObjectEditor::selectObject(FWObject *o)
    ((GroupObjectDialog *) dialogs[ visible ])->selectObject(o);
 }
 
-void ObjectEditor::selectionChanged(FWObject *o)
+void ObjectEditor::selectionChanged(FWObject*)
 {
     /*
     if (visible==-1) return;
@@ -551,7 +603,7 @@ void ObjectEditor::actionChanged(FWObject *o)
     }
     //if (opened==o) return;
     openOpt (o,ObjectEditor::optAction);
-    
+
     show();
 }
 
