@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectEditor.h,v 1.21 2006/09/07 15:42:12 vkurland Exp $
+  $Id: ObjectEditor.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,7 +27,7 @@
 #ifndef  __OBJECTEDITOR_H_
 #define  __OBJECTEDITOR_H_
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 
 #include "qdialog.h"
@@ -40,8 +40,9 @@
 
 class ObjectTreeViewItem;
 class QComboBox;
-class QPopupMenu;
-class QWidgetStack;
+class QMenu;
+class QStackedWidget;
+class ProjectPanel;
 
 class ObjectEditor : public QObject {
 
@@ -52,9 +53,10 @@ class ObjectEditor : public QObject {
     
     libfwbuilder::FWObject          *opened;
     int                              visible;
-    QWidgetStack *parentWidget;
+    QStackedWidget *parentWidget;
     QPushButton *closeButton;
     QPushButton *applyButton;
+    ProjectPanel *m_project;
 
     void disconnectSignals();
     
@@ -62,11 +64,10 @@ public:
     enum OptType{optAction,optComment,optMetric,optNone};
 private: 
    OptType  openedOpt;
-    
 
 public:
    
-    ObjectEditor( QWidget *parent );
+    ObjectEditor( QWidget *parent, ProjectPanel *project);
     virtual ~ObjectEditor() {}
     
 
