@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectTreeView.cpp,v 1.43 2007/07/07 05:39:34 vkurland Exp $
+  $Id: ObjectTreeView.cpp 521 2008-09-10 06:15:27Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,14 +17,14 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -32,30 +32,40 @@
 #include "ObjectTreeView.h"
 #include "ObjectTreeViewItem.h"
 #include "ObjectManipulator.h"
-#include "ObjectEditor.h"
 #include "FWObjectDrag.h"
-#include "FWWindow.h"
+#include "FWBSettings.h"
 
+#include "FWObjectPropertiesFactory.h"
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/Group.h"
 
-#include <qdragobject.h>
-#include <qlistview.h>
-#include <qheader.h>
+#include <QAbstractItemView>
+#include <QHeaderView>
+#include <QBitmap>
 #include <qpainter.h>
 #include <qpixmapcache.h>
+#include <QMimeData>
+#include <QDragMoveEvent>
+#include <QDropEvent>
+#include <QDragLeaveEvent>
+#include <QFocusEvent>
+#include <QPixmap>
+#include <QMouseEvent>
+#include <QKeyEvent>
+#include <QDragEnterEvent>
 
 #include <iostream>
 #include <algorithm>
+#include "ProjectPanel.h"
 
 using namespace std;
 using namespace libfwbuilder;
 
 ObjectTreeView* ObjectTreeViewItem::getTree()
 {
-    return dynamic_cast<ObjectTreeView*>(listView());
+    return dynamic_cast<ObjectTreeView*>(treeWidget());
 }
 
 /****************************************************************************
@@ -64,10 +74,16 @@ ObjectTreeView* ObjectTreeViewItem::getTree()
  *
  ****************************************************************************/
 
-ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
-    QListView(parent,name,f),
-    singleClickTimer(this)
+ObjectTreeView::ObjectTreeView(ProjectPanel* project,
+                               QWidget* parent,
+                               const char * name,
+                               Qt::WFlags f) :
+    QTreeWidget(parent), 
+    m_project(project)
 {
+    setObjectName(name);
+    this->setParent(parent, f);
+    setFont (st->getTreeFont());
 //    setAcceptDrops( TRUE );
     item_before_drag_started=NULL;
     lastSelected = NULL;
@@ -86,79 +102,135 @@ ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
      * start drag without switching object in the editor. The problem
      * is that mouseReleaseEvent is received in this widget after the
      * d&d ends with a drop somewhere else, which triggers call to
-     * contentsMouseReleaseEvent which switches object in the
+     * mouseReleaseEvent which switches object in the
      * editor. This is undesired when the editor shows a group and we
      * try to drag and drop an object into that group. Flag
      * process_mouse_release_event is used to suppress object
-     * switching when contentsMouseReleaseEvent is called after
+     * switching when mouseReleaseEvent is called after
      * successfull drop.
      */
     process_mouse_release_event = true;
 
-    connect( this, SIGNAL(currentChanged(QListViewItem*)),
-             this, SLOT(currentChanged(QListViewItem*)) );
+    connect(this, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
+            this, SLOT(currentItemChanged(QTreeWidgetItem*)));
 
-    connect( this, SIGNAL( selectionChanged() ),
-             this, SLOT( selectionChanged() ) );
+    connect(this, SIGNAL(itemSelectionChanged()),
+            this, SLOT(itemSelectionChanged()));
 
-    connect(this, SIGNAL( collapsed(QListViewItem*)),
-            this, SLOT( collapsed(QListViewItem*)) );
+    connect(this, SIGNAL(itemCollapsed(QTreeWidgetItem*)),
+            this, SLOT(itemCollapsed(QTreeWidgetItem*)));
 
-    connect(this, SIGNAL( expanded(QListViewItem*)),
-            this, SLOT( expanded(QListViewItem*)) );
+    connect(this, SIGNAL(itemExpanded(QTreeWidgetItem*)),
+            this, SLOT(itemExpanded(QTreeWidgetItem*)));
 
-    connect( &singleClickTimer, SIGNAL( timeout() ),
-             this, SLOT( resetSelection() ) );
+    setColumnCount(1);
 
-
-
-    addColumn( tr( "Object" ) );
+    QStringList qsl;
+    qsl.push_back(tr("Object"));
+    setHeaderLabels(qsl);
 
     header()->hide();
 
-    setMinimumSize( QSize( 200, 0 ) );
+    setMinimumSize( QSize( 100, 0 ) );
 
-//    QFont objTreeView_font(  font() );
-//    setFont( objTreeView_font ); 
-//    setCursor( QCursor( 0 ) );
-    setResizePolicy( QListView::AutoOneFit );
-
-    setColumnWidthMode(0, QListView::Maximum);
-    setItemMargin( 2 );
-
-    setDragAutoScroll( TRUE );
+    setAutoScroll( TRUE );
     setAllColumnsShowFocus( TRUE );
-    setSelectionMode( QListView::Extended );
-    setShowSortIndicator( FALSE );
+    setSelectionMode( ExtendedSelection );
+    setAcceptDrops( true );
+    setDragDropMode( QAbstractItemView::DragDrop );
     setRootIsDecorated( TRUE );
-    setResizeMode( QListView::AllColumns );
-    setTreeStepSize( 15 );
 
-    setSorting(0,true);
+// disable sorting, otherwise gui crashes when built with
+// QT 4.3.4 (discovered on Ubuntu Hardy). Crash happened when
+// second object was added to any branch of the tree.
+//
+//    setSortingEnabled(true);
 }
 
-void ObjectTreeView::currentChanged(QListViewItem *itm)
+bool ObjectTreeView::event( QEvent *event )
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::currentChanged  itm=%s",itm->text(0).ascii());
+    if (event->type() == QEvent::ToolTip)
+    {
+        QHelpEvent *he = (QHelpEvent*) event;
+        QPoint pos = he->pos();
+
+        if (st->getObjTooltips())
+        {
+            int cx = pos.x(), cy = pos.y();
+
+            FWObject  *obj=NULL;
+            QRect      cr;
+
+            QTreeWidgetItem *itm = itemAt(QPoint(cx, cy - header()->height()));
+            if (itm==NULL) return false;
+            ObjectTreeViewItem *oivi  = dynamic_cast<ObjectTreeViewItem*>(itm);
+            assert(oivi!=NULL);
+            obj = oivi->getFWObject();
+
+            if (obj==NULL) return false;
+
+            cr = visualItemRect(itm);
+
+            QRect global = QRect(
+                viewport()->mapToGlobal(cr.topLeft()),
+                viewport()->mapToGlobal(cr.bottomRight()));
+
+            //finally stretch rect up to component's width and even more
+            //(it fixes bug with horizontal scroll)
+            global.setWidth(width() + horizontalOffset());
+
+            QToolTip::showText(mapToGlobal( he->pos() ),
+                FWObjectPropertiesFactory::getObjectPropertiesDetailed(obj,
+                                                                       true,
+                                                                       true),
+                this, global);
+        }
+
+        return true;
+    }
+
+    return QTreeWidget::event(event);
+}
+
+
+void ObjectTreeView::currentItemChanged(QTreeWidgetItem *cur)
+{
+    if (fwbdebug) qDebug("ObjectTreeView::currentChanged  itm=%s",
+                         cur->text(0).toAscii().constData());
     expandOrCollapse = false;
 
 //    lastSelected = ovi;
 //    lastSelected = currentItem();
 }
 
-void ObjectTreeView::collapsed(QListViewItem* itm)
+void ObjectTreeView::itemCollapsed(QTreeWidgetItem* itm)
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::collapsed  itm=%s",itm->text(0).ascii());
     expandOrCollapse = true;
+
+    ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem*>(itm);
+    assert(otvi!=NULL);
+    FWObject *o = otvi->getFWObject();
+    if (o)
+    {
+        int id = o->getId();
+        expanded_objects.erase(id);
+        if (fwbdebug) qDebug("Tree item for object id=%d collapsed", id);
+    }
 }
 
-void ObjectTreeView::expanded(QListViewItem* itm)
+void ObjectTreeView::itemExpanded(QTreeWidgetItem* itm)
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::expanded  itm=%s",itm->text(0).ascii());
     expandOrCollapse = true;
+
+    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+    assert(otvi!=NULL);
+    FWObject *o = otvi->getFWObject();
+    if (o)
+    {
+        int id = o->getId();
+        expanded_objects.insert(id);
+        if (fwbdebug) qDebug("Tree item for object id=%d expanded", id);
+    }
 }
 
 /*
@@ -187,7 +259,7 @@ std::vector<libfwbuilder::FWObject*> ObjectTreeView::getSimplifiedSelection()
 
 FWObject* ObjectTreeView::getCurrentObject()
 {
-    QListViewItem *ovi = currentItem();
+    QTreeWidgetItem *ovi = currentItem();
     ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
     assert(otvi!=NULL);
     return otvi->getFWObject();
@@ -205,59 +277,55 @@ bool ObjectTreeView::isUnlockable()
 
 void ObjectTreeView::focusInEvent(QFocusEvent* ev)
 {
-    if (fwbdebug) qDebug("ObjectTreeView::focusInEvent 1");
-    QListView::focusInEvent(ev);
-    QListViewItem *ci = currentItem();
-    if (ci) repaintItem(ci);
-    if (fwbdebug) qDebug("ObjectTreeView::focusInEvent 2");
+    QTreeWidget::focusInEvent(ev);
+    QTreeWidgetItem *ci = currentItem();
+    if (ci) repaint();
 }
 
 void ObjectTreeView::focusOutEvent(QFocusEvent* ev)
 {
-    if (fwbdebug) qDebug("ObjectTreeView::focusOutEvent 1");
-    QListView::focusOutEvent(ev);
-    QListViewItem *ci = currentItem();
-    if (ci) repaintItem(ci);
-    if (fwbdebug) qDebug("ObjectTreeView::focusOutEvent 2");
+    QTreeWidget::focusOutEvent(ev);
+    QTreeWidgetItem *ci = currentItem();
+    if (ci) repaint();
 }
 
 void ObjectTreeView::updateTreeItems()
 {
     if (fwbdebug) qDebug("ObjectTreeView::updateTreeItems 1");
-    QListViewItemIterator it(this);
-    QListViewItem *itm;
+    QTreeWidgetItemIterator it(this);
+    QTreeWidgetItem *itm;
     ObjectTreeViewItem *otvi;
     FWObject *obj;
     QString icn;
 
     QPixmap pm_lock;
-    if ( ! QPixmapCache::find( "lock.png", pm_lock) )
+    if ( ! QPixmapCache::find( ":/Icons/lock.png", pm_lock) )
     {
-        pm_lock = QPixmap::fromMimeSource( "lock.png" );
-        QPixmapCache::insert( "lock.png", pm_lock);
+        pm_lock.load( ":/Icons/lock.png" );
+        QPixmapCache::insert( ":/Icons/lock.png", pm_lock);
     }
-    
-    while ( it.current() ) 
+
+    while ( *it )
     {
-        itm= it.current();
+        itm= *it;
         otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
         obj=otvi->getFWObject();
-        
-        if (FWBTree::isSystem(obj))
-            icn="folder1.png";
+
+        if (m_project->isSystem(obj))
+            icn=":/Icons/folder1.png";
         else
-            icn=Resources::global_res->getObjResourceStr(obj, "icon-tree").c_str();
-        
+            icn=(":/Icons/"+obj->getTypeName()+"/icon-tree").c_str();
+
         QPixmap pm_obj;
         if ( ! QPixmapCache::find( icn, pm_obj) )
         {
-            pm_obj = QPixmap::fromMimeSource( icn );
+            pm_obj.load( icn );
             QPixmapCache::insert( icn, pm_obj);
         }
 
-        if (obj->getBool("ro"))  itm->setPixmap(0, pm_lock );
-        else                     itm->setPixmap(0, pm_obj );
-        
+        if (obj->getBool("ro"))  itm->setIcon(0, pm_lock);//setPixmap(0, pm_lock );
+        else                     itm->setIcon(0, pm_obj );
+
         Firewall *fw=Firewall::cast(obj);
         if (fw!=NULL)
         {
@@ -265,52 +333,56 @@ void ObjectTreeView::updateTreeItems()
                     QString::fromUtf8(fw->getName().c_str())+" *":
                     QString::fromUtf8(fw->getName().c_str()));
         }
-        
+
         ++it;
     }
-    triggerUpdate ();  
+
+    update(); //for replacement as previous string
     if (fwbdebug) qDebug("ObjectTreeView::updateTreeItems 2");
 }
 
-/*
- * This is where drag starts
- */
-QDragObject* ObjectTreeView::dragObject()
+void ObjectTreeView::startDrag(Qt::DropActions supportedActions)
 {
-    if (fwbdebug) qDebug("ObjectTreeView::dragObject");
-
-    QListViewItem *ovi = currentItem();
+    QTreeWidgetItem *ovi = currentItem();
     ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
 
     FWObject *current_obj = getCurrentObject();
 
-/* can't drag system folders 
+    if (fwbdebug) qDebug("ObjectTreeView::startDrag: this: %p current_obj: %s",
+                         this, current_obj->getName().c_str());
+
+/* can't drag system folders
 
     in fact, I have to allow to drag system folders because otherwise
     QListView triggers highlighting of objects in the tree when user
     drags mouse cursor across them. This is weird behavior and there
     does not seem to be any way to turn it off. It happens close to
-    the end of void QListView::contentsMouseMoveEvent( QMouseEvent * e)
+    the end of void QListView::mouseMoveEvent( QMouseEvent * e)
     (See code after they decided that they do not need to call startDrag())
 
-    if (FWBTree::isSystem(obj)) return NULL;
+    if (m_project->isSystem(obj)) return NULL;
 */
-    QString icn =
-        Resources::global_res->getObjResourceStr(current_obj, "icon-ref").c_str();
+    QString icn = (":/Icons/"+current_obj->getTypeName()+"/icon-ref").c_str();
 
     vector<FWObject*> so = getSimplifiedSelection();
 
     list<FWObject*> dragobj;
     for (vector<FWObject*>::iterator v=so.begin(); v!=so.end(); v++)
-        dragobj.push_back( *v );
+    {   
+        //m_project->check4Depends(*v, dragobj);
 
-    FWObjectDrag    *drag = new FWObjectDrag(dragobj, this);
-    //QPixmap          pm   = QPixmap::fromMimeSource( icn_filename );
+        if (fwbdebug)
+            qDebug("ObjectTreeView::startDrag: adding object to drag list: %s",
+                   (*v)->getName().c_str());
+
+        dragobj.push_back( *v );
+    }
+    FWObjectDrag *drag = new FWObjectDrag(dragobj, this);
 
     QPixmap pm;
     if ( ! QPixmapCache::find( icn, pm) )
     {
-        pm = QPixmap::fromMimeSource( icn );
+        pm.load( icn );
         QPixmapCache::insert( icn, pm);
     }
 
@@ -319,7 +391,7 @@ QDragObject* ObjectTreeView::dragObject()
         QPixmap npm(32,32);
         QPainter p( &npm );
         p.fillRect( 0,0,32,32, QBrush( QColor("white"),Qt::SolidPattern ) );
-        p.setBackgroundMode( TransparentMode );
+        p.setBackgroundMode( Qt::TransparentMode );
         p.drawPixmap( 0, 32-pm.rect().height(), pm);
         p.setPen( QColor("red") );
         p.setBrush( QBrush( QColor("red"),Qt::SolidPattern ) );
@@ -331,14 +403,11 @@ QDragObject* ObjectTreeView::dragObject()
                                 txt );
         p.setPen( QColor("white") );
         p.drawText( 24-br.width()/2 , 4+br.height()/2, txt );
+        p.end();
         npm.setMask( npm.createHeuristicMask() );
-        drag->setPixmap( npm,
-                         QPoint( npm.rect().width() / 2,
-                                 npm.rect().height() / 2 ) );
+        drag->setPixmap( npm );
     } else
-        drag->setPixmap( pm,
-                         QPoint( pm.rect().width() / 2,
-                                 pm.rect().height() / 2 ) );
+        drag->setPixmap( pm );
 
 /*
  * This fragment returns selection in the tree back to the object that
@@ -357,26 +426,26 @@ QDragObject* ObjectTreeView::dragObject()
  * to that library before doing d&d. When they switch, ObjectTree
  * shown in the left panel becomes different from the tree in which
  * the group is located. When d&d finishes, the ObjectTree object
- * receives contentsMouseReleaseEvent event. Since it is not the right
+ * receives mouseReleaseEvent event. Since it is not the right
  * tree object, it can not properly restore selection and choses an
  * object that was previously opened in that tree, which in turn
  * changes the object opened in the editor panel. To make things
  * worse, this event is only delivered to the tree object on Mac OS X.
  *
- * 
+ *
  */
     if (fwbdebug) qDebug("ObjectTreeView::dragObject()  this=%p visible=%d",
                          this,visible);
 
-    FWObject *edit_obj = oe->getOpened();
+    FWObject *edit_obj = m_project->getOpenedEditor();
 
-    if (oe->isVisible() && 
+    if (m_project->isEditorVisible() &&
         dragobj.size()==1 &&
         edit_obj!=NULL &&
         current_obj->getLibrary()==edit_obj->getLibrary() )
     {
         if (fwbdebug) qDebug("ObjectTreeView::dragObject() reset selection");
-        setSelected(otvi,false);
+        otvi->setSelected(false);
         resetSelection();
     }
 
@@ -388,7 +457,7 @@ QDragObject* ObjectTreeView::dragObject()
      * we are dragging one object, and
      * object opened in editor is not the same as the one we are dragging
      */
-    if (oe->isVisible() && dragobj.size()==1 && oe->getOpened()!=obj)
+    if (m_project->isEditorVisible() && dragobj.size()==1 && m_project->getOpenedEditor()!=obj)
     {
         setSelected(otvi,false);
         resetSelection();
@@ -396,175 +465,145 @@ QDragObject* ObjectTreeView::dragObject()
 #endif
     if (fwbdebug) qDebug("ObjectTreeView::dragObject()  returns !NULL");
 
-    return drag;
+    drag->start(supportedActions);
 }
 
 void ObjectTreeView::dragEnterEvent( QDragEnterEvent *ev)
 {
-    ev->accept( FWObjectDrag::canDecode(ev) );
-    QListViewItem *ovi = currentItem();
-    item_before_drag_started = ovi;
+    ev->setAccepted(ev->mimeData()->hasFormat(FWObjectDrag::FWB_MIME_TYPE) );
+    ev->setDropAction(Qt::MoveAction);
+}
+
+bool ObjectTreeView::isCurrReadOnly(QDragMoveEvent *ev)
+{
+// the tree can accept drop only if it goes into a group and if that group
+//  validates the object and tree is not read-only
+    QTreeWidgetItem *ovi = itemAt(ev->pos());
+
+    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
+    FWObject *trobj;
+    if (otvi && (trobj = otvi->getFWObject()))
+        return trobj->isReadOnly();
+    return false;
 }
 
 void ObjectTreeView::dragMoveEvent( QDragMoveEvent *ev)
 {
-    bool    acceptE = false;
-    
-    if (FWObjectDrag::canDecode(ev))
+    if (isCurrReadOnly(ev) ||
+          !ev->mimeData()->hasFormat(FWObjectDrag::FWB_MIME_TYPE))
     {
-        int hy;
+        ev->setAccepted(false);
+        return;
+    }
+    list<FWObject*> dragol;
+    if (!FWObjectDrag::decode(ev, dragol))
+       ev->setAccepted(false);
+    for (list<FWObject*>::iterator i=dragol.begin();i!=dragol.end(); ++i)
+    {
+        FWObject *dragobj = *i;
+        assert(dragobj!=NULL);
 
-//        hy=header()->height();    // if header is shown
-        hy=0;
-
-        QListViewItem *ovi = itemAt( ev->pos() );
-
-        ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-        if (otvi==NULL)
-        {   
-            ev->accept(acceptE);
+        if (m_project->isSystem(dragobj))
+        {
+// can not drop system folder anywhere 
+            ev->setAccepted(false);
             return;
         }
-
-        FWObject *trobj = otvi->getFWObject();
-
-/* the tree can accept drop only if it goes into a group and if that group
- * validates the object and tree is not read-only
- */
-        if (Group::cast(trobj)!=NULL  && 
-            !FWBTree::isSystem(trobj) &&
-            !trobj->isReadOnly()
-        )
-        {
-            acceptE = true;
-
-            Group    *g     = Group::cast(trobj);
-            list<FWObject*> dragol;
-            if (FWObjectDrag::decode(ev, dragol))
-            {
-                for (list<FWObject*>::iterator i=dragol.begin();
-                     i!=dragol.end(); ++i)
-                {
-                    FWObject *dragobj = *i;
-                    assert(dragobj!=NULL);
-
-                    if (FWBTree::isSystem(dragobj))
-                    {
-/* can not drop system folder anywhere */
-                        acceptE = false;
-                        break;
-                    }
-
-                    bool t= g->validateChild(dragobj);
-                    if (!t)
-                    {
-                        acceptE = false;
-                        break;
-                    }
-
-                    if (g->getPath(true) == "Services/Groups" && t)
-                        setOpen(ovi,true);
-
-                    if (g->getPath(true) == "Objects/Groups" && t)
-                        setOpen(ovi,true);
-                }
-            }
-        }
     }
-    ev->accept(acceptE);
+    ev->setAccepted(true);
 }
 
 void ObjectTreeView::dropEvent(QDropEvent *ev)
 {
-    QListViewItem *ovi = itemAt( ev->pos() );
-    ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
-    if (otvi==NULL) return;
-    FWObject *trobj = otvi->getFWObject();
+    if (fwbdebug) qDebug("ObjectTreeView::dropEvent");
 
-/* the tree can accept drop only if it goes into a group and if that group
- * validates the object and the tree is not read-only
- */
-    if (Group::cast(trobj)!=NULL  && 
-        !FWBTree::isSystem(trobj) &&
-        !trobj->isReadOnly()
-    )
+    list<FWObject*> dragol;
+    if (FWObjectDrag::decode(ev, dragol))
     {
-        Group *g=Group::cast(trobj);
-
-        item_before_drag_started=NULL;
-
-        list<FWObject*> dragol;
-        if (FWObjectDrag::decode(ev, dragol))
+        for (list<FWObject*>::iterator i=dragol.begin(); i!=dragol.end(); ++i)
         {
-            for (list<FWObject*>::iterator i=dragol.begin();
-                 i!=dragol.end(); ++i)
+            FWObject *dragobj = *i;
+            assert(dragobj);
+
+            ObjectTreeView * otvsource =
+                dynamic_cast<ObjectTreeView *>(ev->source());
+            if (otvsource!=NULL)
             {
-                FWObject *dragobj = *i;
-                assert(dragobj!=NULL);
+                FWObjectDatabase* root =
+                    otvsource->getCurrentObject()->getRoot();
 
-/* check for duplicates */
-                string cp_id=dragobj->getId();
-                list<FWObject*>::iterator j;
-                for(j=g->begin(); j!=g->end(); ++j)     
+                int id = dragobj->getId();
+                FWObject * item = root->getById(id, true);
+                if (fwbdebug)
                 {
-                    FWObject *o1=*j;
-                    if(cp_id==o1->getId()) continue;
-
-                    FWReference *ref;
-                    if( (ref=FWReference::cast(o1))!=NULL &&
-                        cp_id==ref->getPointerId()) return;
+                    qDebug("%d", item->getId());
+                    qDebug(item->getTypeName().c_str());
+                    qDebug("%d", dragobj->getId());
+                    qDebug(dragobj->getTypeName().c_str());
                 }
-
-                g->addRef(dragobj);
+                //ProjectPanel * ppsource =  otvsource->m_project ;
             }
-            setCurrentItem(ovi);
-            setSelected(ovi, true);
 
-//            emit objectDropped_sign(g);
+            FWObject *target = getDropTarget(ev, dragobj);
+            if (fwbdebug)
+                qDebug("ObjectTreeView::dropEvent  paste %s -> %s",
+                       dragobj->getName().c_str(), target->getName().c_str());
+
+            m_project->pasteTo(target, dragobj);
         }
     }
+    if (fwbdebug) qDebug("ObjectTreeView::dropEvent done");
+}
+
+FWObject *ObjectTreeView::getDropTarget(QDropEvent *ev, FWObject*)
+{
+    QTreeWidgetItem *ovi = itemAt(ev->pos());
+
+    ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem*>(ovi);
+    return otvi->getFWObject();
 }
 
 void ObjectTreeView::dragLeaveEvent( QDragLeaveEvent *ev)
 {
-    QListView::dragLeaveEvent(ev);
+    QTreeWidget::dragLeaveEvent(ev);
 
-    if (item_before_drag_started!=NULL)
-    {
-        setCurrentItem(item_before_drag_started);
-        setSelected(item_before_drag_started, true);
-    }
+    clearSelection();
 }
 
-void ObjectTreeView::contentsMouseMoveEvent( QMouseEvent * e )
+void ObjectTreeView::mouseMoveEvent( QMouseEvent * e )
 {
+    QTreeWidget::mouseMoveEvent(e);
     if (e==NULL)  return;
-
-    QListView::contentsMouseMoveEvent(e);
 }
 
-
-void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
+void ObjectTreeView::mousePressEvent( QMouseEvent *e )
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::contentsMousePressEvent");
+    if (fwbdebug) qDebug("ObjectTreeView::mousePressEvent");
 
     second_click = false;
     process_mouse_release_event = true;
 
     if (fwbdebug)
     {
-        qDebug(QString("ObjectTreeView::contentsMousePressEvent :: currentItem=%1")
-               .arg((currentItem())?currentItem()->text(0):"nil")
+        qDebug(QString("ObjectTreeView::mousePressEvent :: currentItem=%1")
+               .arg((currentItem())?currentItem()->text(0):"nil").toAscii().constData()
         );
-        qDebug(QString("ObjectTreeView::contentsMousePressEvent :: lastSelected=%2")
-               .arg((lastSelected)?lastSelected->text(0):"nil")
+        qDebug(QString("ObjectTreeView::mousePressEvent :: lastSelected=%2")
+                .arg((lastSelected)?lastSelected->text(0):"nil").toAscii().constData()
         );
     }
 
     lastSelected = currentItem();
 
-    QListView::contentsMousePressEvent(e);
+    QTreeWidget::mousePressEvent(e);
+
+    if (e->button() == Qt::LeftButton)
+    {
+        startingDrag = true;
+    }
+
+    if (e->button() == Qt::RightButton)
+        emit contextMenuRequested_sign(e->pos());
 }
 
 /*
@@ -587,21 +626,20 @@ void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
  * without switching object in the editor. The problem is that
  * mouseReleaseEvent is received in this widget after the d&d ends
  * with a drop somewhere else, which triggers call to
- * contentsMouseReleaseEvent which switches object in the editor. This
+ * mouseReleaseEvent which switches object in the editor. This
  * is undesired when the editor shows a group and we try to drag and
  * drop an object into that group. Flag process_mouse_release_event is
- * used to suppress object switching when contentsMouseReleaseEvent is
+ * used to suppress object switching when mouseReleaseEvent is
  * called after successfull drop.
  *
  */
-void ObjectTreeView::contentsMouseReleaseEvent( QMouseEvent *e )
+void ObjectTreeView::mouseReleaseEvent( QMouseEvent *e )
 {
     if (fwbdebug)
-        qDebug("ObjectTreeView::contentsMouseReleaseEvent 1 this=%p  process_mouse_release_event=%d",
+        qDebug("ObjectTreeView::mouseReleaseEvent 1 this=%p  process_mouse_release_event=%d",
                this,process_mouse_release_event);
 
-    QListView::contentsMouseReleaseEvent(e);
-
+    QTreeWidget::mouseReleaseEvent(e);
 
     if (!process_mouse_release_event)
     {
@@ -612,11 +650,11 @@ void ObjectTreeView::contentsMouseReleaseEvent( QMouseEvent *e )
     }
 
     if (fwbdebug)
-        qDebug("ObjectTreeView::contentsMouseReleaseEvent 2 selectedObjects.size()=%d getCurrentObject()=%p current object %s",
-               selectedObjects.size(),
+        qDebug("ObjectTreeView::mouseReleaseEvent 2 selectedObjects.size()=%d getCurrentObject()=%p current object %s",
+               int(selectedObjects.size()),
                getCurrentObject(),
                (getCurrentObject()!=NULL)?getCurrentObject()->getName().c_str():"nil");
-    
+
     if (expandOrCollapse) return;  // user expanded or collapsed subtree,
                                    // no need to change object in the editor
 
@@ -626,14 +664,32 @@ void ObjectTreeView::contentsMouseReleaseEvent( QMouseEvent *e )
     {
         // user selected multiple objects
         // do not let them if editor has unsaved changes
-        // 
-        if (oe->isVisible() && oe->isModified())
+        //
+        if (m_project->isEditorVisible() && m_project->isEditorModified())
             emit switchObjectInEditor_sign( getCurrentObject() );
         else
-            oe->blank();
+            m_project->blankEditor();
     }
 }
 
+/*
+ * normally QAbstractItemView::edit starts in-place editing. WE use
+ * double click to open object in a separate editor panel, but use
+ * this virtual method to tell unerlying class that editing has
+ * started, which prevents it from expanding/collapsing tree branch.
+ */
+bool ObjectTreeView::edit(const QModelIndex &index,
+                          EditTrigger trigger, QEvent *event)
+{
+    if (fwbdebug) qDebug("ObjectTreeView::edit");
+    if (trigger==QAbstractItemView::DoubleClicked)
+    {
+        editCurrentObject();
+        return true;
+    }
+    return QTreeWidget::edit(index, trigger, event);
+}
+
 /*
  * sends signal that should be connected to a slot in
  * ObjectManipulator which opens editor panel if it is closed and then
@@ -641,32 +697,9 @@ void ObjectTreeView::contentsMouseReleaseEvent( QMouseEvent *e )
  */
 void ObjectTreeView::editCurrentObject()
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::editCurrentObject");
-
+    if (fwbdebug) qDebug("ObjectTreeView::editCurrentObject");
     emit editCurrentObject_sign();
-
-    if (fwbdebug)
-        qDebug("ObjectTreeView::editCurrentObject done");
-}
-
-void ObjectTreeView::contentsMouseDoubleClickEvent( QMouseEvent *e )
-{
-    if (fwbdebug)
-        qDebug("ObjectTreeView::contentsMouseDoubleClickEvent");
-
-    second_click=true;
-    singleClickTimer.stop();
-
-    FWObject *obj = getCurrentObject();
-
-/* system folders open on doubleclick, while for regular objects it
- * opens an editor 
- */
-    if (FWBTree::isSystem(obj))
-        QListView::contentsMouseDoubleClickEvent(e);
-    else
-        editCurrentObject();
+    if (fwbdebug) qDebug("ObjectTreeView::editCurrentObject done");
 }
 
 void ObjectTreeView::keyPressEvent( QKeyEvent* ev )
@@ -684,8 +717,8 @@ void ObjectTreeView::keyPressEvent( QKeyEvent* ev )
         emit deleteObject_sign(obj);
         ev->accept();
         return;
-    }    
-    QListView::keyPressEvent(ev);
+    }
+    QTreeWidget::keyPressEvent(ev);
 }
 
 void ObjectTreeView::keyReleaseEvent( QKeyEvent* ev )
@@ -693,7 +726,7 @@ void ObjectTreeView::keyReleaseEvent( QKeyEvent* ev )
     if (fwbdebug)
         qDebug("ObjectTreeView::keyReleaseEvent");
 
-    QListView::keyReleaseEvent(ev);
+    QTreeWidget::keyReleaseEvent(ev);
 
     if (selectedObjects.size()==1)
         emit switchObjectInEditor_sign( getCurrentObject() );
@@ -701,22 +734,31 @@ void ObjectTreeView::keyReleaseEvent( QKeyEvent* ev )
     {
         // user selected multiple objects
         // do not let them if editor has unsaved changes
-        // 
-        if (oe->isVisible() && oe->isModified())
+        //
+        if (m_project->isEditorVisible() && m_project->isEditorModified())
             emit switchObjectInEditor_sign( getCurrentObject() );
         else
-            oe->blank();
+            m_project->blankEditor();
     }
 }
 
-void ObjectTreeView::returnPressed(QListViewItem *itm)
+/*void ObjectTreeView::keyPressEvent(QKeyEvent *ke)
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::returnPressed");
+    if (ke->key() == Qt::Key_Enter)
+    {
+        if (fwbdebug)
+            qDebug("ObjectTreeView::keyPressed");
 
+        editCurrentObject();
+    }
+
+    QTreeWidget::returnPressed(ke);
+}*/
+
+void ObjectTreeView::itemOpened ()
+{
+    if (fwbdebug) qDebug("ObjectTreeView::itemOpened");
     editCurrentObject();
-
-    QListView::returnPressed(itm);
 }
 
 void ObjectTreeView::clearLastSelected()
@@ -728,16 +770,16 @@ void ObjectTreeView::clearLastSelected()
 void ObjectTreeView::resetSelection()
 {
     if (fwbdebug)
-        qDebug(QString("ObjectTreeView::resetSelection :: lastSelected=%1").arg(lastSelected->text(0)));
+        qDebug(QString("ObjectTreeView::resetSelection :: lastSelected=%1").arg(lastSelected->text(0)).toAscii().constData());
 
     setCurrentItem(lastSelected);
-    setSelected( lastSelected , true);
+    lastSelected->setSelected(true);
 }
 
-void ObjectTreeView::selectionChanged()
+void ObjectTreeView::itemSelectionChanged()
 {
     if (fwbdebug)
-        qDebug("ObjectTreeView::selectionChanged selectionFrozen=%d",
+        qDebug("ObjectTreeView::itemSelectionChanged selectionFrozen=%d",
                selectionFrozen);
 
     if (selectionFrozen) return;
@@ -746,40 +788,49 @@ void ObjectTreeView::selectionChanged()
 
     selectedObjects.clear();
 
-    QListViewItemIterator it(this);
-    while ( it.current() )
+    QList<QTreeWidgetItem*> selected = selectedItems();
+    QList<QTreeWidgetItem*>::Iterator it;
+//    QTreeWidgetItemIterator it(this);
+//    while ( *it )
+    for (it=selected.begin(); it!=selected.end(); it++)
     {
-        if (it.current()->isSelected())
-        {
-            QListViewItem *itm= it.current();
-            ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+//        if ((*it)->isSelected())
+//        {
+            QTreeWidgetItem *itm = (*it);
+            ObjectTreeViewItem *otvi = dynamic_cast<ObjectTreeViewItem*>(itm);
+
             selectedObjects.push_back(otvi->getFWObject());
 
-            if (fwbdebug)
-                qDebug("ObjectTreeView::selectionChanged: selected otvi=%p object %s", otvi, otvi->getFWObject()->getName().c_str());
-        }
-        ++it;
+            if (fwbdebug) qDebug(
+                "ObjectTreeView::selectionChanged: selected otvi=%p object %s",
+                otvi, otvi->getFWObject()->getName().c_str());
+//        }
+//        ++it;
     }
     setLockFlags();
+
+    if (fwbdebug)
+        qDebug("ObjectTreeView::itemSelectionChanged completed");
 /* now list  selectedObjects   holds all selected items */
 }
 
 void ObjectTreeView::setLockFlags()
 {
-    QListViewItemIterator it(this);
+    QTreeWidgetItemIterator it(this);
     Lockable=false;
     Unlockable=false;
-    while ( it.current() )
+    while ( *it )
     {
-        if (it.current()->isSelected())
+        if ((*it)->isSelected())
         {
-            QListViewItem *itm= it.current();
+            QTreeWidgetItem *itm = *it;
             ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
 
             FWObject *lib = otvi->getFWObject()->getLibrary();
             // these lbraries are locked anyway, do not let the user
             // lock objects inside because they won't be able to unlock them.
-            if (lib->getId()!=STANDARD_LIB && lib->getId()!=TEMPLATE_LIB)
+            if (lib->getId()!=FWObjectDatabase::STANDARD_LIB_ID &&
+                lib->getId()!=FWObjectDatabase::TEMPLATE_LIB_ID)
             {
                 if (otvi->getFWObject()->getBool("ro"))  Unlockable=true;
                 else                                     Lockable=true;
@@ -804,3 +855,22 @@ int  ObjectTreeView::getNumSelected()
     return selectedObjects.size();
 }
 
+void ObjectTreeView::updateAfterPrefEdit()
+{
+     setFont(st->getTreeFont());
+}
+
+void ObjectTreeView::ExpandTreeItems(const set<int> &ids)
+{
+    QTreeWidgetItemIterator it(this);
+    while ( *it )
+    {
+        QTreeWidgetItem *itm = *it;
+        ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(itm);
+        FWObject *obj = otvi->getFWObject();
+        if (ids.count(obj->getId()))
+            itm->setExpanded(true);
+        ++it;
+    }
+}
+
