@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PrefsDialog.h,v 1.6 2006/05/08 01:26:18 vkurland Exp $
+  $Id: PrefsDialog.h 486 2008-08-24 00:18:49Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,21 +27,24 @@
 #ifndef __PREFSDIALOG_H_
 #define __PREFSDIALOG_H_
 
-#include "config.h"
-#include <prefsdialog_q.h>
+#include "../../config.h"
+#include <ui_prefsdialog_q.h>
 
 #include "FWBSettings.h"
 #include "listOfLibraries.h"
+#include "HttpGet.h"
 
 #include <list>
 #include <string>
 #include <map>
+#include <qfont.h>
 
 #include <qstring.h>
 
 class QPushButton;
 
-class PrefsDialog : public prefsDialog_q
+
+class PrefsDialog : public QDialog
 {
     Q_OBJECT
 
@@ -49,9 +52,20 @@ class PrefsDialog : public prefsDialog_q
     void changeColor(QPushButton *btn,FWBSettings::LabelColors colorCode);
 
     std::map<int,QString> colors;
+    Ui::prefsDialog_q *m_dialog;
+
+    HttpGet current_version_http_getter;
     
- public:
+    QFont rulesFont;
+    QFont treeFont;
+    QFont uiFont;
+    void changeFont(QFont &font);
+
+public:
     PrefsDialog(QWidget *parent);
+    ~PrefsDialog();
+
+    QString getFontDescription(const QFont &font);
     
 public slots:
     virtual void accept();
@@ -59,7 +73,7 @@ public slots:
     virtual void addLibrary();
     virtual void remLibrary();
     virtual void findSSH();
-    virtual void libClick(QListViewItem* itm, const QPoint &p, int col);
+    virtual void libClick(QTreeWidgetItem* itm, int col);
     virtual void changeRedColor();
     virtual void changeOrangeColor();
     virtual void changeYellowColor();
@@ -67,7 +81,13 @@ public slots:
     virtual void changeBlueColor();
     virtual void changePurpleColor();
     virtual void changeGrayColor();
-    
+    virtual void changeIconSize25();
+    virtual void changeIconSize16();
+    virtual void changeShowIcons();
+    virtual void changeRulesFont();
+    virtual void changeTreeFont();
+    virtual void checkSwUpdates();
+    virtual void checkForUpgrade(const QString&);
 };
 
 #endif // __PREFSDIALOG_H
