@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland vadim@fwbuilder.org
 
-  $Id: RCS.cpp,v 1.64 2008/03/10 03:42:20 vkurland Exp $
+  $Id: RCS.cpp 413 2008-07-25 05:40:18Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,15 +17,18 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
+#include "utils_no_qt.h"
+
+#include "RCS.h"
 
 // need this for FS_SEPARATOR
 #include "fwbuilder/libfwbuilder-config.h"
@@ -33,7 +36,6 @@
 
 //#include "FWWindow.h"
 
-#include <RCS.h>
 #include <qdir.h>
 #include <qregexp.h>
 #include <qmessagebox.h>
@@ -108,7 +110,7 @@ void Revision::operator=(const Revision &r)
     log        = r.log      ;
 }
 
-bool Revision::operator<(const Revision &r)
+bool Revision::operator<(const Revision &r) const
 {
     for(int i=1; ; i++)
     {
@@ -120,17 +122,17 @@ bool Revision::operator<(const Revision &r)
         if (v1!="" && v2=="") return false;
         if (v1.toInt()>v2.toInt()) return false;
         if (v1.toInt()<v2.toInt()) return true;
-        i++;
+//        i++;
     }
     return true;
 }
 
-bool Revision::operator==(const Revision &r)
+bool Revision::operator==(const Revision &r) const
 {
     return rev==r.rev;
 }
 
-bool Revision::operator!=(const Revision &r)
+bool Revision::operator!=(const Revision &r) const
 {
     return rev!=r.rev;
 }
@@ -145,9 +147,9 @@ RCSEnvFix::RCSEnvFix()
     int tzoffset;
     QString tzsign = "";
 
-    time_t clock; 
-    struct tm *ltm; 
-    time(&clock); 
+    time_t clock;
+    struct tm *ltm;
+    time(&clock);
     ltm = (struct tm *) localtime(&clock);
 
 #if defined(HAVE_STRUCT_TM_TM_ZONE)
@@ -170,7 +172,7 @@ RCSEnvFix::RCSEnvFix()
     } else {
         tzsign = "+";
     }
-    
+
 #else
 // global variable timezone has seconds West of GMT (positive in
 // timezones west of GMT)
@@ -190,7 +192,7 @@ RCSEnvFix::RCSEnvFix()
     TZOffset = tzsign + TZOffset;
 
     if (fwbdebug)
-	qDebug("tzoffset: %d TZOffset: '%s'",tzoffset,TZOffset.ascii());
+	qDebug("tzoffset: %d TZOffset: '%s'",tzoffset,TZOffset.toAscii().constData());
 
 #ifdef _WIN32
 /* need this crap because Windows does not set environment variable TZ
@@ -209,7 +211,8 @@ RCSEnvFix::RCSEnvFix()
  * everything works. When the program is started with some other
  * directory as current dir, RCS tools fail without any error message.
  */
-    env.push_back( QString("PATH=%1;%2").arg(appRootDir.c_str()).arg(getenv("PATH")) );
+    env.push_back(
+        QString("PATH=%1;%2").arg(getPathToBinary("").c_str()).arg(getenv("PATH")) );
 #endif
 
 /* also need to set env variable USER for rcs tools, but if the user name
@@ -244,19 +247,19 @@ RCS::RCS(const QString &file)
     {
 #ifdef _WIN32
         string ts;
-        ts     = appRootDir+FS_SEPARATOR+RCS_FILE_NAME      ;
+        ts = getPathToBinary(RCS_FILE_NAME);
         rcs_file_name     = ts.c_str();
 
-        ts    = appRootDir+FS_SEPARATOR+RLOG_FILE_NAME     ;
+        ts = getPathToBinary(RLOG_FILE_NAME);
         rlog_file_name    = ts.c_str();
 
-        ts = appRootDir+FS_SEPARATOR+RCSDIFF_FILE_NAME  ;
+        ts = getPathToBinary(RCSDIFF_FILE_NAME);
         rcsdiff_file_name = ts.c_str();
 
-        ts      = appRootDir+FS_SEPARATOR+CI_FILE_NAME       ;
+        ts = getPathToBinary(CI_FILE_NAME);
         ci_file_name      = ts.c_str();
 
-        ts      = appRootDir+FS_SEPARATOR+CO_FILE_NAME       ;
+        ts = getPathToBinary(CO_FILE_NAME);
         co_file_name      = ts.c_str();
 #else
         rcs_file_name     = RCS_FILE_NAME      ;
@@ -277,15 +280,14 @@ RCS::RCS(const QString &file)
 
     ciproc = new QProcess();
     proc = new QProcess();
-    proc->setCommunication( QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
 
-    connect(proc, SIGNAL(readyReadStdout()), this,  SLOT(readFromStdout() ) );
-    connect(proc, SIGNAL(readyReadStderr()), this,  SLOT(readFromStderr() ) );
+    connect(proc, SIGNAL(readyReadStandardOutput()), this,  SLOT(readFromStdout() ) );
+    connect(proc, SIGNAL(readyReadStandardError()), this,  SLOT(readFromStderr() ) );
 
 
     try
     {
-        QString  rcspath=filename.left( filename.findRev("/") );
+        QString  rcspath=filename.left( filename.lastIndexOf("/") );
         QDir     rcsdir;
         rcsdir.cd(rcspath);
 
@@ -293,12 +295,13 @@ RCS::RCS(const QString &file)
  * rlog is started with environment defined by RCSEnvFix, which does
  * not have env. var LANG so it always runs in english
  */
-        QStringList split_log = QStringList::split(QRegExp("------|======"),rlog());
-                                                   
+        QString rl = rlog();
+        QStringList split_log = rl.split(QRegExp("------|======"));
+
         QString head_section = split_log[0];
 
         QRegExp head_rx("head:\\s+([0-9\\.]+)\\s*\\n");
-        int pos = head_rx.search( head_section );
+        int pos = head_rx.indexIn( head_section );
         if (pos>-1) head = head_rx.cap(1);
 
         QStringList::iterator i;
@@ -314,7 +317,7 @@ RCS::RCS(const QString &file)
             r.log = "";
 
             QRegExp rev_rx("revision\\s+([0-9\\.]+)");
-            match = rev_rx.search( section );
+            match = rev_rx.indexIn( section );
             if (match>-1)
             {
                 r.rev = rev_rx.cap(1);
@@ -322,7 +325,7 @@ RCS::RCS(const QString &file)
 
             QRegExp lock_rx("revision\\s+([0-9\\.]+)\\s+locked by:\\s+(\\S+);");
             lock_rx.setMinimal(true);
-            match = lock_rx.search( section );
+            match = lock_rx.indexIn( section );
             if (match>-1)
             {
                 r.locked_by = lock_rx.cap(2);
@@ -335,7 +338,7 @@ RCS::RCS(const QString &file)
 // we'll do the same here to maintain compatibility
             QRegExp rev2_rx("(revision.+)\\n");
             rev2_rx.setMinimal(true);
-            match = rev2_rx.search( section );
+            match = rev2_rx.indexIn( section );
             if (match>-1)
             {
                 r.log += rev2_rx.cap(1) + "\n";
@@ -344,7 +347,7 @@ RCS::RCS(const QString &file)
 
             QRegExp date_rx("date:\\s+([^;]+);\\s+author:\\s+(\\S+);");
             date_rx.setMinimal(true);
-            match = date_rx.search( section );
+            match = date_rx.indexIn( section );
             if (match>-1)
             {
                 r.date   = date_rx.cap(1);
@@ -353,7 +356,7 @@ RCS::RCS(const QString &file)
 
             QRegExp log_rx("date:.*\\n(.*)$");
             log_rx.setMinimal(true);
-            match = log_rx.search( section );
+            match = log_rx.indexIn( section );
             if (match>-1)
                 r.log += log_rx.cap(1);
 
@@ -362,62 +365,15 @@ RCS::RCS(const QString &file)
             if (r.rev != "")
             {
                 revisions.push_back(r);
-                if (fwbdebug) qDebug("revision %s: '%s'",r.rev.ascii(),r.log.ascii());
-            }
-
-        }
-
-#if 0
-        for ()
-        {
-            if ( (*i).find("head: ")==0)
-            {
-                head=(*i).section(QRegExp("\\s+"),1,1);
-                continue;
-            }
-
-            if ( (*i).find(QRegExp("^=========="))==0 ) break;
-
-            if ((*i).find(QRegExp("^revision\\s+[0-9\\.]+"))==0)
-            {
-                if (fwbdebug) qDebug("revision '%s'",(*i).ascii());
-
-                Revision r(filename);
-
-                r.rev       = (*i).section(QRegExp("\\s+"),1,1);
-                QString lb  = (*i).section(QRegExp("[\\s;]+"),4,4);
-                if (lb!="")
-                {       
-                    r.locked_by = lb;
-                    locked      = true;
-                    locked_by   = lb;
-                    locked_rev  = r.rev;
-                }
-
-                r.log="";
-
-                for ( ;   (*i).find(QRegExp("^=========="))==-1 &&
-                          (*i).find(QRegExp("^----------"))==-1 &&
-                          i!=rcslog.end(); ++i )
-                {
-                    if ((*i).find(QRegExp("^date:.*author:.*state:"))==0)
-                    {
-                        r.date      = (*i).section(QRegExp("[\\s;]+"),1,2);
-                        r.author    = QString("    ")+(*i).section(QRegExp("[\\s;]+"),4,4);
-                        continue;
-                    }
-                    if ((*i).find(QRegExp("^branches:"))==0) continue;
-                    r.log= r.log + *i + "\n";
-                }
-                r.log.replace('\r',"");
-                revisions.push_back(r);
-                if (fwbdebug) qDebug("revision %s: '%s'",r.rev.ascii(),r.log.ascii());
-
-                if (i==rcslog.end()) break;
+                if (fwbdebug)
+                    qDebug("revision %s: '%s'",
+                           r.rev.toAscii().constData(),
+                           r.log.toAscii().constData());
             }
         }
-//        qBubbleSort( revisions.begin() , revisions.end() );
-#endif
+        // sort list revisions; its defined like this:
+        // QList<Revision> revisions
+        qSort(revisions);
 
         inrcs         = true;
         tracking_file = true;
@@ -450,12 +406,22 @@ RCSEnvFix*   RCS::getRCSEnvFix()
 
 void RCS::readFromStdout()
 {
-    stdoutBuffer=stdoutBuffer + QString(proc->readStdout());
+    QString s = QString(proc->readAllStandardOutput());
+    //qDebug("RCS::readFromStdout() reads: %s",s.toAscii().constData());
+    stdoutBuffer=stdoutBuffer + s;
 }
 
 void RCS::readFromStderr()
 {
-    stderrBuffer=stderrBuffer + QString(proc->readStderr());
+    QString s = QString(proc->readAllStandardError());
+    //qDebug("RCS::readFromStderr() reads: %s", s.toAscii().constData());
+    stderrBuffer=stderrBuffer + s;
+}
+
+void RCS::setFileName(const QString &fn)
+{
+    filename=fn;
+    if (fwbdebug)  qDebug("RCS::setFileName fn = %s",fn.toAscii().constData());
 }
 
 /*********************************************************************
@@ -467,26 +433,31 @@ void RCS::abandon()
     if (!isInRCS()) return;
 
 /* check out head revision and unlock it */
-    proc->clearArguments();
+    QStringList arglist;
 
-    proc->addArgument( co_file_name );
-    proc->addArgument( "-q" );
-    proc->addArgument( "-f" );
-    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-    proc->addArgument( QString("-u") );
-    proc->addArgument( filename );
+    arglist << "-q" << "-f" << QString("-z") + rcsenvfix->getTZOffset() << QString("-u") << filename ;
 
     stdoutBuffer="";
     stderrBuffer="";
 
     if (fwbdebug) qDebug("starting co with environment '%s'",
-                         rcsenvfix->getEnv()->join("\n").ascii());
+                         rcsenvfix->getEnv()->join(" ").toAscii().constData());
+    if (fwbdebug) qDebug("executing command '%s %s'",
+                         co_file_name.toAscii().constData(),
+                         arglist.join(" ").toAscii().constData());
 
-    if (proc->start( rcsenvfix->getEnv() ) )
+    proc->setEnvironment(*rcsenvfix->getEnv());
+    proc->start( co_file_name, arglist );
+    proc->waitForStarted();
+
+    if (fwbdebug) qDebug("running co");
+
+    if (proc->state() == QProcess::Running)
     {
-        while (proc->isRunning()) ; // cxx_sleep(1);
-        if (proc->normalExit() && proc->exitStatus()==0)
+        proc->waitForFinished();
+        if (proc->exitCode() == 0 && proc->state() == QProcess::NotRunning)
         {
+            if (fwbdebug) qDebug("finished successfully");
             checked_out = false;
             locked      = false;
             selectedRev = head;
@@ -498,11 +469,11 @@ void RCS::abandon()
     selectedRev = "";
 
     checked_out=false;
-    
+
     QString err = tr("Error checking file out: %1").arg(stderrBuffer);
     QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
 
-    throw(FWException(err.latin1()));
+    throw(FWException(err.toLatin1().constData()));
 }
 
 /**
@@ -510,48 +481,44 @@ void RCS::abandon()
  */
 void RCS::add() throw(libfwbuilder::FWException)
 {
-    proc->clearArguments();
-
-    int      i=filename.findRev("/");
+    int      i=filename.lastIndexOf("/");
     QString  rcspath=filename.left(i);
     QDir     rcsdir;
     rcsdir.cd(rcspath);
 
     if (!rcsdir.exists("RCS")) rcsdir.mkdir("RCS");
 
-    proc->addArgument( rcs_file_name );
-    proc->addArgument( "-q" );
-    proc->addArgument( "-i" );
-    proc->addArgument( "-kb" );
-    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-    proc->addArgument( "-t-\"Initial checkin\"" );
-    proc->addArgument( filename );
+    QStringList arglist;
+
+    arglist << "-q" << "-i" << "-kb" << QString("-z") + rcsenvfix->getTZOffset() << "-t-\"Initial checkin\"" << filename;
 
     stdoutBuffer="";
     stderrBuffer="";
 
-    if (proc->start( rcsenvfix->getEnv() ) )
+    proc->setEnvironment(*rcsenvfix->getEnv());
+    proc->start( rcs_file_name, arglist );
+    proc->waitForStarted();
+
+    if (proc->state() == QProcess::Running)
     {
-        while (proc->isRunning()) ; // cxx_sleep(1);
-        if (proc->normalExit() && proc->exitStatus()==0)
+        proc->waitForFinished();
+        if (proc->state() == QProcess::NotRunning && proc->exitCode()==0)
         {
-            proc->clearArguments();
-            proc->addArgument( ci_file_name );
-            proc->addArgument( "-q" );
-            proc->addArgument( "-u" );
-            proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-            proc->addArgument( filename );
+            arglist.clear();
+
+            arglist << "-q" << "-u" << QString("-z") + rcsenvfix->getTZOffset() << filename;
 
             stdoutBuffer="";
             stderrBuffer="";
 
-            if (fwbdebug) qDebug("starting ci with environment '%s'",
-                                 rcsenvfix->getEnv()->join("\n").ascii());
+            proc->setEnvironment(*rcsenvfix->getEnv());
+            proc->start( ci_file_name, arglist );
+            proc->waitForStarted();
 
-            if (proc->start( rcsenvfix->getEnv() ) )
+            if (proc->state() == QProcess::Running)
             {
-                while (proc->isRunning()) ; // cxx_sleep(1);
-                if (proc->normalExit() && proc->exitStatus()==0)
+                proc->waitForFinished();
+                if (proc->state() == QProcess::NotRunning && proc->exitCode()==0)
                 {
                     inrcs       = true;
                     selectedRev = "1.1";
@@ -561,37 +528,39 @@ void RCS::add() throw(libfwbuilder::FWException)
             }
         }
     }
-    QByteArray outp = proc->readStdout();
+    QByteArray outp = proc->readAllStandardOutput();
     QString msg=QObject::tr("Fatal error during initial RCS checkin of file %1 :\n %2\nExit status %3")
-	.arg(filename).arg(outp.data()).arg(proc->exitStatus());
-    throw(FWException( msg.latin1()  ));
+	.arg(filename).arg(outp.data()).arg(proc->exitCode());
+    throw(FWException( msg.toLatin1().constData()  ));
 }
 
 bool RCS::isInRCS()
 {
     if (tracking_file) return inrcs;
 
-    proc->clearArguments();
+    QStringList arglist;
 
-    proc->addArgument( rlog_file_name );
-    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-    proc->addArgument( "-R" );
-    proc->addArgument( filename );
+    arglist << QString("-z") + rcsenvfix->getTZOffset() << "-R" << filename;
 
     stdoutBuffer="";
     stderrBuffer="";
 
-    if (!proc->start( rcsenvfix->getEnv() ) ) throw(FWException("Fatal error running rlog "));
+    proc->setEnvironment(*rcsenvfix->getEnv());
 
-    while (proc->isRunning()) ; // cxx_sleep(1);
+    proc->start( rlog_file_name, arglist );
+    proc->waitForStarted();
+    if (proc->state() != QProcess::Running)
+        throw(FWException("Fatal error running rlog "));
 
-    if (proc->normalExit() && proc->exitStatus()==1)
+    while (proc->state() == QProcess::Running) ; // cxx_sleep(1);
+
+    if (proc->state() == QProcess::NotRunning && proc->exitCode()==1)
     {
 /* exist status '1' means the file is not in RCS */
         inrcs=false;
 	if (fwbdebug)
 	{
-          QByteArray outp = proc->readStdout();
+          QByteArray outp = proc->readAllStandardOutput();
           qDebug("Error running rlog: %s",outp.data());
 	}
         return false;
@@ -643,70 +612,70 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
     {
         if (rev==head || rev=="") return true;
 
-/* check out requested revision to stdout 
+/* check out requested revision to stdout
  *
  * TODO: right now it loads the whole file into memory, then writes it
  * to the temp file. It should be more efficient to read and write in
  * chunks.
  *
  */
-        proc->clearArguments();
+        QStringList arglist;
 
-        proc->addArgument( co_file_name );
-        proc->addArgument( QString("-q") );
-        proc->addArgument( QString("-kb") );
-        proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-        proc->addArgument( QString("-p")+rev );
-        proc->addArgument( filename );
+        arglist << QString("-q") << QString("-kb") <<
+                QString("-z") + rcsenvfix->getTZOffset() << QString("-p")+rev << filename;
 
         stdoutBuffer="";
         stderrBuffer="";
 
         if (fwbdebug) qDebug("starting co with environment '%s'",
-                             rcsenvfix->getEnv()->join("\n").ascii());
+                             rcsenvfix->getEnv()->join("\n").toAscii().constData());
+        if (fwbdebug) qDebug("executing command '%s %s'",
+                         co_file_name.toAscii().constData(),
+                         arglist.join(" ").toAscii().constData());
 
-        if (proc->start( rcsenvfix->getEnv() ) )
+        proc->setEnvironment(*rcsenvfix->getEnv());
+        proc->start( co_file_name, arglist );
+        proc->waitForStarted();
+
+        if (fwbdebug) qDebug("running co");
+
+        if (proc->state() == QProcess::Running)
         {
-            while (proc->isRunning())
-            {
-                QByteArray ba = proc->readStdout();
-                if (ba.size()!=0)  stdoutBuffer=stdoutBuffer + QString(ba);
-//                cerr << "read " << ba.size() << " bytes from stdout" << endl;
-//                cxx_sleep(1);
-            }
-            if (proc->normalExit() && proc->exitStatus()==0)
-            {
+            proc->waitForFinished();
 
+            if (proc->state() == QProcess::NotRunning && proc->exitCode()==0)
+            {
+                if (fwbdebug) qDebug("finished successfully");
 #ifdef _WIN32
                 char tname[1024];
-                strncpy(tname, filename.left(filename.findRev("/")+1).latin1(),sizeof(tname)-20);
+                strncpy(tname, filename.left(filename.lastIndexOf("/")+1).toLatin1().constData(),sizeof(tname)-20);
                 strcat(tname,"tmpXXXXXX");
                 _mktemp(tname);
                 int fd = _open(tname, _O_RDWR|_O_CREAT|_O_EXCL|_O_BINARY , _S_IREAD|_S_IWRITE );
 #else
                 char tname[PATH_MAX];
-                strncpy(tname, filename.latin1(), sizeof(tname)-20 );
+                strncpy(tname, filename.toLatin1().constData(), sizeof(tname)-20 );
                 strcat(tname,"_temp_XXXXXX");
                 int fd = mkstemp(tname);
-#endif 
+#endif
 		if (fd<0)
 		{
                     QString err = tr("Error creating temporary file ")+tname+QString(" :\n")+strerror(errno);
                     QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
-                    throw(FWException(err.latin1()));
+                    throw(FWException(err.toLatin1().constData()));
 		}
 #ifdef _WIN32
-                if (_write(fd,stdoutBuffer.latin1(),stdoutBuffer.length() )<0)
+                if (_write(fd,stdoutBuffer.toLatin1().constData(),stdoutBuffer.length() )<0)
 		{
 		    _close(fd);
 #else
-                if ( write(fd,stdoutBuffer.latin1(),stdoutBuffer.length() )<0)
+                if ( write(fd,stdoutBuffer.toLatin1().constData(),stdoutBuffer.length() )<0)
 		{
 		    close(fd);
 #endif
                     QString err = tr("Error writing to temporary file ")+tname+QString(" :\n")+strerror(errno);
                     QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
-                    throw(FWException(err.latin1()));
+                    throw(FWException(err.toLatin1().constData()));
                 }
                 close(fd);
 
@@ -723,7 +692,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
 
         QString err = tr("Error checking file out: %1").arg(stderrBuffer);
         QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
-        throw(FWException(err.latin1()));
+        throw(FWException(err.toLatin1().constData()));
 
     } else
     {
@@ -735,7 +704,7 @@ bool RCS::co(const QString &rev,bool force) throw(libfwbuilder::FWException)
             if (me!=locked_by)
             {
                 switch (QMessageBox::warning(
-                            app->activeWindow(),"Firewall Builder", 
+                            app->activeWindow(),"Firewall Builder",
                             tr("File is opened and locked by %1.\nYou can only open it read-only.")
                             .arg(locked_by),
                             "Open &read-only", "&Cancel", QString::null,
@@ -774,26 +743,34 @@ after the program crashed.").arg(locked_rev),
  checkout:
 
 /* check out and lock */
-        proc->clearArguments();
+        QStringList arglist;
+         arglist.clear();
 
-        proc->addArgument( co_file_name );
-        proc->addArgument( "-q" );
-        if (force)  proc->addArgument( "-f" );
-        proc->addArgument( QString("-l")+rev );
-        proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-        proc->addArgument( filename );
+        arglist << "-q";
+        if (force)  arglist << "-f";
+        arglist << QString("-l")+rev << QString("-z") + rcsenvfix->getTZOffset() << filename;
 
         stdoutBuffer="";
         stderrBuffer="";
 
         if (fwbdebug) qDebug("starting co with environment '%s'",
-                             rcsenvfix->getEnv()->join("\n").ascii());
+                             rcsenvfix->getEnv()->join("\n").toAscii().constData());
+        if (fwbdebug) qDebug("executing command '%s %s'",
+                         co_file_name.toAscii().constData(),
+                         arglist.join(" ").toAscii().constData());
 
-        if (proc->start( rcsenvfix->getEnv() ) )
+        proc->setEnvironment(*rcsenvfix->getEnv());
+        proc->start( co_file_name, arglist );
+        proc->waitForStarted();
+
+        if (fwbdebug) qDebug("running co");
+
+        if (proc->state() == QProcess::Running)
         {
-            while (proc->isRunning()) ; // cxx_sleep(1);
-            if (proc->normalExit() && proc->exitStatus()==0)
+            proc->waitForFinished();
+            if (proc->state() == QProcess::NotRunning && proc->exitCode()==0)
             {
+                if (fwbdebug) qDebug("finished successfully");
                 checked_out = true;
                 locked      = true;
                 selectedRev = rev;
@@ -807,7 +784,7 @@ after the program crashed.").arg(locked_rev),
         QString err = tr("Error checking file out: %1").arg(stderrBuffer);
         QMessageBox::critical(app->activeWindow(), "Firewall Builder", err, tr("&Continue") );
 
-        throw(FWException(err.latin1()));
+        throw(FWException(err.toLatin1().constData()));
     }
     return false;
 }
@@ -825,31 +802,25 @@ bool RCS::ci( const QString &_lm,
 
     if (fwbdebug)
         qDebug("RCS::ci  log message (%d characters): '%s'",
-               logmsg.length(), logmsg.ascii());
+               logmsg.length(), logmsg.toAscii().constData());
 
-    ciproc->setCommunication( QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
+    QStringList arglist;
 
-    ciproc->clearArguments();
-
-    ciproc->addArgument( ci_file_name );
-    if (unlock) ciproc->addArgument( "-u" );
-    else        ciproc->addArgument( "-l" );
-    ciproc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-
-    ciproc->addArgument( filename );
+    if (unlock) arglist << "-u";
+    else        arglist << "-l";
+    arglist << QString("-z") + rcsenvfix->getTZOffset();
+    arglist << filename;
 
     stdoutBuffer="";
     stderrBuffer="";
 
     if (fwbdebug) qDebug("starting ci with environment '%s'",
-                         rcsenvfix->getEnv()->join("\n").ascii());
+                         rcsenvfix->getEnv()->join("\n").toAscii().constData());
 
-    QCString rcslog = logmsg.utf8();
+    QByteArray rcslog = logmsg.toUtf8();
 
     QString obuf;
 
-    connect(ciproc, SIGNAL(processExited()), this,  SLOT(completeCI() ) );
-    connect(ciproc, SIGNAL(wroteToStdin()),  this,  SLOT(completeCI() ) );
 
 /*
  * under some circumstances, ci may exit immediately (e.g. when there
@@ -862,47 +833,38 @@ bool RCS::ci( const QString &_lm,
  * Also it seems on windows all data is sent to the process and slot
  * is called while we still are inside launch, so that once we exit
  * from it, all is done and there is no need to enter event loop.
- */    
+ */
     ciRunning=true;
-    if (!ciproc->start(rcsenvfix->getEnv() ) )
-    {
+    ciproc->setEnvironment(*rcsenvfix->getEnv());
+    ciproc->start( ci_file_name, arglist );
+    ciproc->waitForStarted();
+    if (ciproc->state() != QProcess::Running)
+    {//if not started
         if (fwbdebug) qDebug("Checkin error: file=%s error=%s",
-                             filename.latin1(),obuf.latin1());
+                             filename.toLatin1().constData(),obuf.toLatin1().constData());
 
         throw( FWException( (obuf+"\n"+
-                             ciproc->arguments().join(" ")+"\n"+
-                             rcsenvfix->getEnv()->join("\n")).ascii() ) );
+                             arglist.join(" ")+"\n"+
+                             rcsenvfix->getEnv()->join("\n")).toAscii().constData() ) );
     }
 
 /* make a copy, omitting trailing '\0' so it won't get sent to ci */
     QByteArray rcslogCopy;
-    rcslogCopy.duplicate((const char*)(rcslog),rcslog.length());
+    rcslogCopy = rcslog;
 
-    ciproc->writeToStdin(rcslogCopy);
-    ciproc->writeToStdin("\n.\n");
+    ciproc->write((const char*)rcslogCopy, rcslog.length());
 
-    if (fwbdebug) qDebug("launch succeeded, entering event loop");
-
-    if (ciRunning)
-        QApplication::eventLoop()->enterLoop();
+    QByteArray arr;
+    arr = "\n.\n";
+    ciproc->write((const char*)(arr),arr.length());
 
     if (fwbdebug) qDebug("all data sent to ci");
-    do
-    {
-        cxx_sleep(1);
-        QString s=QString(ciproc->readStdout());
-        obuf = obuf + s;
-        if (fwbdebug) qDebug("stdoutBuffer='%s'",obuf.ascii());
-    } while (ciproc->isRunning());
 
-    if (fwbdebug) qDebug("all data collected from stdout/stderr");
-
-    while (ciproc->isRunning())
-        cxx_sleep(1);
+    ciproc->waitForFinished();
 
     if (fwbdebug) qDebug("ci exited");
 
-    if (ciproc->normalExit() && ciproc->exitStatus()==0)
+    if (ciproc->state() == QProcess::NotRunning && ciproc->exitCode()==0)
     {
         if (fwbdebug) qDebug("ci exited normally");
         if (unlock)
@@ -912,21 +874,10 @@ bool RCS::ci( const QString &_lm,
         }
         return true;
     }
-        
+
     return true;
 }
 
-void RCS::completeCI()
-{
-    ciRunning=false;
-
-    int loopLevel = QApplication::eventLoop()->loopLevel();
-
-    if (fwbdebug) qDebug("completeCI: loop level %d",loopLevel);
-
-    if (loopLevel>1) QApplication::eventLoop()->exitLoop();
-}
-
 /**
  * rlog - run rlog in the background and collect RCS log
  *
@@ -943,100 +894,83 @@ void RCS::completeCI()
  */
 QString RCS::rlog() throw(libfwbuilder::FWException)
 {
-    proc->clearArguments();
+    QStringList arglist;
 
-    proc->addArgument( rlog_file_name );
-    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
+    arglist << QString("-z") + rcsenvfix->getTZOffset() << filename;
 //    proc->addArgument( "-zLT" );
-    proc->addArgument( filename );
 
     if (fwbdebug)
-	qDebug("Running rlog:  %s",proc->arguments().join(" ").ascii());
+        qDebug("Running rlog: %s %s",rlog_file_name.toAscii().constData(),arglist.join(" ").toAscii().constData());
 
     stdoutBuffer="";
     stderrBuffer="";
 
-    if (!proc->start( rcsenvfix->getEnv() ) )
+    //proc->setEnvironment(*rcsenvfix->getEnv());
+    proc->start( rlog_file_name, arglist );
+    proc->waitForStarted();
+
+    if (proc->state() != QProcess::Running)
         throw(FWException("Fatal error running rlog "));
 
     if (fwbdebug) qDebug("Running rlog");
 
-    while (proc->isRunning()) 
-    {
-	QByteArray ba = proc->readStdout();
-	if (ba.size()!=0)
-        {
-            /* I've discovered that sometimes log records contain zero
-             * characters; this may be because of incorrect encoding
-             * while log records were created.
-             */
-            unsigned int pos=0;
-            int pp=-1;
-            while (pos<ba.size() && (pp=ba.find('\0',pos))!=-1)
-            {
-                pos=pp;
-                ba.at(pos)='\n';
-                ++pos;
-            }
+    proc->waitForFinished();
 
-            ba.at(ba.size()-1)='\0';
+    if (fwbdebug) qDebug("Running rlog: finished reading");
 
-            stdoutBuffer=stdoutBuffer + QString(ba);
+    QString rlogTxt = QString::fromUtf8(stdoutBuffer.toAscii().constData());
 
-            if (fwbdebug)
-                qDebug("******** Reading rlog: bufsize=%d, %s",
-                       ba.size(),(const char*)(QCString(ba)));
-
-        }
-    }
-
-    QString rlogTxt = QString::fromUtf8(stdoutBuffer.ascii());
-
-    if (proc->normalExit() && proc->exitStatus()==0)
+    if (proc->state() == QProcess::NotRunning && proc->exitCode()==0)
         return rlogTxt;
 
     QString msg=QObject::tr("Fatal error running rlog for %1").arg(filename);
-    throw( FWException( msg.latin1() ) );
+    throw( FWException( msg.toLatin1().constData() ) );
 }
 
-QStringList RCS::rcsdiff(const QString &rev) throw(libfwbuilder::FWException)
+QStringList RCS::rcsdiff(const QString&) throw(libfwbuilder::FWException)
 {
     isDiff();
-    return QStringList::split("\n",stdoutBuffer );
+    QString temp = stdoutBuffer;
+    return temp.split("\n");
 }
 
 bool        RCS::isDiff(const QString &rev) throw(libfwbuilder::FWException)
 {
-    proc->clearArguments();
+    QStringList arglist;
 
-    proc->addArgument( rcsdiff_file_name );
-    proc->addArgument( "-q" );
-    if (rev!="") proc->addArgument( QString("-r")+rev );
+    arglist << "-q";
+    if (rev!="") arglist << QString("-r")+rev;
     else
     {
-        if (selectedRev!="") proc->addArgument( QString("-r")+selectedRev );
+        if (selectedRev!="") arglist << QString("-r")+selectedRev;
     }
-    proc->addArgument( QString("-z") + rcsenvfix->getTZOffset() );
-    proc->addArgument( filename );
+    arglist << QString("-z") + rcsenvfix->getTZOffset() << filename;
 
     stdoutBuffer="";
     stderrBuffer="";
 
-    if (proc->start( rcsenvfix->getEnv() ) ) 
+    proc->setEnvironment(*rcsenvfix->getEnv());
+    proc->start( rcsdiff_file_name, arglist );
+    proc->waitForStarted();
+
+
+    if (proc->state() == QProcess::Running)
     {
-        while (proc->isRunning())
+        proc->waitForFinished();
+        /*while (proc->state() == QProcess::Running)
         {
-            QByteArray ba = proc->readStdout();
+            QByteArray ba = proc->readAllStandardOutput();
             if (ba.size()!=0)  stdoutBuffer=stdoutBuffer + QString(ba);
-        }
+        }*/
     } else
         throw(FWException("Fatal error running rcsdiff "));
 
-//    while (proc->isRunning()) ; // cxx_sleep(1);
+//    while (proc->state() == QProcess::Running) ; // cxx_sleep(1);
 
-    if (proc->normalExit()) return (proc->exitStatus()!=0);
-    QString msg=QObject::tr("Fatal error running rcsdiff for file %1").arg(filename);
-    throw( FWException( msg.latin1() ) );
+    if (proc->state() == QProcess::NotRunning) return (proc->exitCode()!=0);
+    QString msg = QObject::tr(
+        "Fatal error running rcsdiff for file %1").arg(filename);
+    throw( FWException( msg.toLatin1().constData() ) );
 }
 
 QString RCS::getHead()
