@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHPIX.cpp,v 1.23 2007/07/13 05:32:55 vkurland Exp $
+  $Id: SSHPIX.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,7 +17,7 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
@@ -25,7 +25,7 @@
 
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -43,7 +43,7 @@
 #include <errno.h>
 #ifndef errno
 extern int errno;
-#endif 
+#endif
 
 using namespace std;
 
@@ -103,8 +103,8 @@ QString SSHPIX::cmd(QProcess *proc,const QString &cmd)
 {
     stdoutBuffer="";
 
-    proc->writeToStdin( cmd + "\n"  );
-//    proc->writeToStdin( "\n" );
+    proc->write( (cmd + "\n").toAscii() );
+//    proc->write( "\n" );
 
     state=EXECUTING_COMMAND;
     qApp->processEvents();
@@ -128,10 +128,10 @@ bool SSHPIX::checkForErrors()
     for (QStringList::const_iterator i=errptr->begin();
          i!=errptr->end(); ++i)
     {
-        if ( stdoutBuffer.findRev(*i,-1)!=-1 )
+        if ( stdoutBuffer.lastIndexOf(*i,-1)!=-1 )
         {
             if (fwbdebug)
-                qDebug(QString("Got known error message: %1").arg(stdoutBuffer));
+                qDebug(QString("Got known error message: %1").arg(stdoutBuffer).toAscii().constData());
 
             emit printStdout_sign( tr("\n*** Fatal error :") );
             emit printStdout_sign( stdoutBuffer+"\n" );
@@ -155,8 +155,8 @@ void SSHPIX::stateMachine()
         if ( cmpPrompt(stdoutBuffer,QRegExp(pwd_prompt)) )
         {
             stdoutBuffer="";
-            proc->writeToStdin( pwd + "\n" );
-            // proc->writeToStdin( "\n" );
+            proc->write( (pwd + "\n").toAscii() );
+            // proc->write( "\n" );
             break;
         }
 /* we may get to LOGGEDIN state directly from NONE, for example when
@@ -170,29 +170,29 @@ void SSHPIX::stateMachine()
             emit printStdout_sign( tr("Logged in") + "\n" );
             emit printStdout_sign( tr("Switching to enable mode...") + "\n");
             stdoutBuffer="";
-            proc->writeToStdin( "enable\n" );
-            //proc->writeToStdin( "\n" );
+            proc->write( "enable\n" );
+            //proc->write( "\n" );
         }
 
 /* we may even get straight to the enable prompt, e.g. if
  * user account is configured with "privilege 15"
  */
         if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
-        {       
+        {
             state=WAITING_FOR_ENABLE;
             stateMachine();
             break;
         }
 
         QString fingerprint;
-        int n1,n2;
-        if (stdoutBuffer.find(newKeyOpenSSH)!=-1 ||
-            stdoutBuffer.find(newKeyPlink)!=-1   ||
-            stdoutBuffer.find(newKeySSHComm)!=-1)
+        //int n1,n2;
+        if (stdoutBuffer.indexOf(newKeyOpenSSH)!=-1 ||
+            stdoutBuffer.indexOf(newKeyPlink)!=-1   ||
+            stdoutBuffer.indexOf(newKeySSHComm)!=-1)
         {
 /* new key */
-            bool unix_y_n = (stdoutBuffer.find(newKeyOpenSSH)!=-1 ||
-                             stdoutBuffer.find(newKeySSHComm)!=-1);
+            bool unix_y_n = (stdoutBuffer.indexOf(newKeyOpenSSH)!=-1 ||
+                             stdoutBuffer.indexOf(newKeySSHComm)!=-1);
 
             if (fwbdebug) qDebug("New host key message detected");
 
@@ -214,8 +214,8 @@ void SSHPIX::stateMachine()
             stdoutBuffer="";
             if (res==0)
             {
-                if (unix_y_n) proc->writeToStdin( "yes\n" );
-                else          proc->writeToStdin( "y\n" );
+                if (unix_y_n) proc->write( "yes\n" );
+                else          proc->write( "y\n" );
                 break;
             } else
             {
@@ -232,8 +232,8 @@ void SSHPIX::stateMachine()
         if ( cmpPrompt(stdoutBuffer,QRegExp(epwd_prompt)) )
         {
             stdoutBuffer="";
-            if (!epwd.isEmpty()) proc->writeToStdin( epwd + "\n" );
-            else                 proc->writeToStdin( "\n" );
+            if (!epwd.isEmpty()) proc->write( (epwd + "\n").toAscii() );
+            else                 proc->write( "\n" );
             state=WAITING_FOR_ENABLE;
         }
         break;
@@ -242,13 +242,13 @@ void SSHPIX::stateMachine()
         if ( cmpPrompt(stdoutBuffer,QRegExp(epwd_prompt)) )
         {
             stdoutBuffer="";
-            if (!epwd.isEmpty()) proc->writeToStdin( epwd + "\n" );
-            else                 proc->writeToStdin( "\n" );
+            if (!epwd.isEmpty()) proc->write( (epwd + "\n").toAscii() );
+            else                 proc->write( "\n" );
             state=WAITING_FOR_ENABLE;
             break;
         }
         if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
-        {       
+        {
             emit printStdout_sign( tr("In enable mode."));
             emit printStdout_sign( "\n");
             state=ENABLE;  // and go to ENABLE target in switch
@@ -272,11 +272,11 @@ void SSHPIX::stateMachine()
                 QString cmd = pre_config_commands.front();
                 pre_config_commands.pop_front();
 
-                if (cmd.find("reload in")!=-1)
+                if (cmd.indexOf("reload in")!=-1)
                     state = SCHEDULE_RELOAD_DIALOG;
 
-                proc->writeToStdin( cmd + "\n" );
-                // proc->writeToStdin( "\n" );
+                proc->write( (cmd + "\n").toAscii() );
+                // proc->write( "\n" );
                 break;
             }
 
@@ -297,8 +297,8 @@ void SSHPIX::stateMachine()
                 break;
             }
 
-            proc->writeToStdin( "config t\n" );
-            //proc->writeToStdin( "\n" );
+            proc->write( "config t\n" );
+            //proc->write( "\n" );
             state=WAITING_FOR_CONFIG_PROMPT;
         }
         break;
@@ -307,19 +307,19 @@ void SSHPIX::stateMachine()
         if ( cmpPrompt(stdoutBuffer,QRegExp("System config.* modified\\. Save?")) )
         {
             stdoutBuffer="";
-            proc->writeToStdin( "n" );  // no \n needed
+            proc->write( "n" );  // no \n needed
             break;
         }
         if ( cmpPrompt(stdoutBuffer,QRegExp("Proceed with reload?")) )
         {
             stdoutBuffer="";
-            proc->writeToStdin( "y" );  // no \n needed
+            proc->write( "y" );  // no \n needed
             break;
         }
         if ( cmpPrompt(stdoutBuffer,QRegExp("SHUTDOWN")) )
         {
             stdoutBuffer="";
-            proc->writeToStdin( "\n" );
+            proc->write( "\n" );
             state = ENABLE;
             break;
         }
@@ -327,8 +327,9 @@ void SSHPIX::stateMachine()
 
     case EXECUTING_COMMAND:
         if ( cmpPrompt(stdoutBuffer,QRegExp(enable_prompt)) )
-        {       
-            QApplication::eventLoop()->exitLoop();
+        {
+            //QApplication::eventLoop()->exitLoop();
+            QCoreApplication::exit();
             state=COMMAND_DONE;
         }
         break;
@@ -340,7 +341,7 @@ void SSHPIX::stateMachine()
 
 /* install full policy */
             QString ff = wdir+"/"+conffile;
-            config_file = new ifstream(ff.latin1());
+            config_file = new ifstream(ff.toLatin1().constData());
             if ( ! *config_file)
             {
                 emit printStdout_sign(
@@ -364,15 +365,15 @@ void SSHPIX::stateMachine()
                     if (store)
                     {
                         QString s(s0.c_str());
-                        s.stripWhiteSpace();
+                        s = s.trimmed();
                         allConfig.push_back(s);
                         nLines++;
 /*
  * store names of access-lists and object-groups actually used in the config
  */
-                        if (s.find("access-list ")==0)
+                        if (s.indexOf("access-list ")==0)
                             newAcls.push_back(s.section(' ',1,1));
-                        if (s.find("object-group ")==0)
+                        if (s.indexOf("object-group ")==0)
                             newObjectGroups.push_back(s.section(' ',1,1));
                     }
                 }
@@ -418,7 +419,7 @@ void SSHPIX::stateMachine()
                 if (!verbose)
                 {
                     QString rl="";
-                    if (s.find("! Rule ")!=-1)  rl=s.mid(7);
+                    if (s.indexOf("! Rule ")!=-1)  rl=s.mid(7);
                     if ( !rl.isEmpty())
                     {
                         emit printStdout_sign( tr("Rule %1").arg(rl) + "\n" );
@@ -430,7 +431,7 @@ void SSHPIX::stateMachine()
                 {
                     if ( !s.isEmpty())  ncmd++;
                     stdoutBuffer="";
-                    proc->writeToStdin( s+"\n"); // send even if s is empty
+                    proc->write( (s+"\n").toAscii() ); // send even if s is empty
                     qApp->processEvents();
                     break;
                 } else
@@ -448,7 +449,7 @@ void SSHPIX::stateMachine()
 
                 state=EXIT_FROM_CONFIG;
                 emit printStdout_sign( tr("End") + "\n" );
-                proc->writeToStdin( "exit\n" );
+                proc->write( "exit\n" );
             }
         }
         break;
@@ -498,14 +499,14 @@ void SSHPIX::stateMachine()
                 QString cmd = post_config_commands.front();
                 post_config_commands.pop_front();
 
-                proc->writeToStdin( cmd + "\n" );
-                //proc->writeToStdin( "\n" );
+                proc->write( (cmd + "\n").toAscii() );
+                //proc->write( "\n" );
                 break;
             }
 
             stdoutBuffer="";
             state=EXIT;
-            proc->writeToStdin( "exit\n");
+            proc->write( "exit\n");
         }
         break;
 
@@ -540,15 +541,15 @@ void SSHPIX::PIXbackup()
     if (state==FINISH) return;
     if (state==COMMAND_DONE)
     {
-        ofstream ofs(backupFile.latin1());
-        ofs << cfg;
+        ofstream ofs(backupFile.toLatin1().constData());
+        ofs << cfg.toAscii().constData();
         ofs.close();
 
         backup=false;  // backup is done
         state=ENABLE;
     }
 
-    proc->writeToStdin( "\n" );
+    proc->write( "\n" );
 }
 
 void SSHPIX::getACLs()
@@ -563,7 +564,7 @@ void SSHPIX::getACLs()
     QString     sa=cmd(proc,"show access-list");
 
     QStringList showAcls;
-    showAcls=QStringList::split("\n",sa);
+    showAcls=sa.split("\n");
 
     verbose=sv;
     quiet=sq;
@@ -575,17 +576,17 @@ void SSHPIX::getACLs()
         for (QStringList::iterator i=showAcls.begin(); i!=showAcls.end(); i++)
         {
 //            if (fwbdebug) qDebug("%s",(*i).ascii());
-            if ((*i).find("access-list ")==0 && (*i).find(";")==-1)
+            if ((*i).indexOf("access-list ")==0 && (*i).indexOf(";")==-1)
             {
                 QString an=(*i).section(' ',1,1);
-                if (an!="cached" && currentAcls.find(an)==currentAcls.end())
+                if (an!="cached" && currentAcls.indexOf(an)==-1)
                     currentAcls.push_back(an);
             }
         }
         state=GET_OG;
     }
 
-    proc->writeToStdin( "\n" );
+    proc->write( "\n" );
 }
 
 void SSHPIX::clearACLs()
@@ -595,16 +596,16 @@ void SSHPIX::clearACLs()
     emit printStdout_sign( "\n");
     emit printStdout_sign(tr("*** Clearing unused access lists"));
     emit printStdout_sign( "\n");
-    
+
     QString ca;
-        
+
     while (currentAcls.size()!=0)
     {
         ca=currentAcls.front();
         currentAcls.pop_front();
-        if (newAcls.find(ca)==newAcls.end())
+        if (newAcls.indexOf(ca)==-1)//newAcls.end())
         {
-            if (fwbdebug) qDebug("clear access-list %s",ca.ascii());
+            if (fwbdebug) qDebug("clear access-list %s",ca.toAscii().constData());
             cmd(proc,QString("clear access-list %1").arg(ca));
 
 /* if state changed to FINISH, there was an error and ssh terminated */
@@ -613,7 +614,7 @@ void SSHPIX::clearACLs()
     }
 
     state=CLEAR_OG;
-    proc->writeToStdin( "\n" );
+    proc->write( "\n" );
 }
 
 void SSHPIX::getObjectGroups()
@@ -628,7 +629,7 @@ void SSHPIX::getObjectGroups()
     QString sog=cmd(proc,"show object-group");
 
     QStringList showOG;
-    showOG=QStringList::split("\n",sog);
+    showOG=sog.split("\n");
 
     verbose=sv;
     quiet=sq;
@@ -640,17 +641,17 @@ void SSHPIX::getObjectGroups()
         for (QStringList::iterator i=showOG.begin(); i!=showOG.end(); i++)
         {
 //            if (fwbdebug) qDebug("%s",(*i).ascii());
-            if ((*i).find("object-group ")==0)
+            if ((*i).indexOf("object-group ")==0)
             {
                 QString ogn=(*i).section(' ',1,1);
-                if (currentObjectGroups.find(ogn)==currentObjectGroups.end())
+                if (currentObjectGroups.indexOf(ogn)==-1)//currentObjectGroups.end())
                     currentObjectGroups.push_back(ogn);
             }
         }
         state=CLEAR_ACLS;
     }
 
-    proc->writeToStdin( "\n" );
+    proc->write( "\n" );
 }
 
 void SSHPIX::clearObjectGroups()
@@ -660,16 +661,16 @@ void SSHPIX::clearObjectGroups()
     emit printStdout_sign( "\n");
     emit printStdout_sign(tr("*** Clearing unused object groups"));
     emit printStdout_sign( "\n");
-    
+
     QString ca;
-        
+
     while (currentObjectGroups.size()!=0)
     {
         ca=currentObjectGroups.front();
         currentObjectGroups.pop_front();
-        if (newObjectGroups.find(ca)==newObjectGroups.end())
+        if (newObjectGroups.indexOf(ca)==-1)//==newObjectGroups.end())
         {
-            if (fwbdebug) qDebug("clear object-group %s",ca.ascii());
+            if (fwbdebug) qDebug("clear object-group %s",ca.toAscii().constData());
             cmd(proc,QString("clear object-group %1").arg(ca));
 
 /* if state changed to FINISH, there was an error and ssh terminated */
@@ -679,7 +680,7 @@ void SSHPIX::clearObjectGroups()
 
     state=EXIT_FROM_CONFIG;
     emit printStdout_sign( tr("*** End ") + "\n" );
-    proc->writeToStdin( "exit\n" );
+    proc->write( "exit\n" );
 }
 
 void SSHPIX::PIXincrementalInstall()
@@ -710,8 +711,8 @@ void SSHPIX::PIXincrementalInstall()
     if (state==COMMAND_DONE)
     {
         QString statefile = wdir+"/"+conffile + "_current";
-        ofstream ofs(statefile.latin1());
-        ofs << current_config;
+        ofstream ofs(statefile.toLatin1().constData());
+        ofs << current_config.toAscii().constData();
         ofs.close();
 
         emit printStdout_sign(tr("Generating configuration diff"));
@@ -722,9 +723,9 @@ void SSHPIX::PIXincrementalInstall()
 //        emit printStdout_sign(tr("Running command: %1\n").arg(cm));
 
 #ifdef _WIN32
-        FILE *f = _popen( cm.latin1(), "r");
+        FILE *f = _popen( cm.toLatin1().constData(), "r");
 #else
-        FILE *f = popen( cm.latin1(), "r");
+        FILE *f = popen( cm.toLatin1().constData(), "r");
 #endif
         if (f==NULL)
         {
@@ -733,7 +734,7 @@ void SSHPIX::PIXincrementalInstall()
             emit printStdout_sign( "\n");
             switch (errno)
             {
-            case EAGAIN: 
+            case EAGAIN:
             case ENOMEM:
                 emit printStdout_sign(tr("Not enough memory.") + "\n");
                 break;
@@ -746,7 +747,7 @@ void SSHPIX::PIXincrementalInstall()
             }
             emit printStdout_sign( "\n");
             state=FINISH;
-            proc->writeToStdin( "\n" );
+            proc->write( "\n" );
             return;
         }
 
@@ -765,15 +766,15 @@ void SSHPIX::PIXincrementalInstall()
 
 	if (allConfig.isEmpty())
 	{
-	    allConfig="";
+	    allConfig=QStringList();
             emit printStdout_sign(tr("Empty configuration diff"));
             emit printStdout_sign( "\n");
 	}
 
         if (save_diff)
         {
-            ofstream odiff(wdir+"/"+diff_file.latin1());
-            odiff << allConfig.join("");
+            ofstream odiff((wdir+"/"+diff_file).toLatin1().constData());
+            odiff << allConfig.join("").toAscii().constData();
             odiff.close();
         }
 
@@ -782,6 +783,6 @@ void SSHPIX::PIXincrementalInstall()
         if (!dry_run)
             emit printStdout_sign(tr("Pushing firewall configuration") + "\n");
     }
-    proc->writeToStdin( "\n" );
+    proc->write( "\n" );
 }
 
