@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: SSHSession.cpp,v 1.28 2007/12/15 23:19:04 vkurland Exp $
+  $Id: SSHSession.cpp 467 2008-08-13 06:17:54Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,7 +17,7 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
@@ -25,7 +25,7 @@
 
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -38,7 +38,7 @@
 #include <qmessagebox.h>
 #include <qapplication.h>
 #include <qeventloop.h>
-#include <qcstring.h>
+//#include <qcstring.h>
 #include <qtextcodec.h>
 #include <qapplication.h>
 #include <qdatetime.h>
@@ -52,14 +52,14 @@
 
 using namespace std;
 
-char  *SSHSession::newKeyOpenSSH  ="Are you sure you want to continue connecting (yes/no)?";
-char  *SSHSession::newKeyPlink    ="Store key in cache? (y/n)";
-char  *SSHSession::newKeyVsh      ="Accept and save? (y/n)";
-char  *SSHSession::newKeySSHComm  ="You can get a public key's fingerprint by running";
+const char  *SSHSession::newKeyOpenSSH  ="Are you sure you want to continue connecting (yes/no)?";
+const char  *SSHSession::newKeyPlink    ="Store key in cache? (y/n)";
+const char  *SSHSession::newKeyVsh      ="Accept and save? (y/n)";
+const char  *SSHSession::newKeySSHComm  ="You can get a public key's fingerprint by running";
 
 
-char  *SSHSession::fingerprintPrompt1="key fingerprint is";
-char  *SSHSession::fingerprintPrompt2="Key fingerprint:";
+const char  *SSHSession::fingerprintPrompt1="key fingerprint is";
+const char  *SSHSession::fingerprintPrompt2="Key fingerprint:";
 
 
 
@@ -109,18 +109,18 @@ SSHSession::SSHSession(QWidget *_par,
 
 QString SSHSession::findKeyFingerprint(QString &buffer)
 {
-    char *fp = fingerprintPrompt1;
+    const char *fp = fingerprintPrompt1;
     int n1,n2;
 
-    if ( (n1=buffer.find(fp))==-1)
-    {        
+    if ( (n1=buffer.indexOf(fp))==-1)
+    {
         fp = fingerprintPrompt2;
-        if ( (n1=buffer.find(fp))==-1)
+        if ( (n1=buffer.indexOf(fp))==-1)
             return QString("");
     }
 
     n1 += strlen(fp)+1;
-    n2 = buffer.find("\n", n1+4);
+    n2 = buffer.indexOf("\n", n1+4);
     return buffer.mid(n1,n2-n1);
 }
 
@@ -128,8 +128,6 @@ void SSHSession::startSession()
 {
     proc = new QProcess();
     retcode = -1;
-    proc->setCommunication(
-        QProcess::Stdin|QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
 
     startHeartBeat();
 
@@ -137,18 +135,30 @@ void SSHSession::startSession()
         qDebug("SSHSession::startSession  this=%p  proc=%p   heartBeatTimer=%p",
                this,proc,heartBeatTimer);
 
-    connect(proc,SIGNAL(readyReadStdout()), this,  SLOT(readFromStdout() ) );
-    connect(proc,SIGNAL(readyReadStderr()), this,  SLOT(readFromStderr() ) );
-    connect(proc,SIGNAL(processExited()),   this,  SLOT(processExited() ) );
+    connect(proc,SIGNAL(readyReadStandardOutput()),
+            this,  SLOT(readFromStdout() ) );
+    connect(proc,SIGNAL(readyReadStandardError()),
+            this,  SLOT(readFromStderr() ) );
+    connect(proc,SIGNAL(finished( int, QProcess::ExitStatus )),
+            this,  SLOT(finished( int ) ) );
 
     QTextCodec::setCodecForCStrings(QTextCodec::codecForName("latin1"));
 
-    for (QStringList::const_iterator i=args.begin(); i!=args.end(); ++i)
+    assert(args.size() > 0);
+
+    QStringList arguments;
+    QStringList::const_iterator i=args.begin();
+    QString program = *i;
+    ++i;
+
+    for ( ; i!=args.end(); ++i)
     {
-        proc->addArgument( *i );
+        arguments << *i;
+        //proc->addArgument( *i );
         cmd += *i;
     }
 
+
     QStringList env;
 
 #ifdef _WIN32
@@ -175,15 +185,27 @@ void SSHSession::startSession()
 
 //    emit printStdout_sign( tr("Running command %1\n").arg(cmd) );
 
-    if ( ! proc->start(&env))
+    proc->setEnvironment(env);
+
+    if (fwbdebug)
+    {
+        qDebug("Launch external ssh client %s", program.toAscii().constData());
+        qDebug("Arguments:");
+        QStringList::const_iterator i;
+        for (i=arguments.begin(); i!=arguments.end(); ++i)
+            qDebug("    %s", (*i).toAscii().constData());
+    }
+
+    proc->start(program, arguments);
+
+    if ( !proc->waitForStarted() )
     {
         emit printStdout_sign( tr("Failed to start ssh") + "\n" );
         return;
     }
 
     if (fwbdebug)
-        qDebug("SSHSession::startSession   started child process pid=%ld",
-               proc->processIdentifier());
+        qDebug("SSHSession::startSession   started child process");
 
 
     logged_in = false;
@@ -231,27 +253,27 @@ void SSHSession::terminate()
 
     if (proc!=NULL)
     {
-        disconnect(proc,SIGNAL(readyReadStdout()),
-                   this,SLOT(readFromStdout() ) );
-        disconnect(proc,SIGNAL(readyReadStderr()),
-                   this,SLOT(readFromStderr() ) );
-        disconnect(proc,SIGNAL(processExited()),
-                   this,SLOT(processExited() ) );
+//        disconnect(proc,SIGNAL(readyReadStdout()),
+//                   this,SLOT(readFromStdout() ) );
+//        disconnect(proc,SIGNAL(readyReadStderr()),
+//                   this,SLOT(readFromStderr() ) );
+
+        disconnect(proc,SIGNAL(finished(int, QProcess::ExitStatus)),
+                   this,SLOT(finished(int) ) );
 
         if (fwbdebug)
-            qDebug("SSHSession::terminate   terminating child process pid=%ld",
-                   proc->processIdentifier());
+            qDebug("SSHSession::terminate   terminating child process");
 #ifdef _WIN32
-        if (proc->processIdentifier() != NULL)
+        if (proc->pid() != NULL)
 #else
-        if (proc->processIdentifier() != -1)
+            if (proc->pid() != -1)
 #endif
         {
             // process is stll alive, killing
-            QString s=QString(proc->readStdout());
+            QString s=QString(proc->readAllStandardOutput());
             if (!quiet)
             {
-                s.replace('\r',"");    
+                s.replace('\r',"");
                 emit printStdout_sign(s);
             }
             proc->kill();
@@ -294,8 +316,11 @@ void SSHSession::sendLine()
     {
         string s = input.front();
         s = s + "\n";
+#if 0
         if (fwbdebug)
-            qDebug("SSHSession::sendLine : %d lines to go -- %s",input.size(),s.c_str());
+            qDebug("SSHSession::sendLine : %d lines to go -- %s",
+                   int(input.size()), s.c_str());
+#endif
         input.pop_front();
 
         stdoutBuffer="";
@@ -315,13 +340,13 @@ void SSHSession::sendLine()
  * to support old QT 3.x, the code must work around this problem.
  */
         QByteArray buf;
-        buf.duplicate( s.c_str(), s.length() );
-        proc->writeToStdin(buf);
+        buf = s.c_str();
+        proc->write/*ToStdin*/(buf);
 
         n++;
     }
     emit updateProgressBar_sign(input.size(),false);
-        
+
     if (input.size()==0)
     {
         if (fwbdebug) qDebug("SSHUnx::sendLine - entire file sent, closeStdin=%d",
@@ -335,12 +360,12 @@ void SSHSession::allDataSent()
     if (fwbdebug)
         qDebug("SSHSession::allDataSent   closing stdin");
 
-    disconnect(proc,SIGNAL(wroteToStdin()),this,SLOT(readyToSend()));
+    disconnect(proc,SIGNAL(bytesWritten(qint64)),this,SLOT(readyToSend()));
 
 #ifdef _WIN32
     Sleep(2000);
 #endif
-    proc->closeStdin();
+    proc->closeWriteChannel();
 #ifdef _WIN32
     Sleep(1000);
 #endif
@@ -364,7 +389,7 @@ void SSHSession::heartBeat()
     if (fwbdebug) qDebug("SSHSession::heartBeat");
     readFromStderr();
     readFromStdout();
-    if (endOfCopy && closeStdin) 
+    if (endOfCopy && closeStdin)
     {
         allDataSent();
         endOfCopy = false;
@@ -379,10 +404,10 @@ void SSHSession::readFromStdout()
         {
             QTime t = QTime::currentTime();
             qDebug(QString("SSHSession::readFromStdout()  on entry: %1").
-                   arg(t.toString("hh:mm:ss.zzz")));
+                    arg(t.toString("hh:mm:ss.zzz")).toAscii().constData());
         }
 
-        QByteArray ba = proc->readStdout();
+        QByteArray ba = proc->readAllStandardOutput();
         int  basize  = ba.size();
         if (basize==0) return;
 
@@ -394,13 +419,13 @@ void SSHSession::readFromStdout()
         QString lastLine = "";
 
         // split on LF
-        QStringList bufLines = QStringList::split("\n",buf,TRUE);
+        QStringList bufLines = buf.split("\n", QString::KeepEmptyParts);
 
         if (fwbdebug)
         {
             QTime t = QTime::currentTime();
             qDebug(QString("SSHSession::readFromStdout()  on check 1: %1").
-                   arg(t.toString("hh:mm:ss.zzz")));
+                    arg(t.toString("hh:mm:ss.zzz")).toAscii().constData());
         }
 
 #if 0
@@ -408,7 +433,7 @@ void SSHSession::readFromStdout()
         {
             qDebug("- - - - - - - - - - - - - - - - - - - - - - - - - -");
             qDebug("basize='%d'",basize);
-            qDebug("buffer='%s'",buf.ascii());
+            qDebug("buffer='%s'",buf.toAscii().constData());
             qDebug("endsWithLF=%d",endsWithLF);
             qDebug("bufLines.size()=%d",bufLines.size());
         }
@@ -432,7 +457,7 @@ void SSHSession::readFromStdout()
             QString s = pendingLogLine + *i + "\n";
             if (!quiet)
             {
-                s.replace('\r',"");    
+                s.replace('\r',"");
                 emit printStdout_sign(s);
             }
             pendingLogLine = "";
@@ -444,7 +469,7 @@ void SSHSession::readFromStdout()
         {
             QTime t = QTime::currentTime();
             qDebug(QString("SSHSession::readFromStdout()  on check 2: %1").
-                   arg(t.toString("hh:mm:ss.zzz")));
+                   arg(t.toString("hh:mm:ss.zzz")).toAscii().constData());
         }
 
         stateMachine();
@@ -453,30 +478,38 @@ void SSHSession::readFromStdout()
         {
             QTime t = QTime::currentTime();
             qDebug(QString("SSHSession::readFromStdout()  finish: %1").
-                   arg(t.toString("hh:mm:ss.zzz")));
+                    arg(t.toString("hh:mm:ss.zzz")).toAscii().constData());
         }
 
     }
 }
 
+/*
+ * note: we set qprocess channel mode to "merged" but despite that
+ * QProcess does not seem to merge stdout and stderr on Windows 
+ * (QT 4.4.0)
+ * Will merge them here
+ */
 void SSHSession::readFromStderr()
 {
     if (proc)
     {
-        QByteArray ba = proc->readStderr();
+        QByteArray ba = proc->readAllStandardError();
         if (ba.size()!=0)
         {
             QString s=QString(ba);
+            if (fwbdebug) qDebug("SSHSession::readFromStderr  buf=%s",
+                                 s.toAscii().constData());
             emit printStdout_sign(s);
-            stderrBuffer=stderrBuffer + QString(s);
+            stdoutBuffer.append(s);
+            stateMachine();
         }
     }
 }
 
 void SSHSession::sessionComplete(bool err)
 {
-    if (fwbdebug)
-        qDebug(QString("SSHSession::sessionComplete  err=%1").arg(err));
+    if (fwbdebug) qDebug("SSHSession::sessionComplete  err=%d", err);
 
     error = err;
     if (error)
@@ -485,12 +518,10 @@ void SSHSession::sessionComplete(bool err)
         emit sessionFinished_sign();
 }
 
-void SSHSession::processExited()
+void SSHSession::finished(int retcode)
 {
     if (fwbdebug) qDebug("SSHSession::processExited");
 
-    if (proc) retcode=proc->exitStatus();
-
     if (fwbdebug) qDebug("SSHSession::processExited   proc=%p retcode=%d",proc,retcode);
 
     // background process has exited now, we do not need proc object anymore
@@ -498,7 +529,7 @@ void SSHSession::processExited()
     proc=NULL;
 
     QString exitStatus = (retcode)?QObject::tr("ERROR"):QObject::tr("OK");
-    
+
     emit printStdout_sign(tr("SSH session terminated, exit status: %1").arg(retcode) + "\n");
     sessionComplete( retcode!=0 );
 //    if (retcode) error=true;
@@ -509,15 +540,15 @@ bool SSHSession::cmpPrompt(const QString &str,const QString &prompt)
 {
     if (fwbdebug)
         qDebug("SSHSession::cmpPrompt: str='%s' prompt='%s'",
-               str.ascii(),prompt.ascii());
+               str.toAscii().constData(),prompt.toAscii().constData());
 
     if (str.isEmpty()) return false;
 
-    bool res=(str.findRev(prompt,-1)!=-1);
+    bool res=(str.lastIndexOf(prompt,-1)!=-1);
     if (!res)
     {
-        QString s=str.stripWhiteSpace();
-        res=(s.findRev(prompt,-1)!=-1);
+        QString s=str.trimmed();
+        res=(s.lastIndexOf(prompt,-1)!=-1);
     }
 
     if (fwbdebug)
@@ -530,11 +561,11 @@ bool SSHSession::cmpPrompt(const QString &str,const QRegExp &prompt)
 {
     if (fwbdebug)
         qDebug("SSHSession::cmpPrompt: str='%s' prompt='%s' (regexp)",
-               str.ascii(),prompt.pattern().ascii());
+               str.toAscii().constData(),prompt.pattern().toAscii().constData());
 
     if (str.isEmpty()) return false;
 
-    bool res=(str.findRev(prompt,-1)!=-1);
+    bool res=(str.lastIndexOf(prompt,-1)!=-1);
 
     if (fwbdebug)
         qDebug("SSHSession::cmpPrompt: res=%d",res);
