@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: UDPServiceDialog.h,v 1.10 2006/05/13 06:53:05 vkurland Exp $
+  $Id: UDPServiceDialog.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,21 +27,26 @@
 #ifndef __UDPSERVICEDIALOG_H_
 #define __UDPSERVICEDIALOG_H_
 
-#include "config.h"
-#include <udpservicedialog_q.h>
+#include "../../config.h"
+#include <ui_udpservicedialog_q.h>
+#include <QWidget>
 
 #include "fwbuilder/FWObject.h"
 
 
-class UDPServiceDialog : public UDPServiceDialog_q
+class ProjectPanel;
+class UDPServiceDialog : public QWidget
 {
     Q_OBJECT
 
     libfwbuilder::FWObject *obj;
     bool init;
+    Ui::UDPServiceDialog_q *m_dialog;
+    ProjectPanel *m_project;
 
  public:
-    UDPServiceDialog(QWidget *parent) : UDPServiceDialog_q(parent) { obj=NULL; }
+    UDPServiceDialog(ProjectPanel *project, QWidget *parent);
+    ~UDPServiceDialog();
     
 public slots:
     virtual void changed();
