@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.h,v 1.37 2007/12/15 23:19:05 vkurland Exp $
+  $Id: instDialog.h 392 2008-07-19 00:49:58Z svn $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,10 +30,11 @@
 
 #include "../../config.h"
 
-#include "instdialog_q.h"
+#include <ui_instdialog_q.h>
 #include "instConf.h"
+#include "fakeWizard.h"
 
-#include <instoptionsdialog_q.h>
+#include "instOptionsDialog.h"
 
 #include <qstring.h>
 #include <qstringlist.h>
@@ -49,10 +50,10 @@ class SSHSession;
 class QTextEdit;
 class QListViewItem;
 class QCheckListItem;
-class QButton;
+class QPushButton;
 class QProgressBar;
 class QStringList;
-class QCheckTableItem;
+//class QCheckTableItem;
 
 namespace libfwbuilder
 {
@@ -61,17 +62,19 @@ namespace libfwbuilder
 
 enum BatchOperation {BATCH_INSTALL,BATCH_COMPILE} ;
 
-typedef std::map<libfwbuilder::Firewall *,QListViewItem *> t_listMap;
-typedef std::map<libfwbuilder::Firewall *,QCheckTableItem *> t_tableMap;
+typedef std::map<libfwbuilder::Firewall *,QTreeWidgetItem *> t_listMap;
+typedef std::map<libfwbuilder::Firewall *,QTableWidgetItem *> t_tableMap;
 typedef std::list<libfwbuilder::Firewall *> t_fwList;
 typedef std::pair<QString,QString> t_procMess; // first - compilation result, second - installation result;
 typedef std::set<libfwbuilder::Firewall*> t_fwSet;
 
 
-class instDialog : public instDialog_q {
+class instDialog : public QDialog, public FakeWizard
+{
 
     Q_OBJECT
 
+    Ui::instDialog_q *m_dialog;
     bool        ready;
     bool        activationCommandDone;
     instConf    cnf;
@@ -111,19 +114,21 @@ class instDialog : public instDialog_q {
     t_tableMap           compileMapping;
     t_tableMap           installMapping;
     
-    QStringList          args;
+    QString              path; //path of the program to execute
+    QStringList          args; //arguments for that program
         
     QTextEdit           *currentLog;
-    QButton             *currentSaveButton;
-    QButton             *currentStopButton;
+    QPushButton             *currentSaveButton;
+    QPushButton             *currentStopButton;
     QProgressBar        *currentProgressBar;
     QProgressBar        *currentFirewallsBar;
     QLabel              *currentLabel;
     QLabel              *currentFWLabel;
     QString              currentSearchString;
+    bool                 creatingTable;
     
     BatchOperation       operation;
-    instOptionsDialog_q *dlg;
+    instOptionsDialog   *dlg;
     QString              pendingLogLine;
 
     int progress;
@@ -131,6 +136,7 @@ class instDialog : public instDialog_q {
     int processedRules;
     int lastPage;
     bool stopProcessFlag;
+    bool rejectDialogFlag;
     bool compileFlag;
     bool customScriptFlag;
     bool showSelectedFlag;
@@ -145,14 +151,15 @@ class instDialog : public instDialog_q {
     bool testFirewall(libfwbuilder::Firewall*);
     void finishInstall(bool success=true);
     void fillInstallOpList();
+    void installNext();
     void initInstall();
     void analyseInstallQueue(bool &fPix, bool &fCustInst);
-    libfwbuilder::Firewall *findFirewallbyListItem(QListViewItem* item);
-    libfwbuilder::Firewall *findFirewallbyTableItem(QCheckTableItem *item);
+    libfwbuilder::Firewall *findFirewallbyListItem(QTreeWidgetItem* item);
+    libfwbuilder::Firewall *findFirewallbyTableItem(QTableWidgetItem *item);
     
     
  public:
-    instDialog(QWidget* p, BatchOperation op);
+    instDialog(QWidget* p, BatchOperation op, t_fwSet reqFirewalls_);
     virtual ~instDialog();
     
     void setReady(bool f) { ready=f; }
@@ -160,7 +167,7 @@ class instDialog : public instDialog_q {
     void summary();
 
 
-    QWidget* page(int n) { return QWizard::page(n); }
+    QWidget* page(int n) { return m_dialog->stackedWidget->widget(n); }
     
     void initiateCopy(const QString &file);
     void runSSH(SSHSession *s);
@@ -171,7 +178,7 @@ class instDialog : public instDialog_q {
     bool isTableHasChecked();
     void clearReqFirewalls();
     void addReqFirewall(libfwbuilder::Firewall *f);
-    void addToLog(const QString &buf); 
+    void interpretLogLine(const QString &buf);
     
 protected:
 
@@ -186,32 +193,40 @@ protected:
     QString getFullPath(instConf &cnf, const QString &file );
 
  protected slots:
-    void processExited();
-    void installerFinished();
+    void processExited(int code);
+    void installerSuccess();
     void installerError();
     void installSelected();
-    void selected(const QString &title);
-    void reject();
+    void showPage(const int page);
+     
+    void finishClicked();
+    void cancelClicked();
+     
     void testRunRequested();
      
-    void append(const QString &line);
-    void appendRich(const QString &line);
+    void addToLog(const QString &buf); 
     void updateProgressBar(int n,bool setsize);
 
-    void continueRun();
     virtual void saveLog();
     virtual void togleDetailMC();
 
     virtual void readFromStdout();
+    //virtual void readFromStderr();
     virtual void selectAllFirewalls();
     virtual void deselectAllFirewalls();
+    
+    virtual void nextClicked();
+    virtual void backClicked();
 
     void stopSessionAndDisconnectSignals();
+    void continueRun();
+    void restartSession();
+    void sessionCleanupOnError();
     
     void compileSelected();
     void stopCompile();
     void stopInstall();
-    void findFirewallInCompileLog(QListViewItem*);
+    void findFirewallInCompileLog(QTreeWidgetItem*);
     void showSelected();
     void tableValueChanged(int row, int col);
     
