@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ipfwAdvancedDialog.cpp,v 1.13 2007/01/06 22:03:25 vkurland Exp $
+  $Id: ipfwAdvancedDialog.cpp 545 2008-09-19 20:27:58Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,19 +17,19 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "platforms.h"
 
 #include "ipfwAdvancedDialog.h"
 #include "SimpleTextEditor.h"
-#include "ObjectManipulator.h"
+#include "FWWindow.h"
 
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Management.h"
@@ -40,18 +40,24 @@
 #include <qcombobox.h>
 #include <qradiobutton.h>
 #include <qlineedit.h>
-#include <qwidgetstack.h>
+#include <qstackedwidget.h>
 #include <qregexp.h>
-#include <qtextview.h>
 
 
 using namespace std;
 using namespace libfwbuilder;
 
+ipfwAdvancedDialog::~ipfwAdvancedDialog()
+{
+    delete m_dialog;
+}
 
 ipfwAdvancedDialog::ipfwAdvancedDialog(QWidget *parent,FWObject *o)
-    : ipfwAdvancedDialog_q(parent)
+    : QDialog(parent)
 {
+    m_dialog = new Ui::ipfwAdvancedDialog_q;
+    m_dialog->setupUi(this);
+
     obj=o;
 
     FWOptions *fwopt=(Firewall::cast(obj))->getOptionsObject();
@@ -67,43 +73,57 @@ ipfwAdvancedDialog::ipfwAdvancedDialog(QWidget *parent,FWObject *o)
     if (!Resources::getTargetOptionBool(
             obj->getStr("host_OS"),"user_can_change_install_dir"))
     {
-        ipfw_fw_dir->setEnabled(false);
+        m_dialog->ipfw_fw_dir->setEnabled(false);
         fwopt->setStr("firewall_dir","");
     }
 
     if (fwopt->getStr("add_check_state_rule").empty())
         fwopt->setBool("add_check_state_rule",true);
 
-    data.registerOption( ipfw_add_check_state_rule  ,fwopt, "add_check_state_rule");
-    data.registerOption( ipfw_check_shadowing       ,fwopt, "check_shading"       );
-    data.registerOption( ipfw_ignore_empty_groups   ,fwopt, "ignore_empty_groups" );
-    data.registerOption( ipfw_fw_dir                ,fwopt, "firewall_dir"    );
-    data.registerOption( ipfw_user                  ,fwopt, "admUser"         );
-    data.registerOption( altAddress                 ,fwopt, "altAddress"      );
-    data.registerOption( sshArgs                    ,fwopt, "sshArgs"         );
-    data.registerOption( activationCmd              ,fwopt, "activationCmd"   );
+    data.registerOption(m_dialog->ipv4before_2,
+                        fwopt,
+                        "ipv4_6_order",
+                        QStringList() <<  "IPv4 before IPv6"
+                        <<"ipv4_first" <<
+                        "IPv6 before IPv4"
+                        << "ipv6_first"
+    );
 
-    data.registerOption( ipfw_manage_virtual_addr   ,fwopt, "manage_virtual_addr"  );
-    data.registerOption( ipfw_configure_interfaces  ,fwopt, "configure_interfaces" );
-    data.registerOption( ipfw_debug                 ,fwopt, "debug"           );
+    data.registerOption( m_dialog->ipfw_add_check_state_rule, fwopt,
+                         "add_check_state_rule");
+    data.registerOption( m_dialog->ipfw_check_shadowing, fwopt,
+                         "check_shading");
+    data.registerOption( m_dialog->ipfw_ignore_empty_groups, fwopt,
+                         "ignore_empty_groups" );
+    data.registerOption( m_dialog->ipfw_fw_dir, fwopt, "firewall_dir");
+    data.registerOption( m_dialog->ipfw_user, fwopt, "admUser");
+    data.registerOption( m_dialog->altAddress, fwopt, "altAddress");
+    data.registerOption( m_dialog->sshArgs, fwopt, "sshArgs");
+    data.registerOption( m_dialog->activationCmd, fwopt, "activationCmd");
 
-    data.registerOption( compiler                   ,fwopt, "compiler" );
-    data.registerOption( compilerArgs               ,fwopt, "cmdline"  );
-    data.registerOption( outputFileName       , fwopt,  "output_file"  );
+    data.registerOption( m_dialog->ipfw_manage_virtual_addr, fwopt,
+                         "manage_virtual_addr");
+    data.registerOption( m_dialog->ipfw_configure_interfaces, fwopt,
+                         "configure_interfaces");
+    data.registerOption( m_dialog->ipfw_debug, fwopt, "debug");
 
-    data.registerOption( mgmt_ssh             , fwopt,  "mgmt_ssh"  );
-    data.registerOption( mgmt_addr            , fwopt,  "mgmt_addr" );
+    data.registerOption( m_dialog->compiler, fwopt, "compiler" );
+    data.registerOption( m_dialog->compilerArgs, fwopt, "cmdline");
+    data.registerOption( m_dialog->outputFileName, fwopt, "output_file");
+
+    data.registerOption( m_dialog->mgmt_ssh, fwopt, "mgmt_ssh"  );
+    data.registerOption( m_dialog->mgmt_addr, fwopt, "mgmt_addr" );
 
     PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
 
-    installScript->setText(     pis->getCommand().c_str() );
-    installScriptArgs->setText( pis->getArguments().c_str() );
+    m_dialog->installScript->setText(     pis->getCommand().c_str() );
+    m_dialog->installScriptArgs->setText( pis->getArguments().c_str() );
 
 /* page "Prolog/Epilog" */
-    data.registerOption( prolog_script    , fwopt,
+    data.registerOption( m_dialog->prolog_script, fwopt,
                          "prolog_script"  );
 
-    data.registerOption( epilog_script    , fwopt,
+    data.registerOption( m_dialog->epilog_script, fwopt,
                          "epilog_script"  );
 
 
@@ -125,34 +145,34 @@ void ipfwAdvancedDialog::accept()
     data.saveAll();
 
     PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
-    pis->setCommand( installScript->text().latin1() );
-    pis->setArguments( installScriptArgs->text().latin1() );
+    pis->setCommand( m_dialog->installScript->text().toLatin1().constData() );
+    pis->setArguments( m_dialog->installScriptArgs->text().toLatin1().constData() );
 
-    om->updateLastModifiedTimestampForAllFirewalls(obj);
-    ipfwAdvancedDialog_q::accept();
+    mw->updateLastModifiedTimestampForAllFirewalls(obj);
+    QDialog::accept();
 }
 
 void ipfwAdvancedDialog::reject()
 {
-    ipfwAdvancedDialog_q::reject();
+    QDialog::reject();
 }
 
 void ipfwAdvancedDialog::editProlog()
 {
     SimpleTextEditor edt(this,
-                         prolog_script->text(),
+                         m_dialog->prolog_script->toPlainText(),
                           true, tr( "Script Editor" ) );
     if ( edt.exec() == QDialog::Accepted )
-        prolog_script->setText( edt.text() );
+        m_dialog->prolog_script->setText( edt.text() );
 }
 
 void ipfwAdvancedDialog::editEpilog()
 {
     SimpleTextEditor edt(this,
-                         epilog_script->text(),
+                         m_dialog->epilog_script->toPlainText(),
                           true, tr( "Script Editor" ) );
     if ( edt.exec() == QDialog::Accepted )
-        epilog_script->setText( edt.text() );
+        m_dialog->epilog_script->setText( edt.text() );
 }
 
 
