@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: listOfLibraries.h,v 1.7 2004/12/08 08:06:30 vkurland Exp $
+  $Id: listOfLibraries.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -26,7 +26,7 @@
 #ifndef __LISTOFLIBRARIES_H_
 #define __LISTOFLIBRARIES_H_
 
-#include "config.h"
+#include "../../config.h"
 
 #include <list>
 #include <qstring.h>
@@ -39,13 +39,13 @@
 class libData
 {
  public:
-    QString id;
+    int id;
     QString name;
     QString path;
     bool    mandatory;
     bool    load;
 
-    libData(const QString &i,const QString &n,const QString &p,bool f)
+    libData(int i, const QString &n, const QString &p, bool f)
         { id=i; name=n; path=p; mandatory=f; load=false; }
 };
 
@@ -70,7 +70,7 @@ class listOfLibraries : public std::list<libData>
     bool getLoad(const QString &libPath);
 
     bool isLoaded(const QString &libName);
-    bool isKnown(const QString &id);
+    bool isKnown(int id);
 
     void load();
     void save();
