@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: longTextDialog.cpp,v 1.1 2004/06/26 22:41:10 vkurland Exp $
+  $Id: longTextDialog.cpp 332 2008-07-06 20:11:03Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,14 +17,14 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
@@ -40,15 +40,23 @@
 using namespace libfwbuilder;
 using namespace std;
 
+longTextDialog::~longTextDialog()
+{
+    delete m_dialog;
+}
+
 longTextDialog::longTextDialog(QWidget *p,
                                const QString &txt,const QString &ltxt)
-    : longTextDialog_q(p) 
+    : QDialog(p)
 {
-    setCaption("Firewall Builder");
+    m_dialog = new Ui::longTextDialog_q;
+    m_dialog->setupUi(this);
 
-    dlgText->setText(txt);
+    setWindowTitle("Firewall Builder");
 
-    icn->setPixmap( QMessageBox::standardIcon(QMessageBox::Critical) );
+    m_dialog->dlgText->setText(txt);
 
-    dlgLongText->setText(ltxt);
+    m_dialog->icn->setPixmap( QMessageBox::standardIcon(QMessageBox::Critical) );
+
+    m_dialog->dlgLongText->setText(ltxt);
 }
