@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,8 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: main.cpp,v 1.121 2007/06/03 03:36:42 vkurland Exp $
+  $Id: main.cpp 565 2008-10-04 14:34:24Z vadim $
+
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,15 +18,17 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
+#include "VERSION.h"
+#include "../common/commoninit.h"
 
 #ifdef HAVE_GETOPT_H
 #  include <getopt.h>
@@ -38,10 +41,16 @@
 #endif
 
 #include <qapplication.h>
+#include <qmainwindow.h>
 #include <qtextcodec.h>
 #include <qtooltip.h>
 #include <qtimer.h>
 #include <qpixmapcache.h>
+#include <QTextCodec>
+#include <QLocale>
+#include <QDir>
+#include <QTranslator>
+#include <QLibraryInfo>
 
 /*
 #ifdef _WIN32
@@ -52,18 +61,17 @@
 #include "FWBSettings.h"
 #include "RCS.h"
 #include "FWWindow.h"
-#include "ObjectManipulator.h"
-#include "ObjectEditor.h"
 #include "FWObjectClipboard.h"
 #include "FWBTree.h"
-#include "findDialog.h"
-#include "listOfLibraries.h"
 #include "platforms.h"
+#include "listOfLibraries.h"
+#include "ObjectEditor.h"
+#include "findDialog.h"
+#include "ProjectPanel.h"
 
 #include "fwbuilder/FWObject.h"
 #include "fwbuilder/Tools.h"
 #include "fwbuilder/dns.h"
-//#include "fwbuilder/crypto.h"
 #include "fwbuilder/XMLTools.h"
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWException.h"
@@ -110,32 +118,31 @@ extern bool init2(const std::string &a1,
                   const std::string &moduleName,
                   const std::string &rp,
                   const std::string &rp1,
-                  bool f1, bool f2);
+                  bool f1, bool f2, bool d);
 #endif
 
+#if defined(Q_WS_MAC)
+extern void connectOdocHandler();
+#endif
+
+
 using namespace libfwbuilder;
 using namespace std;
 
 static QString    filename;
-static QString    objid;
+static QString    print_output_file_name;
 
 QApplication      *app        = NULL;
 FWWindow          *mw         = NULL;
-ObjectManipulator *om         = NULL;
-ObjectEditor      *oe         = NULL;
-QTextEdit         *oi         = NULL;
 FWBSettings       *st         = NULL;
-findDialog        *fd         = NULL;
 int                fwbdebug   = 0;
 bool               safemode   = false;
 bool               registered = false;
 bool               gui_experiment1 = false;
-
-listOfLibraries  *addOnLibs;
-
+bool               cli_print        = false;
+QString            cli_print_fwname = "";
 
 #ifndef _WIN32
-
 #ifndef HAVE_CFMAKERAW
 static inline void cfmakeraw(struct termios *termios_p)
 {
@@ -149,83 +156,83 @@ static inline void cfmakeraw(struct termios *termios_p)
 
 #ifndef HAVE_FORKPTY
 
-#include <unistd.h> 
-#include <stdlib.h> 
-#include <sys/ioctl.h> 
-#include <sys/stream.h> 
-#include <sys/stropts.h> 
+#include <unistd.h>
+#include <stdlib.h>
+#include <sys/ioctl.h>
+//#include <sys/stream.h>
+#include <sys/stropts.h>
 
-/* fork_pty() remplacement for Solaris. 
-* ignore the last two arguments 
-* for the moment 
-*/ 
+/* fork_pty() remplacement for Solaris.
+* ignore the last two arguments
+* for the moment
+*/
 int forkpty (int *amaster, char *name, void *unused1, void *unused2)
 {
-    int master, slave; 
-    char *slave_name; 
-    pid_t pid; 
+    int master, slave;
+    char *slave_name;
+    pid_t pid;
 
-    master = open("/dev/ptmx", O_RDWR); 
-    if (master < 0) 
-        return -1; 
+    master = open("/dev/ptmx", O_RDWR);
+    if (master < 0)
+        return -1;
 
-    if (grantpt (master) < 0) 
-    { 
-        close (master); 
-        return -1; 
-    } 
+    if (grantpt (master) < 0)
+    {
+        close (master);
+        return -1;
+    }
 
-    if (unlockpt (master) < 0) 
-    { 
-        close (master); 
-        return -1; 
-    } 
+    if (unlockpt (master) < 0)
+    {
+        close (master);
+        return -1;
+    }
 
-    slave_name = ptsname (master); 
-    if (slave_name == NULL) 
-    { 
-        close (master); 
-        return -1; 
-    } 
+    slave_name = ptsname (master);
+    if (slave_name == NULL)
+    {
+        close (master);
+        return -1;
+    }
 
-    slave = open (slave_name, O_RDWR); 
-    if (slave < 0) 
-    { 
-        close (master); 
-        return -1; 
-    } 
+    slave = open (slave_name, O_RDWR);
+    if (slave < 0)
+    {
+        close (master);
+        return -1;
+    }
 
-    if (ioctl (slave, I_PUSH, "ptem") < 0 
-        || ioctl (slave, I_PUSH, "ldterm") < 0) 
-    { 
-        close (slave); 
-        close (master); 
-        return -1; 
-    } 
+    if (ioctl (slave, I_PUSH, "ptem") < 0
+        || ioctl (slave, I_PUSH, "ldterm") < 0)
+    {
+        close (slave);
+        close (master);
+        return -1;
+    }
 
-    if (amaster) 
-        *amaster = master; 
+    if (amaster)
+        *amaster = master;
 
-    if (name) 
-        strcpy (name, slave_name); 
+    if (name)
+        strcpy (name, slave_name);
 
-    pid = fork (); 
-    switch (pid) 
-    { 
-    case -1: /* Error */ 
-        return -1; 
-    case 0: /* Child */ 
-        close (master); 
-        dup2 (slave, STDIN_FILENO); 
-        dup2 (slave, STDOUT_FILENO); 
-        dup2 (slave, STDERR_FILENO); 
-        return 0; 
-    default: /* Parent */ 
-        close (slave); 
-        return pid; 
-    } 
+    pid = fork ();
+    switch (pid)
+    {
+    case -1: /* Error */
+        return -1;
+    case 0: /* Child */
+        close (master);
+        dup2 (slave, STDIN_FILENO);
+        dup2 (slave, STDOUT_FILENO);
+        dup2 (slave, STDERR_FILENO);
+        return 0;
+    default: /* Parent */
+        close (slave);
+        return pid;
+    }
 
-    return -1; 
+    return -1;
 }
 
 #endif
@@ -325,23 +332,25 @@ char* strndup(const char* s,int n)
 
 void usage()
 {
-    cerr << "Usage: fwbuilder [-?hv] [filename]\n";
+    cerr << "Usage: fwbuilder [-hv] [-P object_name] [-o file_name] [ [-f] filename]\n";
+    cerr << endl;
 }
 
-
-int main( int argc, char ** argv )
+int main( int argc, char *argv[] )
 {
+
+
     bool        ssh_wrapper=false;
     const char *arg[64];
     int         i, j;
 
-    filename="";
-    objid="";
-    fwbdebug=0;
-    safemode=false;
-
-    bool desktopaware = true;
+    filename = "";
+    print_output_file_name = "";
+    fwbdebug = 0;
+    safemode = false;
 
+    if(fwbdebug)
+        qDebug("main()");
 /*
  * I am using njamd a lot, but gtkmm and probably some other libs
  * generate trap in their global static initialization code. Therefore
@@ -362,36 +371,23 @@ int main( int argc, char ** argv )
     i=1;
     j=1;
 
-    for ( ; argv[i]!=NULL; i++)
+    //for ( ; argv[i]!=NULL; i++)
+    for ( ; i<argc; i++)
     {
         if (strncmp(argv[i], "-X", 2)==0) { ssh_wrapper=true; continue; }
         else
             if (strncmp(argv[i], "-d", 2)==0) { fwbdebug++; continue; }
             else
-                arg[j]=strdup(argv[i]);
+                arg[j] = strdup(argv[i]);
         j++;
     }
-    arg[j]=NULL;
+    arg[j] = NULL;
 
     if (ssh_wrapper)
     {
 
 /* need to create and initialize settings to be able to use ssh/scp
  * configuration in the wrapper
- *
- * Note:
- *
- * We need to keep installation data and program settings in registry
- * folders with different names. QSettings always looks into Current
- * User registry first, so if the folders have the same names, then we
- * store evaluation key in Current User, while it is better to put it
- * in the Local Machine branch.
- *
- * So, installation data goes to HKLM Software\NetCitadel\FirewallBuilder
- * and settings to HKCU Software\NetCitadel\FirewallBuilder2
- * 
- * fwbuilder-lm determines folder path for the license file by
- * reading key Install_Dir under HKLM Software\NetCitadel\FirewallBuilder
  */
         st = new FWBSettings();
 
@@ -401,12 +397,19 @@ int main( int argc, char ** argv )
         QString sshcmd=st->getSSHPath();
 
         if (sshcmd.isEmpty()) sshcmd="ssh";
-        
-        arg[0]=strdup( sshcmd.latin1() );
+
+        arg[0]=strdup( sshcmd.toLatin1().constData() );
 
         if (fwbdebug)
+        {
             qDebug("cmd: %s",arg[0]);
-
+            qDebug("Arguments:");
+            for (const char **cptr = arg; *cptr!=NULL; cptr++)
+            {
+                qDebug("    %s", *cptr);
+                cptr++;
+            }
+        }
 
 /* forks ssh with a pty and proxies its communication on stdin/stdout
  * to avoid having to deal with pty. This is only needed on Unix.
@@ -530,21 +533,30 @@ int main( int argc, char ** argv )
     }
 #endif
 
+    if (fwbdebug)
+    {
+        qDebug("Command line:  argc=%d", argc);
+        for (int i=0; i<argc; ++i)
+            qDebug(argv[i]);
+    }
 
+    // can not use "-p" for command line printing because
+    // Mac OS X supplies switch "-psnXXXXX" when program is
+    // started via Finder.
 
     int c;
-    while ((c = getopt (argc , argv , "hvf:o:p:dxsg")) != EOF )
+    while ((c = getopt (argc , argv , "hvf:o:P:dxg")) != EOF )
 	switch (c) {
 	case 'h':
 	    usage();
 	    exit(0);
 
 	case 'f':
-	    filename=optarg;
+	    filename = optarg;
 	    break;
 
 	case 'o':
-	    objid=optarg;
+	    print_output_file_name=optarg;
 	    break;
 
         case 'd':
@@ -555,65 +567,73 @@ int main( int argc, char ** argv )
 	    cout << VERSION << endl;
 	    exit(0);
 
-        case 'x':
-            desktopaware=!desktopaware;
-            break;
-
-        case 's':
-            safemode = true;
-            break;
-            
         case 'g':
             gui_experiment1 = true;
             break;
+
+        case 'P':
+            cli_print = true ;
+            cli_print_fwname = optarg;
 	}
 
     if ( (argc-1)==optind)
         filename = strdup( argv[optind++] );
 
-
     try
     {
+        if (fwbdebug) qDebug("Initializing ...");
+
+        if (fwbdebug) qDebug("Creating app ...");
+        //QApplication::setDesktopSettingsAware(desktopaware);
+        app = new QApplication( argc, argv );
+        app->setOrganizationName(QLatin1String("NetCitadel LLC"));
+        app->setApplicationName(QLatin1String("Firewall Builder"));
 
-        if (fwbdebug) qDebug("initializing ...");
-    
 /* need to initialize in order to be able to use FWBSettings */
         init(argv);
         init_platforms();
 
-        if (fwbdebug) qDebug("creating app ...");
-    
-        QApplication::setDesktopSettingsAware(desktopaware);
-        app = new QApplication( argc, argv );
+        Q_INIT_RESOURCE(MainRes);
 
-        if (fwbdebug) qDebug("reading settings ...");
-    
+        if (fwbdebug) qDebug("Reading settings ...");
         st = new FWBSettings();
-
-        if (fwbdebug) qDebug("creating pixmap factory ...");
-
-/* initialize preferences */
         st->init();
-
         if (fwbdebug) qDebug("done");
 
         QPixmapCache::setCacheLimit(4096);
 
-#ifdef ELC
-        registered=init2(argv0, "Firewall Builder","fwb_gui","FirewallBuilder/2.1",true,true);
-#endif
+//        app->setFont(st->getTreeFont());
 
-        if (fwbdebug) qDebug("reading resources ...");
-        new Resources(respath+FS_SEPARATOR+"resources.xml");
+#ifdef ELC
+        registered = init2(argv0,
+                           "Firewall Builder",
+                           "fwb_gui30",
+                           "3.0",
+                           true, true, fwbdebug);
+#endif
+        if (cli_print)
+        {
+            if (fwbdebug) qDebug("Print from command line");
+            FWWindow::printFirewallFromFile(filename,
+                                            cli_print_fwname,
+                                            print_output_file_name);
+            return 0;
+        }
+
+        string full_res_path = respath + FS_SEPARATOR + "resources.xml";
+
+        if (fwbdebug)
+        {
+            qDebug("reading resources from '%s' ...",full_res_path.c_str());
+        }
+
+        //respath = RES_DIR;
+        new Resources(full_res_path);
         if (fwbdebug) qDebug("done");
 
-#if 0
-        QApplication::setDesktopSettingsAware(desktopaware);
-        app = new QApplication( argc, argv );
-#endif
-
         vector<std::string> platforms=Resources::getListOfPlatforms();
-        if (platforms.empty() || ( platforms.size()==1 && platforms.front()=="unknown" ))
+        if (platforms.empty() || (
+                platforms.size()==1 && platforms.front()=="unknown" ))
         {
             qDebug("Failed to load list of supported platforms");
             exit(1);
@@ -621,94 +641,70 @@ int main( int argc, char ** argv )
 
         if (fwbdebug) qDebug("creating widgets ...");
 
-        new FWBTree();
         new FWObjectDatabase();
         new FWObjectClipboard();
 
-//    cerr << "*** Current locale: " << QTextCodec::locale() << endl;
-
-//    app = new QApplication( argc, argv );
-
         if (fwbdebug) qDebug("loading translation for the current locale ...");
 
-/* this is an ugly hack to work around broken qt_es.qm translation
- * file shipped with QT 3.3 for Fedora-C4 and possibly other
- * distros. Only Spanish translation seems to be affected.  Trolltech
- * support req. N80793 (although I never got any suggestions for the
- * workaround, nor did they clearly admit the problem despite my
- * sending a simple example program to them to illustrate the
- * bug). They did not seem to care since Spanish translation file that
- * support guy had on his computer was fine, but he admitted it was
- * different from the one shipped with QT 3.3.4 for Fedora-C4 and for
- * Mac OS X. All I got from them was a promise to look into packaging
- * process.
- */
-        if (QString(QTextCodec::locale()).find("es")!=0)
-        {
-            QTranslator qt1(0);
-            qt1.load( QString( "qt_" ) + QTextCodec::locale(), QTTRANSLATIONSDIR );
-            app->installTranslator( &qt1 );
-        
-            QTranslator qt2(0);
-            qt2.load( QString( "qt_" ) + QTextCodec::locale(),  localepath.c_str() );
-            app->installTranslator( &qt2 );
-        }
-
+        QString local = QLocale::system().name();//"en_US";//
         QTranslator translator(0);
-        translator.load(QString("fwbuilder_")+QTextCodec::locale(),localepath.c_str());
+        translator.load(QLatin1String("fwbuilder_") +
+                        QString(local), localepath.c_str());
         app->installTranslator (&translator);
 
+        QString qt_resource_dir =
+            QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+
+
+        QTranslator qt_translator(0);
+        qt_translator.load(QLatin1String("qt_") + QLocale::system().name(),
+                           qt_resource_dir);
+        app->installTranslator (&qt_translator);
+
+
 /* must build list of available libraries _after_ creation of
  * FWObjectDatabase and settings */
 
         if (fwbdebug) qDebug("loading libraries ...");
 
-        addOnLibs = new listOfLibraries();
-
         mw  = new FWWindow();
-        oe  = new ObjectEditor((QWidget*)mw->objectEditorStack);
-//        oe->open(mw->db());
-        oe->setCloseButton(mw->closeObjectEditorButton);
-        oe->setApplyButton(mw->applyObjectEditorButton);
-        oe->hide();
-        fd  = new findDialog(mw);
-        fd->hide();
-
-        mw->setSafeMode(safemode);
-        mw->setStartupFileName(filename);
-
-        QToolTip::setWakeUpDelay( st->getTooltipDelay()*1000 );
+        //mw->setSafeMode(safemode);
+        if (filename!="") mw->openDocFiles << filename;
 
         mw->show();
 
-        app->connect( app, SIGNAL( lastWindowClosed() ), app, SLOT( quit() ) );
+        app->connect(app, SIGNAL( lastWindowClosed() ), app, SLOT( quit()));
 
-// setup single shot timer to call loadEverything()
+#if defined(Q_WS_MAC)
+        connectOdocHandler();
+#endif
 
-        QTimer::singleShot( 500, mw, SLOT(startupLoad()) );
+        // setup single shot timer to call startupLoad()
+        QTimer::singleShot(0.5, mw, SLOT(startupLoad()));
 
         app->exec();
 
-        oe->hide();
-        fd->hide();
         mw->hide();  // must do this before settings object is destroyed
-
-        addOnLibs->save();  // ditto
+        if (mw->getAddOnLibs()!=NULL)
+            mw->getAddOnLibs()->save();  // ditto
 
         if ( st->getStartupAction()==1 )
         {
 /* save the state of the GUI (opened firewall, opened object tree page, etc */
-            FWObject *o=mw->getVisibleFirewall();
-            if (fwbdebug)
-                qDebug("Main: closing. VisibleFirewall = %p",o);
+            FWObject *o=mw->getVisibleFirewalls();
 
-            if (o) st->setStr("UI/visibleFirewall", o->getId().c_str() );
+            if (fwbdebug) qDebug("Main: closing. VisibleFirewall = %p",o);
 
-            o=om->getOpened();
-            if (o) st->setStr("UI/visibleObject",   o->getId().c_str() );
+            if (o) st->setStr("UI/visibleFirewall",
+                              FWObjectDatabase::getStringId(
+                                  o->getId()).c_str());
+
+            o=mw->getOpened();
+            if (o) st->setStr("UI/visibleObject",
+                              FWObjectDatabase::getStringId(
+                                  o->getId()).c_str());
         }
 
-
         st->save();
         delete st;
     }
@@ -717,3 +713,5 @@ int main( int argc, char ** argv )
         qDebug("Exception: %s",ex.toString().c_str());
     }
 }
+
+
