@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newGroupDialog.h,v 1.1 2004/06/07 04:02:55 vkurland Exp $
+  $Id: newGroupDialog.h 353 2008-07-13 01:26:39Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,18 +27,23 @@
 #ifndef __NEWGROUPDIALOG_H_
 #define __NEWGROUPDIALOG_H_
 
-#include "config.h"
-#include <newgroupdialog_q.h>
+#include "../../config.h"
+#include <ui_newgroupdialog_q.h>
+#include <QDialog>
 
 #include "fwbuilder/FWObject.h"
 
 
-class newGroupDialog : public newGroupDialog_q
+class newGroupDialog : public QDialog
 {
     Q_OBJECT
+            
 
  public:
     newGroupDialog(QWidget *parent);
+    ~newGroupDialog();
+    
+    Ui::newGroupDialog_q *m_dialog;
 
 public slots:
     virtual void accept();
