@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: pixosAdvancedDialog.h,v 1.1 2004/03/21 03:17:53 vkurland Exp $
+  $Id: pixosAdvancedDialog.h 304 2008-07-03 02:19:16Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,23 +28,26 @@
 #ifndef __PIXOSADVANCEDDIALOG_H_
 #define __PIXOSADVANCEDDIALOG_H_
 
-#include "pixosadvanceddialog_q.h"
+#include <ui_pixosadvanceddialog_q.h>
 #include "DialogData.h"
+#include <QDialog>
 
 namespace libfwbuilder {
     class FWObject;
 };
 
-class pixosAdvancedDialog : public pixosAdvancedDialog_q
+class pixosAdvancedDialog : public QDialog
 {
     Q_OBJECT
 
     libfwbuilder::FWObject *obj;
     DialogData             data;
+Ui::pixosAdvancedDialog_q *m_dialog;
 
  public:
     pixosAdvancedDialog(QWidget *parent,libfwbuilder::FWObject *o);
-
+    ~pixosAdvancedDialog();
+    
 protected slots:
 
     virtual void accept();
