@@ -1,4 +1,4 @@
-/* 
+/*
 
                           Firewall Builder
 
@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: printerStream.cpp,v 1.5 2008/05/09 04:29:28 vkurland Exp $
+  $Id: printerStream.cpp 466 2008-08-13 06:10:55Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -17,20 +17,20 @@
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
- 
+
   To get a copy of the GNU General Public License, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
 
 
-#include "config.h"
+#include "../../config.h"
 #include "global.h"
 #include "utils.h"
 
 #include "printerStream.h"
 
-#include <qscrollview.h>
+#include <QHeaderView>
 #include <qprinter.h>
 #include <qdatetime.h>
 
@@ -38,7 +38,7 @@ printerStream::printerStream(QPrinter *p,
                              float m,
                              bool h,
                              const QString &ht,
-                             PrintingProgressDialog *pd) : pr(),metrics(p)
+                             PrintingProgressDialog *pd) : pr()//,metrics(p)
 {
     printer = p;
     margin = m;
@@ -62,8 +62,8 @@ bool printerStream::begin()
         return false;
     active=true;
 
-    pageWidth = metrics.width();
-    pageHeight = metrics.height();
+    pageWidth = printer->width();
+    pageHeight = printer->height();
 
     if (fwbdebug)
     {
@@ -71,14 +71,14 @@ bool printerStream::begin()
         qDebug("Margin: %.1f", margin);
     }
 
-    dpiy = metrics.logicalDpiY();
+    dpiy = printer->logicalDpiY();
     ymargin = (int) ( (margin/2.54)*dpiy );
 // assuming printer's resolutions by X and Y axes are the same
     xmargin = ymargin;
-    pageBody = QRect( xmargin,
-                      ymargin,
-                      metrics.width() - 2*xmargin,
-                      metrics.height() - 2*ymargin );
+    pageBody=QRect( xmargin,
+                    ymargin,
+                    printer->width() - 2*xmargin,
+                    printer->height() - 2*ymargin );
 
     yHeaderHeight = int((headerHeight/2.54)*dpiy);
     yHeaderLine = int(((headerHeight-0.5)/2.54)*dpiy);
@@ -87,15 +87,15 @@ bool printerStream::begin()
     QFontMetrics fm = pr.fontMetrics();
     QRect br = fm.boundingRect("Page 999");
 
-    headerTextBox = QRect(xmargin,
-                          ymargin + yHeaderLine - fm.lineSpacing() - 1,
-                          metrics.width() - 2 * xmargin,
-                          fm.lineSpacing() + 1);
+    headerTextBox=QRect(xmargin,
+                        ymargin + yHeaderLine - fm.lineSpacing() - 1,
+                        printer->width() - 2 * xmargin,
+                        fm.lineSpacing() + 1);
 
-    headerBox = QRect(xmargin,
-                      ymargin,
-                      metrics.width() - 2 * xmargin,
-                      yHeaderHeight);
+    headerBox=QRect(xmargin,
+                    ymargin,
+                    printer->width() - 2 * xmargin,
+                    yHeaderHeight);
 
     if (fwbdebug)
     {
@@ -115,7 +115,7 @@ bool printerStream::begin()
 
     yPos = 0;
     pageNo = 1;
-     
+
     return true;
 }
 
@@ -171,13 +171,13 @@ void printerStream::flushPage()
 int printerStream::getTextHeight(const QString &txt)
 {
     if (txt.isEmpty()) return 0;
-    if (printer->aborted()) return 0;
+    if (printer->printerState() == QPrinter::Aborted) return 0;
 
     pr.setFont( bodyFont );
     QFontMetrics fm = pr.fontMetrics();
     int nlines=1;
     int i=-1;
-    while ( (i=txt.find("\n",i+1))>=0 ) nlines++;
+    while ( (i=txt.indexOf("\n",i+1))>=0 ) nlines++;
     return nlines*fm.lineSpacing();
 }
 
@@ -185,13 +185,13 @@ void printerStream::printText(const QString &txt, bool newLine)
 {
     if (fwbdebug)
     {       
-        qDebug("printText ------- %s", txt.ascii());
+        qDebug("printText -------");
         qDebug("pageBody.height(): %d", pageBody.height());
         qDebug("yPos: %d", yPos);
     }
 
     if (txt.isEmpty()) return;
-    if (printer->aborted()) return;
+    if (printer->printerState() == QPrinter::Aborted) return;
 
     pr.setFont( bodyFont );
     QFontMetrics fm = pr.fontMetrics();
@@ -206,13 +206,13 @@ void printerStream::printText(const QString &txt, bool newLine)
     if (pageNo>=fromPage && pageNo<=toPage)
     {
         pr.setPen(Qt::black);
-        pr.drawText( xmargin, yPos, metrics.width()-2*xmargin, br.height(),
-                     Qt::ExpandTabs | Qt::DontClip,
+        pr.drawText( xmargin, yPos, printer->width()-2*xmargin, br.height(),
+                     Qt::TextExpandTabs | Qt::TextDontClip,
                      txt );
     }
     int nlines=1;
     int i=-1;
-    while ( (i=txt.find("\n",i+1))>=0 ) nlines++;
+    while ( (i=txt.indexOf("\n",i+1))>=0 ) nlines++;
     if (newLine) yPos = yPos + nlines*fm.lineSpacing();
 }
 
@@ -233,29 +233,30 @@ void printerStream::printPixmap(const QPixmap &pm, bool newLine)
     if (newLine) yPos = yPos + (pm.height()-pmYOffset);
 }
 
-void printerStream::printQTable(QTable *tbl, bool left_margin, bool top_margin)
+void printerStream::printQTable(QTableView *tbl, bool left_margin, bool top_margin)
 {
-
     if (fwbdebug)
-    {       
+    {
         qDebug("printQTable ----------------------------------------------");
         qDebug("Size: %dx%d",tbl->width(),tbl->height());
-        qDebug("Contents: %dx%d",tbl->contentsWidth(),tbl->contentsHeight());
-        qDebug("Visible: %dx%d",tbl->visibleWidth(),tbl->visibleHeight());
+        qDebug("Visible: %dx%d",tbl->contentsRect().width(),tbl->contentsRect().height());
         qDebug("Viewport: %dx%d",
                tbl->viewport()->width(),tbl->viewport()->height());
-        qDebug("Clipper: %dx%d",
-               tbl->clipper()->width(),tbl->clipper()->height());
+        /*qDebug("Clipper: %dx%d",
+               tbl->clipper()->width(),tbl->clipper()->height());*/
+    }
+
         qDebug("YSpace: %d", getYSpace());
         qDebug("pageBody.height(): %d", pageBody.height());
         qDebug("yPos: %d", yPos);
-    }
-
-
     int firstRow = 0;
     int lastRow = 1;
     int tblHeight = tbl->horizontalHeader()->height();
 
+    int columnsWidth = 0;
+    for (int i = 0; i < tbl->model()->columnCount();
+         columnsWidth += tbl->columnWidth(i), i++) ;
+
     if ( tblHeight + tbl->rowHeight(0) > getYSpace() )
     {
         // even one row of the table won't fit on the space left on page
@@ -263,17 +264,18 @@ void printerStream::printQTable(QTable *tbl, bool left_margin, bool top_margin)
         beginPage();
     }
 
-    while (firstRow<=(tbl->numRows()-1))
+    int rowCount = tbl->model()->rowCount();
+    while (firstRow<=(rowCount-1))
     {
         int row = 0;
-        for (row=firstRow; row < tbl->numRows(); ++row)
+        for (row=firstRow; row < rowCount; ++row)
         {
             int nth = tblHeight + tbl->rowHeight(row);
             if ( nth==getYSpace() )  break;
             if ( nth>getYSpace() ) { row--; break; }
             tblHeight  = nth;
-        }
 
+        }
         // if row < firstRow then even single row does not fit on the page
         if (row < firstRow)
         {
@@ -281,20 +283,19 @@ void printerStream::printQTable(QTable *tbl, bool left_margin, bool top_margin)
             tblHeight = tbl->rowHeight(firstRow);
         }
 
-        if (row == tbl->numRows()) row--;
+        if (row == rowCount) row--;
 
         lastRow = row;
-
         if (fwbdebug)
             qDebug("Page %d -- (%d-%d of %d rows) tblHeight: %d",
-                   pageNo, firstRow, lastRow, tbl->numRows(), tblHeight);
+                   pageNo, firstRow, lastRow, rowCount, tblHeight);
 
-        int firstRowPos = tbl->rowPos(firstRow);
-        int lastRowPos = tbl->rowPos(lastRow);
-           
-        if (fwbdebug)
-            qDebug("    firstRowPos: %d lastRowPos: %d",
-                   firstRowPos, lastRowPos);
+
+        int firstRowPos = tbl->verticalHeader()->sectionPosition(firstRow);
+        int lastRowPos = tbl->verticalHeader()->sectionPosition(lastRow);
+
+        qDebug("    firstRowPos: %d lastRowPos: %d",
+               firstRowPos, lastRowPos);
 
         int left_hdr_w = 0;
         if (left_margin && tbl->verticalHeader() != NULL)
@@ -304,19 +305,7 @@ void printerStream::printQTable(QTable *tbl, bool left_margin, bool top_margin)
         if (top_margin && tbl->horizontalHeader() != NULL)
             top_hdr_h = tbl->horizontalHeader()->height();
 
-        int tbl_w = tbl->contentsWidth() + left_hdr_w;
-        int tbl_h = tblHeight;
-        int clip_w = getWorkspaceWidth();
-        int clip_h = tblHeight - top_hdr_h;
-
-        if (fwbdebug) qDebug("    Resize table to %dx%d", tbl_w, tbl_h);
-        tbl->resize(tbl_w, tbl_h);
-        int n_tbl_w = tbl->width();
-        int n_tbl_h = tbl->height();
-        if (fwbdebug) qDebug("    New table size: %dx%d", n_tbl_w, n_tbl_h);
-
-        if (fwbdebug) qDebug("    Resize clipper to %dx%d", clip_w, clip_h);
-        tbl->clipper()->resize(clip_w, clip_h);
+        tbl->resize(columnsWidth + left_hdr_w, tblHeight);
 
         tbl->verticalHeader()->resize(
             tbl->verticalHeader()->width(),
@@ -326,43 +315,42 @@ void printerStream::printQTable(QTable *tbl, bool left_margin, bool top_margin)
             tbl->horizontalHeader()->height());
 
         if (fwbdebug)
-        {       
+        {
             qDebug("    ### After resize:");
             qDebug("    Size: %dx%d",tbl->width(),tbl->height());
-            qDebug("    Contents: %dx%d",
-                   tbl->contentsWidth(),tbl->contentsHeight());
             qDebug("    Visible: %dx%d",
+                   tbl->contentsRect().width(),tbl->contentsRect().height());
+            /*qDebug("    Visible: %dx%d",
                    tbl->visibleWidth(),tbl->visibleHeight());
             qDebug("    Viewport: %dx%d",
                    tbl->viewport()->width(),tbl->viewport()->height());
             qDebug("    Clipper: %dx%d",
-                   tbl->clipper()->width(),tbl->clipper()->height());
+                   tbl->clipper()->width(),tbl->clipper()->height());*/
             qDebug("    vheader size: %dx%d",
                    tbl->verticalHeader()->width(),
                    tbl->verticalHeader()->height());
         }
 
-        tbl->setContentsPos(0,firstRowPos);
-        tbl->updateHeaderStates();
-        tbl->verticalHeader()->setOffset(firstRowPos);
+        tbl->verticalHeader()->setOffsetToSectionPosition(firstRow);
+        tbl->update();
 
         if (fwbdebug)
         {
             qDebug("    ### After scroll:");
-            qDebug("    contents X: %d  contents Y: %d",
-                   tbl->contentsX(),tbl->contentsY());
+            /* qDebug("    contents X: %d  contents Y: %d",
+                   tbl->horizontalOffset(),tbl->verticalOffset()); */
             int count = tbl->verticalHeader()->count();
             int offset= tbl->verticalHeader()->offset();
             qDebug("    vheader count: %d",count);
             qDebug("    vheader offset: %d",offset);
             qDebug("    vheader sectionAt(%d): %d",
                    offset,
-                   tbl->verticalHeader()->sectionAt(offset));
+                   tbl->verticalHeader()->logicalIndexAt(offset));
         }
 
         printPixmap(QPixmap::grabWidget(tbl,0,0,-1,tblHeight));
 
-        if (lastRow>=(tbl->numRows()-1)) break;
+        if (lastRow>=(rowCount-1)) break;
 
         flushPage();
         beginPage();
