@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: printerStream.h,v 1.3 2007/06/21 05:43:26 vkurland Exp $
+  $Id: printerStream.h 304 2008-07-03 02:19:16Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,9 +28,8 @@
 #define  __PRINTERSTREAM_H_
 
 #include <qstring.h>
-#include <qtable.h>
 #include <qpainter.h>
-#include <qpaintdevicemetrics.h>
+#include <qtableview.h>
 
 #include <vector>
 
@@ -41,7 +40,6 @@ class QPrinter;
 class printerStream {
     QPrinter *printer;
     QPainter  pr;
-    QPaintDeviceMetrics metrics;
     QRect     pageBody;
     PrintingProgressDialog *ppd;
 
@@ -85,7 +83,7 @@ class printerStream {
 
     void printText(const QString &txt, bool newLine=true);
     void printPixmap(const QPixmap &pm, bool newLine=true);
-    void printQTable(QTable *tbl, bool left_margin=true, bool top_margin=true);
+    void printQTable(QTableView *tbl, bool left_margin=true, bool top_margin=true);
 
     int  getTextHeight(const QString &txt);
 
