@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: upgradePredicate.h,v 1.3 2004/12/08 08:06:30 vkurland Exp $
+  $Id: upgradePredicate.h 418 2008-07-27 01:34:07Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -38,16 +38,16 @@ class MessageBoxUpgradePredicate: public libfwbuilder::XMLTools::UpgradePredicat
  public:
     MessageBoxUpgradePredicate(QWidget *p=NULL) { parent=p; }
     
-    virtual bool operator()(const std::string &msg) const 
+    virtual bool operator()(const std::string&) const 
     { 
         return QMessageBox::information( parent , "Firewall Builder",
                                          QObject::tr(
-"The data file you are trying to open has been\n\
-saved with an older version of Firewall Builder.\n\
-Opening it in this version will cause it to be\n\
-upgraded, which may prevent older versions of\n\
-the program from reading it. Backup copy of your\n\
-file in the old format will be made in the same\n\
+"The data file you are trying to open has been \
+saved with an older version of Firewall Builder. \
+Opening it in this version will cause it to be \
+upgraded, which may prevent older versions of \
+the program from reading it. Backup copy of your \
+file in the old format will be made in the same \
 directory with extension '.bak'.\n\
 Are you sure you want to open it?"),
                                          QObject::tr("&Upgrade"),
