@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipf.cpp,v 1.27 2006/09/21 06:29:43 vkurland Exp $
+  $Id: ipf.cpp 443 2008-08-03 16:47:47Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -23,10 +23,8 @@
 
 */
 
-#include "config.h"
+#include "../../config.h"
  
-#include <qsettings.h>
-
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -51,6 +49,7 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <assert.h>
+#include <cstring>
 
 #include "PolicyCompiler_ipf.h"
 #include "NATCompiler_ipf.h"
@@ -84,6 +83,8 @@ using namespace std;
 using namespace libfwbuilder;
 using namespace fwcompiler;
 
+int fwbdebug = 0;
+
 static const char      *filename       = NULL;
 static const char      *wdir           = NULL;
 static const char      *fwobjectname   = NULL;
@@ -101,7 +102,7 @@ FWObjectDatabase       *objdb = NULL;
 class UpgradePredicate: public XMLTools::UpgradePredicate
 {
     public:
-    virtual bool operator()(const string &msg) const 
+    virtual bool operator()(const string&) const 
     { 
 	cout << _("Data file has been created in the old version of Firewall Builder. Use fwbuilder GUI to convert it.") << endl;
 	return false;
@@ -265,8 +266,8 @@ int main(int argc, char * const *argv)
 //	objdb->load(filename,  &upgrade_predicate, librespath);
 	if (verbose) cout << _(" done\n");
 
-        FWObject *slib = objdb->getById("syslib000");
-        if ( slib->isReadOnly()) slib->setReadOnly(false);
+        FWObject *slib = objdb->getById(FWObjectDatabase::STANDARD_LIB_ID);
+        if (slib && slib->isReadOnly()) slib->setReadOnly(false);
 
 	/* Review firewall and OS options and generate commands */
 	Firewall*  fw=objdb->findFirewallByName(fwobjectname);
@@ -317,7 +318,7 @@ int main(int argc, char * const *argv)
                 for (list<FWObject*>::iterator j=la.begin(); j!=la.end(); ++j) 
                 {
                     IPv4 *ipv4 = IPv4::cast(*j);
-                    if ( ipv4->getAddress().toString()=="0.0.0.0")
+                    if ( ipv4->getAddressPtr()->isAny())
                     {
                         char errstr[256];
                         sprintf(errstr,
@@ -342,7 +343,7 @@ int main(int argc, char * const *argv)
 	string shell_dbg=(debug)?"-x":"" ;
 	string ipf_dbg=(debug)?"-v":"";
 
-        Preprocessor* prep=new Preprocessor(objdb , fwobjectname);
+        Preprocessor* prep=new Preprocessor(objdb , fwobjectname, false);
         prep->compile();
 
 /*
@@ -351,13 +352,13 @@ int main(int argc, char * const *argv)
 	OSConfigurator *oscnf=NULL;
         string family=Resources::os_res[fw->getStr("host_OS")]->Resources::getResourceStr("/FWBuilderResources/Target/family");
         if ( family=="solaris" )
-            oscnf=new OSConfigurator_solaris(objdb , fwobjectname);
+            oscnf=new OSConfigurator_solaris(objdb , fwobjectname, false);
 
 	if ( family=="openbsd")
-            oscnf=new OSConfigurator_openbsd(objdb , fwobjectname);
+            oscnf=new OSConfigurator_openbsd(objdb , fwobjectname, false);
 
 	if ( family=="freebsd")
-            oscnf=new OSConfigurator_freebsd(objdb , fwobjectname);
+            oscnf=new OSConfigurator_freebsd(objdb , fwobjectname, false);
 
 	if (oscnf==NULL)
 	    throw FWException(_("Unrecognized host OS ")+fw->getStr("host_OS")+"  (family "+family+")");
@@ -367,7 +368,7 @@ int main(int argc, char * const *argv)
 /*
  * create compilers and run the whole thing 
  */
-	PolicyCompiler_ipf c( objdb , fwobjectname , oscnf );
+	PolicyCompiler_ipf c( objdb , fwobjectname, false , oscnf );
 
 	c.setDebugLevel( dl );
 	c.setDebugRule(  drp );
@@ -382,7 +383,7 @@ int main(int argc, char * const *argv)
 	}
 
 
-	NATCompiler_ipf n( objdb , fwobjectname , oscnf );
+	NATCompiler_ipf n( objdb , fwobjectname, false , oscnf );
 
 	n.setDebugLevel( dl );
 	n.setDebugRule(  drn );
@@ -511,6 +512,14 @@ int main(int argc, char * const *argv)
 #else
             ipf_file.open(ipf_file_name.c_str());
 #endif
+
+            if (c.haveErrorsAndWarnings())
+            {
+                ipf_file << "# Policy compiler errors and warnings:"
+                         << endl;
+                ipf_file << c.getErrors("# ");
+            }
+
 	    ipf_file << c.getCompiledScript();
 	    ipf_file.close();
 
@@ -538,6 +547,15 @@ int main(int argc, char * const *argv)
 #else
             nat_file.open(nat_file_name.c_str());
 #endif
+
+            if (n.haveErrorsAndWarnings())
+            {
+                nat_file << "# NAT compiler errors and warnings:"
+                         << endl;
+                nat_file << n.getErrors("# ");
+            }
+
+
 	    nat_file << n.getCompiledScript();
 	    nat_file.close();
 
