@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_PrintRule.cpp,v 1.14 2006/10/08 00:10:02 vkurland Exp $
+  $Id: NATCompiler_PrintRule.cpp 479 2008-08-19 18:55:57Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -41,6 +41,7 @@
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/AddressTable.h"
+#include "fwbuilder/XMLTools.h"
 
 #include "combinedAddress.h"
 
@@ -68,16 +69,35 @@ using namespace std;
  *-----------------------------------------------------------------------
  *                    Methods for printing
  */
+
+void NATCompiler_ipt::PrintRule::InitializeMinusNTracker()
+{
+    NATCompiler_ipt *ipt_comp = dynamic_cast<NATCompiler_ipt*>(compiler);
+    for (list<string>::const_iterator i =
+             NATCompiler_ipt::getStandardChains().begin();
+         i != NATCompiler_ipt::getStandardChains().end(); ++i)
+    {
+        (*(ipt_comp->minus_n_commands))[*i] = true;
+    }
+    minus_n_tracker_initialized = true;
+}
+            
+
+
 /*
  *  check and create new chain if needed
  */
 string NATCompiler_ipt::PrintRule::_createChain(const string &chain)
 {
+    NATCompiler_ipt *ipt_comp = dynamic_cast<NATCompiler_ipt*>(compiler);
     ostringstream  res;
-    if ( ! chains[chain] )
+
+    if (!minus_n_tracker_initialized) InitializeMinusNTracker();
+
+    if ( ipt_comp->minus_n_commands->count(chain)==0 )
     {
 	res << "$IPTABLES -t nat -N " << chain << endl;
-	chains[chain]=true;
+	(*(ipt_comp->minus_n_commands))[chain] = true;
     }
     return res.str();
 }
@@ -267,8 +287,8 @@ string NATCompiler_ipt::PrintRule::_printSrcPorts(Service *srv)
 {
     std::ostringstream  ostr;
     if (TCPService::isA(srv) || UDPService::isA(srv)) {
-	int rs=srv->getInt("src_range_start");
-	int re=srv->getInt("src_range_end");
+	int rs=TCPUDPService::cast(srv)->getSrcRangeStart();
+	int re=TCPUDPService::cast(srv)->getSrcRangeEnd();
 	ostr << _printOPorts(rs,re);
     }
     return ostr.str();
@@ -278,8 +298,8 @@ string NATCompiler_ipt::PrintRule::_printDstPorts(Service *srv)
 {
     std::ostringstream  ostr;
     if (TCPService::isA(srv) || UDPService::isA(srv)) {
-	int rs=srv->getInt("dst_range_start");
-	int re=srv->getInt("dst_range_end");
+	int rs=TCPUDPService::cast(srv)->getDstRangeStart();
+	int re=TCPUDPService::cast(srv)->getDstRangeEnd();
 	ostr << _printOPorts(rs,re);
     }
     return ostr.str();
@@ -289,8 +309,8 @@ string NATCompiler_ipt::PrintRule::_printSNATPorts(Service *srv)
 {
     std::ostringstream  ostr;
     if (TCPService::isA(srv) || UDPService::isA(srv)) {
-	int rs=srv->getInt("src_range_start");
-	int re=srv->getInt("src_range_end");
+	int rs=TCPUDPService::cast(srv)->getSrcRangeStart();
+	int re=TCPUDPService::cast(srv)->getSrcRangeEnd();
 	ostr << _printTPorts(rs,re);
     }
     return ostr.str();
@@ -300,8 +320,8 @@ string NATCompiler_ipt::PrintRule::_printDNATPorts(Service *srv)
 {
     std::ostringstream  ostr;
     if (TCPService::isA(srv) || UDPService::isA(srv)) {
-	int rs=srv->getInt("dst_range_start");
-	int re=srv->getInt("dst_range_end");
+	int rs=TCPUDPService::cast(srv)->getDstRangeStart();
+	int re=TCPUDPService::cast(srv)->getDstRangeEnd();
 	ostr << _printTPorts(rs,re);
     }
     return ostr.str();
@@ -348,7 +368,8 @@ string NATCompiler_ipt::PrintRule::_printSrcService(RuleElementOSrv  *rel)
 	if ( !str.empty() )
         {
             string v = compiler->fw->getStr("version");
-            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
+            if (v.empty() || v=="ge_1.2.6" ||
+                XMLTools::version_compare(v, "1.2.6")>=0)
                 ostr << "--sports ";
             else
                 ostr << "--source-port ";
@@ -392,7 +413,7 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
         if (TagService::isA(srv))
         {
 	    ostr << "-m mark --mark "
-                 << TagService::cast(srv)->getCode() << " ";
+                 << TagService::constcast(srv)->getCode() << " ";
         }
 
     } else {
@@ -417,7 +438,9 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
 	if ( !str.empty() )
         {
             string v = compiler->fw->getStr("version");
-            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
+//            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
+            if (v.empty() || v=="ge_1.2.6" ||
+                XMLTools::version_compare(v, "1.2.6")>=0)
                 ostr << "--dports ";
             else
                 ostr << "--destination-port ";
@@ -427,7 +450,10 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
     return ostr.str();
 }
 
-string NATCompiler_ipt::PrintRule::_printAddr(Address  *o,bool print_mask,bool print_range)
+// Note print_mask is true by default, print_range is false by default.
+string NATCompiler_ipt::PrintRule::_printAddr(Address  *o,
+                                              bool print_mask,
+                                              bool print_range)
 {
     NATCompiler_ipt *ipt_comp=dynamic_cast<NATCompiler_ipt*>(compiler);
     std::ostringstream  ostr;
@@ -453,70 +479,70 @@ string NATCompiler_ipt::PrintRule::_printAddr(Address  *o,bool print_mask,bool p
         assert(atrt==NULL);
     }
 
-    IPAddress addr=o->getAddress();
-    Netmask   mask=o->getNetmask();
-    Interface *iface;
-    if ( (iface=Interface::cast(o))!=NULL ) 
+    if (print_range && AddressRange::cast(o)!=NULL)
     {
-        if (iface->isDyn() && iface->getBool("use_var_address"))
+	InetAddr a1 = AddressRange::cast(o)->getRangeStart();
+	InetAddr a2 = AddressRange::cast(o)->getRangeEnd();
+	ostr << a1.toString() << "-" << a2.toString();
+    } else
+    {
+        Interface *iface = Interface::cast(o);
+        if (iface!=NULL && iface->isDyn() && iface->getBool("use_var_address"))
         {
-            ostr << "$" << ipt_comp->getInterfaceVarName(iface) << " ";
+            ostr << "$" << ipt_comp->getInterfaceVarName(iface, ipt_comp->ipv6)
+                 << " ";
             return ostr.str();
         }
-//        if (Interface::cast(o)->isDyn()) return;
-	mask=Netmask("255.255.255.255");
-    }
 
-    if (IPv4::cast(o)!=NULL) 
-    {
-	mask=Netmask("255.255.255.255");
-    }
+        const InetAddr* addr = o->getAddressPtr();
+        const InetAddr* mask = o->getNetmaskPtr();
 
-    if (print_range && AddressRange::cast(o)!=NULL) {
-	IPAddress a1=AddressRange::cast(o)->getRangeStart();
-	IPAddress a2=AddressRange::cast(o)->getRangeEnd();
-	ostr << a1.toString() << "-" << a2.toString();
-    } else {
-	if (addr.toString()=="0.0.0.0" && mask.toString()=="0.0.0.0") {
-	    ostr << "0/0";
-	} else {	
-	    ostr << addr.toString();
-	    if (print_mask && mask.toString()!="255.255.255.255") {
-		ostr << "/" << mask.getLength();
-	    }
-	}
+        if (addr==NULL)
+        {
+            compiler->warning(
+                string("Empty inet address in object ") +
+                FWObjectDatabase::getStringId(o->getId()));
+            return ostr.str();
+        }
+
+        if (addr->isAny() && mask->isAny())
+        {
+            ostr << "0/0 ";
+        } else 
+        {
+            ostr << addr->toString();
+
+            if (Interface::cast(o)==NULL &&
+                Address::cast(o)->dimension() > 1 &&
+                !mask->isHostMask())
+            {
+                ostr << "/" << mask->getLength();
+            }
+            // do not add space after address because there might be
+            // :port spec coming right after it.
+        }
     }
     return ostr.str();
 }
 
-
 NATCompiler_ipt::PrintRule::PrintRule(const std::string &name) : 
     NATRuleProcessor(name)
 {
-    init=true; 
-    print_once_on_top=true;
-
-    chains["POSTROUTING"]  =true;
-    chains["PREROUTING"]   =true;
-    chains["SNAT"]         =true;
-    chains["DNAT"]         =true;
-    chains["MASQUERADE"]   =true;
-    chains["REDIRECT"]     =true;
-    chains["NETMAP"]       =true;
-    chains["LOG"]          =true;
-    chains["MARK"]         =true;
-    chains["ACCEPT"]       =true;
-    chains["REJECT"]       =true;
-    chains["DROP"]         =true;
-    chains["RETURN"]       =true;
-    chains["OUTPUT"]       =true;
+    init = true; 
+    print_once_on_top = true;
+    minus_n_tracker_initialized = false;
 }
 
 bool NATCompiler_ipt::PrintRule::processNext()
 {
+    NATCompiler_ipt *ipt_comp = dynamic_cast<NATCompiler_ipt*>(compiler);
     NATRule *rule=getNext(); 
     if (rule==NULL) return false;
 
+    string chain = rule->getStr("ipt_chain");
+    if (ipt_comp->chain_usage_counter[chain] == 0)
+        return true;
+
     tmp_queue.push_back(rule);
 
     compiler->output << _printRuleLabel(rule);
@@ -567,17 +593,12 @@ bool NATCompiler_ipt::PrintRule::processNext()
 	    cmdout << " -m mac --mac-source "
                              << physaddress;
         }
-/*
- * fool-proof: this is last resort check for situation when user created IPv4 object
- * for the interface but left it with empty address ( 0.0.0.0 ). 
- */
-        if ( ! physaddress.empty() && osrc->getAddress()==IPAddress("0.0.0.0"))
+
+        const InetAddr *osrc_addr = osrc->getAddressPtr();
+        if (osrc_addr==NULL || !osrc_addr->isAny())
         {
-            ;
-        } else
-        {
-            cmdout << " -s ";
-            cmdout << _printAddr(osrc);
+            string osrc_out = _printAddr(osrc);
+            if (!osrc_out.empty()) cmdout << " -s " << osrc_out;
         }
 
 //	cmdout << " -s ";
@@ -657,7 +678,8 @@ bool NATCompiler_ipt::PrintRule::processNext()
 //    cmdout  << endl;
 
     compiler->output 
-        << dynamic_cast<OSConfigurator_linux24*>(compiler->osconfigurator)->printRunTimeWrappers( rule, cmdout.str());
+        << dynamic_cast<OSConfigurator_linux24*>(compiler->osconfigurator)->
+        printRunTimeWrappers( rule, cmdout.str(), ipt_comp->ipv6);
 
     return true;
 }
