@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipt.cpp,v 1.27 2007/10/29 06:33:14 vkurland Exp $
+  $Id: NATCompiler_ipt.cpp 468 2008-08-14 02:07:36Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -53,6 +53,7 @@
 #include <functional>
 #include <iomanip>
 #include <iostream>
+#include <cstring>
 
 #include <assert.h>
 #include <ctype.h>
@@ -75,17 +76,41 @@ static int chain_no=0;
 #endif
 
 static std::map<std::string,int> tmp_chain_no;
+static std::list<std::string> standard_chains;
+
+const std::list<std::string>& NATCompiler_ipt::getStandardChains()
+{
+    if (standard_chains.size()==0)
+    {
+        standard_chains.push_back("POSTROUTING");
+        standard_chains.push_back("PREROUTING");
+        standard_chains.push_back("SNAT");
+        standard_chains.push_back("DNAT");
+        standard_chains.push_back("MASQUERADE");
+        standard_chains.push_back("REDIRECT");
+        standard_chains.push_back("NETMAP");
+        standard_chains.push_back("LOG");
+        standard_chains.push_back("MARK");
+        standard_chains.push_back("ACCEPT");
+        standard_chains.push_back("REJECT");
+        standard_chains.push_back("DROP");
+        standard_chains.push_back("RETURN");
+        standard_chains.push_back("OUTPUT");
+    }
+    return standard_chains;
+}
+
 
 string NATCompiler_ipt::myPlatformName() { return "iptables"; }
 
-string NATCompiler_ipt::getInterfaceVarName(FWObject *iface)
+string NATCompiler_ipt::getInterfaceVarName(FWObject *iface, bool v6)
 {
     ostringstream  ostr;
     string iname=iface->getName();
     string::size_type p1;
-    while ( (p1=iname.find("."))!=string::npos)
-        iname=iname.replace(p1,1,"_");
+    while ( (p1=iname.find("."))!=string::npos) iname=iname.replace(p1,1,"_");
     ostr << "i_" << iname;
+    if (v6) ostr << "_v6";
     return ostr.str();
 }
 
@@ -130,15 +155,23 @@ string NATCompiler_ipt::debugPrintRule(Rule *r)
     NATRule *rule=NATRule::cast(r);
 
     return NATCompiler::debugPrintRule(rule)+
-        " "+rule->getInterfaceId()+
-        " "+rule->getStr("ipt_chain")+
-        " "+rule->getStr("ipt_target")+
+        " " + FWObjectDatabase::getStringId(rule->getInterfaceId()) +
+        " " + rule->getStr("ipt_chain") +
+        " " + rule->getStr("ipt_target") +
         " (type="+rule->getRuleTypeAsString()+")";
 }
 
 
 int NATCompiler_ipt::prolog()
 {
+    // initialize counters for the standard chains
+    for (list<string>::const_iterator i =
+             NATCompiler_ipt::getStandardChains().begin();
+         i != NATCompiler_ipt::getStandardChains().end(); ++i)
+    {
+        chain_usage_counter[*i] = 1;
+    }
+
     int n=NATCompiler::prolog();
 
     if ( n>0 ) 
@@ -189,16 +222,20 @@ void NATCompiler_ipt::_expandInterface(Interface *iface,
     {
         if (physAddress::cast(*j)!=NULL) continue;
 
-        IPv4 *ipv4=IPv4::cast(*j);
-        if (ipv4!=NULL && use_mac && pa!=NULL)
+        //const InetAddrMask *ipv4 = Address::cast(*j)->getAddressObjectInetAddrMask();
+        const InetAddr *ip_addr = Address::cast(*j)->getAddressPtr();
+        const InetAddr *ip_netm = Address::cast(*j)->getNetmaskPtr();
+
+
+        if (ip_addr!=NULL && use_mac && pa!=NULL)
         {
-            combinedAddress *ca=new combinedAddress(dbcopy,true);
+            combinedAddress *ca = new combinedAddress(dbcopy,true);
             dbcopy->add(ca);
             dbcopy->addToIndex(ca);
             cacheObj(ca);
             ca->setName( "CA("+iface->getName()+")" );
-            ca->setAddress( ipv4->getAddress() );
-            ca->setNetmask( ipv4->getNetmask() );
+            ca->setAddress( *ip_addr );
+            ca->setNetmask( *ip_netm );
             ca->setPhysAddress( pa->getPhysAddress() );
             nol.push_back(ca);
         } else
@@ -208,6 +245,12 @@ void NATCompiler_ipt::_expandInterface(Interface *iface,
     ol=nol;
 }
 
+bool compare_addresses_ptr(const InetAddr* a1, const InetAddr* a2)
+{
+    return (*a1 < *a2);
+}
+
+
 /*
  * call this processor after classifyNATRules
  */
@@ -221,21 +264,22 @@ bool NATCompiler_ipt::ConvertLoadBalancingRules::processNext()
     {
         RuleElementTDst  *tdst=rule->getTDst();  assert(tdst);
 
-        list<IPAddress> al;
+        list<const InetAddr*> al;
         for(list<FWObject*>::iterator i=tdst->begin(); i!=tdst->end(); i++) 
         {
             FWObject *o= *i;
             FWObject *obj = NULL;
-            if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
-            Address *a=Address::cast(obj);
-
-            al.push_back( a->getAddress() );
+            if (FWReference::cast(o)!=NULL)
+                obj=FWReference::cast(o)->getPointer();
+            //const InetAddrMask *a = Address::cast(obj)->getAddressObjectInetAddrMask();
+            const InetAddr *ip_addr = Address::cast(obj)->getAddressPtr();
+            al.push_back( ip_addr );
         }
 
-        al.sort();
+        al.sort(compare_addresses_ptr);
 
-        IPAddress a1=al.front();
-        list<IPAddress>::iterator j=al.begin();
+        const InetAddr* a1 = al.front();
+        list<const InetAddr*>::iterator j=al.begin();
         j++;
 
         for ( ; j!=al.end(); j++)
@@ -244,21 +288,23 @@ bool NATCompiler_ipt::ConvertLoadBalancingRules::processNext()
  * big endian/little endian conversion for me 
  */
             AddressRange tar;
-            tar.setRangeStart( a1 );
-            tar.setRangeEnd( *j );
+            tar.setRangeStart( *a1 );
+            tar.setRangeEnd( *(*j) );
             if ( tar.dimension() != 2 )
             {
                 compiler->abort(
                     string( _("Non-contiguous address range in Translated Destination in load balancing NAT rule ") )+
                     rule->getLabel());
             }
-            a1= *j;
+            a1 = *j;
         }
 
-        AddressRange *ar= AddressRange::cast(compiler->dbcopy->create(AddressRange::TYPENAME) );
-        ar->setRangeStart( al.front() );
-        ar->setRangeEnd( al.back() );
-        ar->setName(string("%")+al.front().toString()+"-"+al.back().toString()+"%" );
+        AddressRange *ar = AddressRange::cast(
+            compiler->dbcopy->create(AddressRange::TYPENAME) );
+        ar->setRangeStart( *(al.front()) );
+        ar->setRangeEnd( *(al.back()) );
+        ar->setName(string("%")+al.front()->toString()
+                    +"-"+al.back()->toString()+"%" );
         compiler->cacheObj(ar); // to keep cache consistent
         compiler->dbcopy->add(ar,false);
         tdst->clearChildren();
@@ -446,7 +492,7 @@ bool NATCompiler_ipt::VerifyRules::processNext()
         Network *a1=Network::cast(compiler->getFirstOSrc(rule));
         Network *a2=Network::cast(compiler->getFirstTSrc(rule));
         if ( a1==NULL || a2==NULL ||
-             a1->getNetmask().getLength()!=a2->getNetmask().getLength() )
+             a1->getNetmaskPtr()->getLength() != a2->getNetmaskPtr()->getLength() )
             throw FWException(_("Original and translated source should both be networks of the same size . Rule ")+rule->getLabel());
     }
 
@@ -455,7 +501,7 @@ bool NATCompiler_ipt::VerifyRules::processNext()
         Network *a1=Network::cast(compiler->getFirstODst(rule));
         Network *a2=Network::cast(compiler->getFirstTDst(rule));
         if ( a1==NULL || a2==NULL ||
-             a1->getNetmask().getLength()!=a2->getNetmask().getLength() )
+             a1->getNetmaskPtr()->getLength() != a2->getNetmaskPtr()->getLength() )
             throw FWException(_("Original and translated destination should both be networks of the same size . Rule ")+rule->getLabel());
     }
 
@@ -1164,10 +1210,10 @@ bool NATCompiler_ipt::separatePortRanges::processNext()
 	assert(s!=NULL);
 
 	if ( TCPService::isA(s) || UDPService::isA(s) ) {
-            int srs=s->getInt("src_range_start");
-            int sre=s->getInt("src_range_end");
-            int drs=s->getInt("dst_range_start");
-            int dre=s->getInt("dst_range_end");
+            int srs=TCPUDPService::cast(s)->getSrcRangeStart();
+            int sre=TCPUDPService::cast(s)->getSrcRangeEnd();
+            int drs=TCPUDPService::cast(s)->getDstRangeStart();
+            int dre=TCPUDPService::cast(s)->getDstRangeEnd();
 
             compiler->normalizePortRange(srs,sre);
             compiler->normalizePortRange(drs,dre);
@@ -1217,8 +1263,8 @@ bool NATCompiler_ipt::separateSourcePorts::processNext()
 	assert(s!=NULL);
 
 	if ( TCPService::isA(s) || UDPService::isA(s) ) {
-            int srs=s->getInt("src_range_start");
-            int sre=s->getInt("src_range_end");
+            int srs=TCPUDPService::cast(s)->getSrcRangeStart();
+            int sre=TCPUDPService::cast(s)->getSrcRangeEnd();
 
             compiler->normalizePortRange(srs,sre);
 
@@ -1272,10 +1318,10 @@ bool NATCompiler_ipt::separateSourceAndDestinationPorts::processNext()
 	assert(s!=NULL);
 
 	if ( TCPService::isA(s) || UDPService::isA(s) ) {
-            int srs=s->getInt("src_range_start");
-            int sre=s->getInt("src_range_end");
-            int drs=s->getInt("dst_range_start");
-            int dre=s->getInt("dst_range_end");
+            int srs=TCPUDPService::cast(s)->getSrcRangeStart();
+            int sre=TCPUDPService::cast(s)->getSrcRangeEnd();
+            int drs=TCPUDPService::cast(s)->getDstRangeStart();
+            int dre=TCPUDPService::cast(s)->getDstRangeEnd();
 
             compiler->normalizePortRange(srs,sre);
             compiler->normalizePortRange(drs,dre);
@@ -2023,7 +2069,6 @@ bool NATCompiler_ipt::processMultiAddressObjectsInRE::processNext()
         dynamic_cast<OSConfigurator_linux24*>(compiler->osconfigurator);
 
     RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
-    bool neg = re->getNeg();
 
     if (re->size()==1) 
     {
@@ -2084,12 +2129,31 @@ bool NATCompiler_ipt::processMultiAddressObjectsInRE::processNext()
     return true;
 }
 
+bool NATCompiler_ipt::countChainUsage::processNext()
+{
+    NATCompiler_ipt *ipt_comp = dynamic_cast<NATCompiler_ipt*>(compiler);
+
+    slurp();
+    if (tmp_queue.size()==0) return false;
+
+    for (deque<Rule*>::iterator k=tmp_queue.begin(); k!=tmp_queue.end(); ++k) 
+    {
+        NATRule *rule = NATRule::cast( *k );
+        ipt_comp->chain_usage_counter[rule->getStr("ipt_target")] += 1;
+    }
+
+    return true;
+}
+
+
 
 void NATCompiler_ipt::compile()
 {
 //    FWOptions* options=fw->getOptionsObject();
 
-    cout << _(" Compiling rules for 'nat' table ...") << endl << flush;
+    cout << " Compiling rules for 'nat' table";
+    if (ipv6) cout << ", IPv6";
+    cout <<  endl << flush;
 
     try {
 
@@ -2102,7 +2166,7 @@ void NATCompiler_ipt::compile()
 #endif
         add( new NATCompiler::Begin());
 
-        add( new printTotalNumberOfRules(                                  ));
+        add( new printTotalNumberOfRules());
 
         add( new recursiveGroupsInOSrc("check for recursive groups in OSRC"));
         add( new recursiveGroupsInODst("check for recursive groups in ODST"));
@@ -2112,79 +2176,112 @@ void NATCompiler_ipt::compile()
         add( new recursiveGroupsInTDst("check for recursive groups in TDST"));
         add( new recursiveGroupsInTSrv("check for recursive groups in TSRV"));
 
-        add( new emptyGroupsInOSrc("check for empty groups in OSRC"      ));
-        add( new emptyGroupsInODst("check for empty groups in ODST"      ));
-        add( new emptyGroupsInOSrv("check for empty groups in OSRV"      ));
+        add( new emptyGroupsInOSrc("check for empty groups in OSRC" ));
+        add( new emptyGroupsInODst("check for empty groups in ODST" ));
+        add( new emptyGroupsInOSrv("check for empty groups in OSRV" ));
 
-        add( new emptyGroupsInTSrc("check for empty groups in TSRC"      ));
-        add( new emptyGroupsInTDst("check for empty groups in TDST"      ));
-        add( new emptyGroupsInTSrv("check for empty groups in TSRV"      ));
+        add( new emptyGroupsInTSrc("check for empty groups in TSRC" ));
+        add( new emptyGroupsInTDst("check for empty groups in TDST" ));
+        add( new emptyGroupsInTSrv("check for empty groups in TSRV"));
+
+	add( new ExpandGroups("Expand groups"));
+
+        // processors that expand objects with multiple addresses
+        // check addresses against current address family using member
+        // ipv6. If all addresses do not match, we may end up with
+        // empty rule element.
+        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+
+        if (ipv6)
+            add( new DropIPv4Rules("drop ipv4 rules"));
+        else
+            add( new DropIPv6Rules("drop ipv6 rules"));
 
-	add( new ExpandGroups(            "Expand groups"                ));
         add( new eliminateDuplicatesInOSRC("eliminate duplicates in OSRC"));
         add( new eliminateDuplicatesInODST("eliminate duplicates in ODST"));
         add( new eliminateDuplicatesInOSRV("eliminate duplicates in OSRV"));
 
-        add( new swapMultiAddressObjectsInOSrc(" swap MultiAddress -> MultiAddressRunTime in OSrc") );
-        add( new swapMultiAddressObjectsInODst(" swap MultiAddress -> MultiAddressRunTime in ODst") );
+        add( new swapMultiAddressObjectsInOSrc(
+                 " swap MultiAddress -> MultiAddressRunTime in OSrc") );
+        add( new swapMultiAddressObjectsInODst(
+                 " swap MultiAddress -> MultiAddressRunTime in ODst") );
 
-        add( new processMultiAddressObjectsInOSrc("process MultiAddress objects in OSrc") );
-        add( new processMultiAddressObjectsInODst("process MultiAddress objects in ODst") );
+        add( new processMultiAddressObjectsInOSrc(
+                 "process MultiAddress objects in OSrc") );
+        add( new processMultiAddressObjectsInODst(
+                 "process MultiAddress objects in ODst") );
 
-        add( new doOSrvNegation(   "process negation in OSrv"            ));
+        add( new doOSrvNegation( "process negation in OSrv" ));
 
         add( new convertToAtomicportForOSrv("convert to atomic rules in OSrv"));
 
-        add( new classifyNATRule(           "classify NAT rule"           ));
-        add( new splitSDNATRule(            "split SDNAT rules"           ));
-        add( new classifyNATRule(           "reclassify rules"            ));
+        add( new classifyNATRule( "classify NAT rule" ));
+        add( new splitSDNATRule( "split SDNAT rules" ));
+        add( new classifyNATRule( "reclassify rules" ));
         add( new ConvertLoadBalancingRules( "convert load balancing rules"));
-        add( new VerifyRules(               "verify rules"                ));
+        add( new VerifyRules( "verify rules" ));
 
-        add( new doOSrcNegation(            "process negation in OSrc"    ));
-        add( new doODstNegation(            "process negation in ODst"    ));
+        add( new doOSrcNegation( "process negation in OSrc" ));
+        add( new doODstNegation( "process negation in ODst" ));
 
 /* call splitOnODst after processing negation */
-        add( new splitOnODst(            "split on ODst"                  ));
+        add( new splitOnODst( "split on ODst" ));
 
-        add( new portTranslationRules(   "port translation rules"         ));
-        add( new specialCaseWithRedirect("check for special case with redirecting port translation rules" ) );
+        add( new portTranslationRules( "port translation rules" ));
+        add( new specialCaseWithRedirect(
+                 "special case with redirecting port translation rules" ) );
 
         if (fwopt->getBool("local_nat") )
         {
             if ( fwopt->getBool("firewall_is_part_of_any_and_networks") )
-                add( new splitIfOSrcAny(  "split rule if OSrc is any"     ));
+                add( new splitIfOSrcAny( "split rule if OSrc is any" ));
 
             add( new splitIfOSrcMatchesFw("split rule if OSrc matches FW" ));
         }
 
-        add( new splitNONATRule("process NAT rules that request no translation"));
+        add( new splitNONATRule("NAT rules that request no translation"));
         add( new localNATRule("process local NAT rules"));
-//        add( new DNATforFW("process DNAT rules for packets originated on the firewall"));
-        add( new decideOnChain(    "decide on chain"                    ) );
-        add( new decideOnTarget(   "decide on target"                   ) );
+// add( new DNATforFW("process DNAT rules for packets originated on the firewall"));
+        add( new decideOnChain( "decide on chain" ) );
+        add( new decideOnTarget( "decide on target" ) );
 
-        add( new splitODstForSNAT( "split rule if objects in ODst belong to different subnets") );
+        add( new splitODstForSNAT(
+                 "split rule if objects in ODst belong to different subnets") );
         add( new ReplaceFirewallObjectsODst("replace firewall in ODst" ) );
 	add( new ReplaceFirewallObjectsTSrc("replace firewall in TSrc" ) );
-        add( new splitOnDynamicInterfaceInODst("split rule if ODst is dynamic interface" ) );
-        add( new splitOnDynamicInterfaceInTSrc("split rule if TSrc is dynamic interface" ) );
+        add( new splitOnDynamicInterfaceInODst(
+                 "split rule if ODst is dynamic interface" ) );
+        add( new splitOnDynamicInterfaceInTSrc(
+                 "split rule if TSrc is dynamic interface" ) );
 
         add( new ExpandMultipleAddresses("expand multiple addresses") );
+        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
 
-        add( new specialCaseWithUnnumberedInterface("check for special cases with dynamic and unnumbered interfaces" ) );
-        add( new checkForDynamicInterfacesOfOtherObjects( "check for dynamic interfaces of other hosts and firewalls" ) );
+        if (ipv6)
+            add( new DropIPv4Rules("drop ipv4 rules"));
+        else
+            add( new DropIPv6Rules("drop ipv6 rules"));
+
+        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
+
+        add( new specialCaseWithUnnumberedInterface(
+                 "special cases with dynamic and unnumbered interfaces" ) );
+        add( new checkForDynamicInterfacesOfOtherObjects(
+                 "dynamic interfaces of other hosts and firewalls" ) );
 
         add( new verifyRuleWithMAC("verify rules using MAC address filtering"));
         add( new ExpandAddressRanges("expand address ranges") );
+        add( new dropRuleWithEmptyRE("drop rules with empty rule elements"));
 
-        add( new splitMultiSrcAndDst("split rules where multiple srcs and dsts are present" ) );
+        add( new splitMultiSrcAndDst(
+                 "split rules where multiple srcs and dsts are present" ) );
 
         add( new splitServices("split on services") );
         add( new VerifyRules2("check correctness of TSrv") );
         add( new separatePortRanges("separate port ranges") );
         add( new separateSourcePorts("separate objects with src") );
-        add( new separateSourceAndDestinationPorts( "separate objects with both src and dest ports" ) );
+        add( new separateSourceAndDestinationPorts(
+                 "separate objects with both src and dest ports" ) );
         add( new prepareForMultiport("prepare for multiport") );
         add( new splitMultipleICMP("split rule with multiple ICMP services") );
 
@@ -2194,9 +2291,12 @@ void NATCompiler_ipt::compile()
 
         add( new AssignInterface("assign rules to interfaces") );
         add( new dynamicInterfaceInODst("split if dynamic interface in ODst") );
-	add( new dynamicInterfaceInTSrc("set target if dynamic interface in TSrc" ) );
+	add( new dynamicInterfaceInTSrc(
+                 "set target if dynamic interface in TSrc" ) );
         add( new convertInterfaceIdToStr("prepare interface assignments") );
 
+        add( new countChainUsage("Count chain usage"));
+
         if (fwopt->getBool("use_iptables_restore"))
         {
             // bug #1812295: we should use PrintRuleIptRstEcho not only
