@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipt.h,v 1.8 2006/06/05 04:53:21 vkurland Exp $
+  $Id: NATCompiler_ipt.h 474 2008-08-18 03:05:45Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -58,8 +58,17 @@ namespace fwcompiler {
 
         NATCompiler_ipt::PrintRule *printRule;
         bool   have_dynamic_interfaces;
+        std::map<std::string, int> chain_usage_counter;
 
-        std::string getInterfaceVarName(libfwbuilder::FWObject *iface);
+        // use minus_n_commands map to track creation of chains.
+        // Using external map object for this to be able to track
+        // new chains across different compiler runs (used to process
+        // rules in different policy or nat objects)
+        std::map<const std::string, bool> *minus_n_commands;
+        
+        static const std::list<std::string>& getStandardChains();
+        std::string getInterfaceVarName(libfwbuilder::FWObject *iface,
+                                        bool v6=false);
         std::string getAddressTableVarName(libfwbuilder::FWObject *iface);
 
         /**
@@ -421,6 +430,12 @@ namespace fwcompiler {
         };
 
 
+	/**
+	 * count how many times each user-defined chain we've created is
+         * used. We should be able to drop unused chains.
+	 */
+        DECLARE_NAT_RULE_PROCESSOR(countChainUsage);
+
 	/**
 	 *   prints single policy rule, assuming all * groups have
 	 *   been expanded, so source, destination and * service hold
@@ -434,9 +449,11 @@ namespace fwcompiler {
 
             bool                             init;
             bool                             print_once_on_top;
+            bool                             minus_n_tracker_initialized;
             std::string                      current_rule_label;
-            std::map<const std::string,bool> chains;
 
+            void InitializeMinusNTracker();
+            
             virtual std::string _createChain(const std::string &chain);
             virtual std::string _startRuleLine();
             virtual std::string _endRuleLine();
@@ -457,7 +474,9 @@ namespace fwcompiler {
             virtual std::string _printDNATPorts(libfwbuilder::Service *srv);
 
             virtual std::string _printMultiport(libfwbuilder::NATRule *r);
-            virtual std::string _printAddr(libfwbuilder::Address  *o,bool print_mask=true,bool print_range=false);
+            virtual std::string _printAddr(libfwbuilder::Address  *o,
+                                           bool print_mask=true,
+                                           bool print_range=false);
             virtual std::string _printChainDirectionAndInterface(libfwbuilder::NATRule *r);
 
             public:
@@ -511,9 +530,16 @@ namespace fwcompiler {
 	public:
 
 	NATCompiler_ipt(libfwbuilder::FWObjectDatabase *_db,
-		       const std::string &fwname,
-		       fwcompiler::OSConfigurator *_oscnf) : NATCompiler(_db,fwname,_oscnf)
-            {have_dynamic_interfaces=false; printRule=NULL;}
+                        const std::string &fwname,
+                        bool ipv6_policy,
+                        fwcompiler::OSConfigurator *_oscnf,
+                        std::map<const std::string, bool> *m_n_commands_map) :
+        NATCompiler(_db, fwname, ipv6_policy, _oscnf)
+        {
+            have_dynamic_interfaces=false;
+            printRule=NULL;
+            minus_n_commands = m_n_commands_map;
+        }
 
 
 	virtual int  prolog();
