@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_PrintRuleIptRstEcho.cpp,v 1.3 2006/06/05 04:53:21 vkurland Exp $
+  $Id: PolicyCompiler_PrintRuleIptRstEcho.cpp 474 2008-08-18 03:05:45Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -60,10 +60,14 @@ using namespace std;
 string PolicyCompiler_ipt::PrintRuleIptRstEcho::_createChain(const string &chain)
 {
     string res;
-    if ( ! chains[chain] )
+    PolicyCompiler_ipt *ipt_comp = dynamic_cast<PolicyCompiler_ipt*>(compiler);
+
+    if (!minus_n_tracker_initialized) InitializeMinusNTracker();
+
+    if ( ipt_comp->minus_n_commands->count(chain)==0 )
     {
 	res = "echo \":" + chain + " - [0:0]\"\n";
-	chains[chain]=true;
+	(*(ipt_comp->minus_n_commands))[chain] = true;
     }
     return res;
 }
@@ -101,7 +105,6 @@ string PolicyCompiler_ipt::PrintRuleIptRstEcho::_declareTable()
 
 string PolicyCompiler_ipt::PrintRuleIptRstEcho::_flushAndSetDefaultPolicy()
 {
-    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     ostringstream res;
 
     res << "echo :INPUT DROP [0:0]" << endl;
