@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h,v 1.32 2007/07/08 01:11:35 vkurland Exp $
+  $Id: PolicyCompiler_ipt.h 474 2008-08-18 03:05:45Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -49,6 +49,18 @@ namespace libfwbuilder {
 #define TCP_SYN_OBJ_ID   "__tcp_syn_obj__"
 #define BCAST_255_OBJ_ID "__bcast_255_obj__"
 
+// a functor to join list<string> into a string with separator sep
+class join : public std::unary_function<std::string, void>
+{
+    std::string *result;
+    std::string  separator;
+public:
+    join(std::string *res, const std::string &s)
+    { result = res; separator = s; }
+    void operator()(std::string &s);
+};
+    
+
 namespace fwcompiler {
 
 
@@ -63,12 +75,26 @@ namespace fwcompiler {
         bool                           have_connmark;
         bool                           have_connmark_in_output;
         std::string                    my_table;
+
+        std::map<std::string, int>     tmp_chain_no;
+        std::map<std::string, int>     chain_usage_counter;
+
+        // use minus_n_commands map to track creation of chains.
+        // Using external map object for this to be able to track
+        // new chains across different compiler runs (used to process
+        // rules in different policy or nat objects)
+        std::map<const std::string, bool> *minus_n_commands;
+
+        static const std::list<std::string>& getStandardChains();
         
         PolicyCompiler_ipt::PrintRule* createPrintRuleProcessor();
 
-        std::string getInterfaceVarName(libfwbuilder::FWObject *iface);
+        std::string getInterfaceVarName(libfwbuilder::FWObject *iface,
+                                        bool v6=false);
         std::string getAddressTableVarName(libfwbuilder::FWObject *iface);
-        
+
+        bool newIptables(const std::string &version);
+
 	/**
 	 * Add some predefined rules controlled by checkboxes in
 	 * firewall settings dialog
@@ -587,6 +613,13 @@ namespace fwcompiler {
          */
         DECLARE_POLICY_RULE_PROCESSOR(splitNonTerminatingTargets);
 
+        /**
+         * drop rules with terminating targets. Used as part of the
+         * shadowing detection for non-terminating rules in the mangle
+         * table.
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(dropTerminatingTargets);
+        
 	/**
 	 * decides what chain this rule should go to if it has not
 	 * been decided in decideOnChainIfFW
@@ -622,6 +655,12 @@ namespace fwcompiler {
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(checkMACinOUTPUTChain);
 
+	/**
+	 * iptables permits using "--m owner --uid-owner" only in
+	 * the OUTPUT chain
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(checkUserServiceInWrongChains);
+
 
 	/**
 	 * expand groups in Srv
@@ -773,6 +812,12 @@ namespace fwcompiler {
          */
         DECLARE_POLICY_RULE_PROCESSOR(SkipActionContinueWithNoLogging);
 
+	/**
+	 * count how many times each user-defined chain we've created is
+         * used. We should be able to drop unused chains.
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(countChainUsage);
+
         
 	/**
 	 *  prints single policy rule, assuming all groups have been
@@ -792,9 +837,11 @@ namespace fwcompiler {
             
             bool                             init;
             bool                             print_once_on_top;
+            bool                             minus_n_tracker_initialized;
             std::string                      current_rule_label;
-            std::map<const std::string,bool> chains;
 
+            void InitializeMinusNTracker();
+            
             virtual std::string _createChain(const std::string &chain);
             virtual std::string _printRuleLabel(libfwbuilder::PolicyRule *r);
 
@@ -840,6 +887,7 @@ namespace fwcompiler {
             virtual std::string _printOptionalGlobalRules();
             virtual std::string _declareTable();
             virtual std::string _flushAndSetDefaultPolicy();
+            virtual std::string _clampTcpToMssRule();
             virtual std::string _commit();
             virtual std::string _quote(const std::string &s);
 
@@ -903,14 +951,19 @@ namespace fwcompiler {
 	public:
 
 	PolicyCompiler_ipt(libfwbuilder::FWObjectDatabase *_db,
-			  const std::string &fwname,
-                fwcompiler::OSConfigurator *_oscnf) : PolicyCompiler(_db,fwname,_oscnf)
+                           const std::string &fwname,
+                           bool ipv6_policy,
+                           fwcompiler::OSConfigurator *_oscnf,
+                           std::map<const std::string, bool> *m_n_commands_map
+        ) :
+        PolicyCompiler(_db, fwname, ipv6_policy, _oscnf)
         {
             have_dynamic_interfaces = false;
             have_connmark = false;
             have_connmark_in_output = false;
             printRule = NULL;
             my_table = "filter";
+            minus_n_commands = m_n_commands_map;
         }
 
 
@@ -923,14 +976,23 @@ namespace fwcompiler {
          * some of the rules out before we begin actual processing
          */
         virtual void addRuleFilter();
+
+        /**
+         * this method registers chain used for the ruleset (most
+         * often branch rule set). Since rules in the same ruleset do
+         * not use this chain as target, rule processor
+         * countChainUsage considers it unused.  Registering it makes
+         * sure its usage counter is > 0.
+         */
+        void registerRuleSetChain(const std::string &chain_name);
         
         void setHaveDynamicInterfaces(bool f) { have_dynamic_interfaces=f; }
         
         virtual std::string flushAndSetDefaultPolicy();
         std::string commit();
 
-	static std::string getNewTmpChainName(libfwbuilder::PolicyRule *rule);
-	static std::string getNewChainName(libfwbuilder::PolicyRule *rule,libfwbuilder::Interface *rule_iface);
+	std::string getNewTmpChainName(libfwbuilder::PolicyRule *rule);
+	std::string getNewChainName(libfwbuilder::PolicyRule *rule,libfwbuilder::Interface *rule_iface);
 
     };
 
