@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt_optimizer.cpp,v 1.12 2007/07/19 01:46:30 vkurland Exp $
+  $Id: PolicyCompiler_ipt_optimizer.cpp 538 2008-09-14 22:43:24Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -68,8 +68,8 @@ void PolicyCompiler_ipt::optimize1::optimizeForRuleElement(PolicyRule    *rule,
     PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     PolicyRule     *r;
 
-    string this_chain  =rule->getStr("ipt_chain");
-    string new_chain=PolicyCompiler_ipt::getNewTmpChainName(rule);
+    string this_chain = rule->getStr("ipt_chain");
+    string new_chain  = ipt_comp->getNewTmpChainName(rule);
 
     r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
     compiler->temp_ruleset->add(r);
@@ -102,7 +102,8 @@ void PolicyCompiler_ipt::optimize1::optimizeForRuleElement(PolicyRule    *rule,
                    if (TCPService::isA(srv))
                    {
                        re->clearChildren();
-                       re->addRef(compiler->dbcopy->findInIndex(ANY_TCP_OBJ_ID));
+                       re->addRef(compiler->dbcopy->findInIndex(
+                                      FWObjectDatabase::getIntId(ANY_TCP_OBJ_ID)));
 /* also leave a flag indicating that further optimization by service
  * is not needed */
                        rule->setBool("do_not_optimize_by_srv",true);
@@ -163,11 +164,17 @@ bool PolicyCompiler_ipt::optimize1::processNext()
         FWObject *o= *i;
         if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 	    
-        Service *s=Service::cast( o );
-        assert(s);
+        Service *service_object = Service::cast( o );
+        if (service_object==NULL)
+        {
+            ostringstream err;
+            err << "Broken Service rule element in rule "
+                << rule->getLabel();
+                compiler->abort(err.str());
+        }
 
 // tcp and udp will be collapsed because we can use multiport module
-        if ( !TCPService::isA(s) && !UDPService::isA(s))
+        if ( !TCPService::isA(service_object) && !UDPService::isA(service_object))
         {
             all_tcp_or_udp = false;
             break;
