@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@vk.crocodile.org>
 
-  $Id: combinedAddress.cpp,v 1.2 2006/09/11 00:41:09 vkurland Exp $
+  $Id: combinedAddress.cpp 355 2008-07-13 05:37:32Z svn $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -23,9 +23,12 @@
 
 */
 
+#include <assert.h>
+#include <iostream>
+
 #include <fwbuilder/libfwbuilder-config.h>
 
-#include <combinedAddress.h>
+#include "combinedAddress.h"
 
 #include <fwbuilder/FWException.h>
 #include <fwbuilder/FWObjectReference.h>
@@ -46,7 +49,7 @@ std::string combinedAddress::getPhysAddress() const
 
 void combinedAddress::setPhysAddress(const std::string &s)
 {
-    physAddress=s;
+    physAddress = s;
 }
 
 bool combinedAddress::isAny() const
@@ -54,3 +57,10 @@ bool combinedAddress::isAny() const
     return (IPv4::isAny() && physAddress=="");
 }
 
+FWObject& combinedAddress::shallowDuplicate(const FWObject *other,
+                                            bool preserve_id) throw(FWException)
+{
+    physAddress = dynamic_cast<const combinedAddress*>(other)->physAddress;
+    return IPv4::shallowDuplicate(other, preserve_id);
+}
+
