@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf.cpp,v 1.18 2007/06/21 03:25:45 vkurland Exp $
+  $Id: NATCompiler_pf.cpp 338 2008-07-07 18:29:33Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -84,19 +84,30 @@ int NATCompiler_pf::prolog()
 	}
 
 	if (!found_ext)
-	    throw FWException(_("At least one interface should be marked as external, can not configure NAT"));
+	    throw FWException(
+                "At least one interface should be marked as external, "
+                "can not configure NAT");
     }
 
-/* pseudo-host with ip address 127.0.0.1  We'll use it for redirection NAT rules */
+/* pseudo-host with ip address 127.0.0.1  We'll use it for redirection
+ * NAT rules
+ */
     //FWObject    *grp;
-    loopback_address=IPv4::cast(dbcopy->create(IPv4::TYPENAME) );
-    loopback_address->setAddress("127.0.0.1");
+    loopback_address = dbcopy->create(IPv4::TYPENAME);
     loopback_address->setName("__loopback_address__");
-    loopback_address->setId("__loopback_address_id__");
+    loopback_address->setId(FWObjectDatabase::generateUniqueId()); // "__loopback_address_id__");
+
+    IPv4::cast(loopback_address)->setAddress(InetAddr::getLoopbackAddr());
+
     dbcopy->add(loopback_address,false);
     cacheObj(loopback_address);
 
-    if (tables) tables->init(dbcopy);
+    if (tables)
+    {
+        tables->init(dbcopy);
+        if (!getSourceRuleSet()->isTop())
+            tables->setRuleSetName(getRuleSetName());
+    }
 
     return n;
 }
@@ -309,7 +320,7 @@ bool NATCompiler_pf::VerifyRules::processNext()
         Network *a1=Network::cast(compiler->getFirstOSrc(rule));
         Network *a2=Network::cast(compiler->getFirstTSrc(rule));
         if ( a1==NULL || a2==NULL ||
-             a1->getNetmask().getLength()!=a2->getNetmask().getLength() )
+             a1->getNetmaskPtr()->getLength()!=a2->getNetmaskPtr()->getLength() )
             throw FWException(_("Original and translated source should both be networks of the same size . Rule ")+rule->getLabel());
     }
 
@@ -318,7 +329,7 @@ bool NATCompiler_pf::VerifyRules::processNext()
         Network *a1=Network::cast(compiler->getFirstODst(rule));
         Network *a2=Network::cast(compiler->getFirstTDst(rule));
         if ( a1==NULL || a2==NULL ||
-             a1->getNetmask().getLength()!=a2->getNetmask().getLength() )
+             a1->getNetmaskPtr()->getLength()!=a2->getNetmaskPtr()->getLength() )
             throw FWException(_("Original and translated destination should both be networks of the same size . Rule ")+rule->getLabel());
     }
 
@@ -382,16 +393,18 @@ bool NATCompiler_pf::addVirtualAddress::processNext()
 	if (rule->getRuleType()==NATRule::DNAT) a=compiler->getFirstODst(rule);
 	else  return true;
     assert(a!=NULL);
+    const InetAddr *a_addr = a->getAddressPtr();
 
-    if ( ! a->isAny() && a->getId()!=compiler->getFwId() ) {
-
+    if ( ! a->isAny() && a->getId()!=compiler->getFwId() && a_addr)
+    {
 	list<FWObject*> l2=compiler->fw->getByType(Interface::TYPENAME);
 	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
         {
 	    Interface *iface=dynamic_cast<Interface*>(*i);
 	    assert(iface);
-
-	    if ( a->getAddress() == iface->getAddress() ) return true;
+            const InetAddr *iface_addr = iface->getAddressPtr();
+	    if (iface_addr && *a_addr == *iface_addr )
+                return true;
 	}
 	compiler->osconfigurator->addVirtualAddressForNAT( a );
     }
@@ -405,13 +418,14 @@ bool NATCompiler_pf::splitForTSrc::processNext()
     NATRule *rule=getNext(); if (rule==NULL) return false;
     RuleElementTSrc  *tsrc=rule->getTSrc();  assert(tsrc);
 
-    map<string,list<FWObject*> > interfaceGroups;
+    map<int,list<FWObject*> > interfaceGroups;
 
     for(list<FWObject*>::iterator i=tsrc->begin(); i!=tsrc->end(); ++i)
     {
         FWObject *o= *i;
         if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-        Interface *iface = compiler->findInterfaceFor(Address::cast(o),compiler->fw);
+        Interface *iface = compiler->findInterfaceFor(Address::cast(o),
+                                                      compiler->fw);
         if (iface!=NULL)
             interfaceGroups[iface->getId()].push_back(o);
     }
@@ -419,7 +433,7 @@ bool NATCompiler_pf::splitForTSrc::processNext()
     if (interfaceGroups.size()<=1) tmp_queue.push_back(rule);
     else
     {
-        map<string,list<FWObject*> >::iterator i;
+        map<int,list<FWObject*> >::iterator i;
         for (i=interfaceGroups.begin(); i!=interfaceGroups.end(); i++)
         {
             list<FWObject*> &objSubset = (*i).second;
@@ -650,7 +664,7 @@ bool NATCompiler_pf::ReplaceObjectsTDst::processNext()
         RuleElementTDst *rel=rule->getTDst();          assert(rel);
         Address         *otdst=compiler->getFirstTDst(rule);
         Interface       *loopback=NULL;
-        IPv4            *loopback_address=NULL;
+        FWObject        *loopback_address=NULL;
 
 /* if firewall is used in TDst in redirection rule, replace it with
  * its loopback interface
@@ -661,12 +675,12 @@ bool NATCompiler_pf::ReplaceObjectsTDst::processNext()
             for (std::list<FWObject*>::iterator i=l2.begin();
                  i!=l2.end(); ++i) 
             {
-                Interface *iface=dynamic_cast<Interface*>(*i);
+                Interface *iface = dynamic_cast<Interface*>(*i);
                 assert(iface);
                 if (iface->isLoopback()) 
                 {
-                    loopback=iface;
-                    loopback_address=IPv4::cast(loopback->getFirstByType(IPv4::TYPENAME));
+                    loopback = iface;
+                    loopback_address = loopback->getFirstByType(IPv4::TYPENAME);
                 }
             }
 
@@ -724,7 +738,14 @@ bool NATCompiler_pf::swapAddressTableObjectsInRE::processNext()
         {
             MultiAddress *atbl = *i;
 
-            string mart_id = atbl->getId()+"_runtime";
+            // Need to make sure the ID of the MultiAddressRunTime
+            // object created here is stable and is always the same
+            // for the same MultiAddress object. In particular this
+            // ensures that we reuse tables between policy and NAT rules
+            string mart_id_str = FWObjectDatabase::getStringId(atbl->getId()) +
+                "_runtime";
+            int mart_id = FWObjectDatabase::registerStringId(mart_id_str);
+
             MultiAddressRunTime *mart = 
                 MultiAddressRunTime::cast(compiler->dbcopy->findInIndex(mart_id));
             if (mart==NULL)
@@ -897,8 +918,10 @@ void NATCompiler_pf::compile()
 {
     bool manage_virtual_addr=fwopt->getBool("manage_virtual_addr");
 
-
-    cout << _(" Compiling NAT rules for ") << fw->getName() << " ..." << endl << flush;
+    cout << " Compiling " << fw->getName();
+    if (!getRuleSetName().empty())  cout << " ruleset " << getRuleSetName();
+    if (ipv6) cout << ", IPv6";
+    cout <<  endl << flush;
 
     try {
 
@@ -966,11 +989,11 @@ void NATCompiler_pf::compile()
                  "replace references to the firewall in TSrc" ) );
         add( new ReplaceObjectsTDst( "replace objects in TDst" ) );
 
-	if ( manage_virtual_addr ) {
-            add( new addVirtualAddress("add virtual addresses for NAT rules"));
-	}
-
         add( new ExpandMultipleAddresses( "expand multiple addresses" ) );
+
+	if ( manage_virtual_addr )
+            add( new addVirtualAddress("add virtual addresses for NAT rules"));
+
         add( new checkForUnnumbered("check for unnumbered interfaces" ) );
         add( new checkForDynamicInterfacesOfOtherObjects(
                  "check for dynamic interfaces of other hosts and firewalls"));
