@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator_macosx.cpp,v 1.2 2006/03/06 03:02:58 vkurland Exp $
+  $Id: OSConfigurator_macosx.cpp 312 2008-07-05 17:40:12Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -72,28 +72,32 @@ void OSConfigurator_macosx::processFirewallOptions()
     }
 }
 
-void OSConfigurator_macosx::addVirtualAddressForNAT(const Network *addr)
+void OSConfigurator_macosx::addVirtualAddressForNAT(const Network*)
 {
 }
 
 void OSConfigurator_macosx::addVirtualAddressForNAT(const Address *addr)
 {
     if (virtual_addresses.empty() || 
-	find(virtual_addresses.begin(),virtual_addresses.end(),addr->getAddress())==virtual_addresses.end()) 
+	find(virtual_addresses.begin(),virtual_addresses.end(),
+             *(addr->getAddressPtr())) == virtual_addresses.end()) 
     {
-        IPv4 *iaddr=IPv4::cast( findAddressFor(addr, fw ) );
+        FWObject *iaddr = findAddressFor(addr, fw );
         if (iaddr!=NULL)
         {
-            Interface *iface=Interface::cast(iaddr->getParent());
+            Address *iaddr_addr = Address::cast(iaddr);
+            assert(iaddr_addr!=NULL);
+            Interface *iface = Interface::cast(iaddr->getParent());
             assert(iface!=NULL);
 
-            output << "add_addr " << addr->getAddress().toString() << " "
-                   << iaddr->getNetmask().toString() <<  " "
+            output << "add_addr " << addr->getAddressPtr()->toString() << " "
+                   << iaddr_addr->getNetmaskPtr()->toString() <<  " "
                    << iface->getName() << endl;
         
-            virtual_addresses.push_back(addr->getAddress());
+            virtual_addresses.push_back(*(addr->getAddressPtr()));
         } else
-            warning(_("Can not add virtual address ") + addr->getAddress().toString() );
+            warning(_("Can not add virtual address ") +
+                    addr->getAddressPtr()->toString() );
     }
 }
 
@@ -174,13 +178,13 @@ void  OSConfigurator_macosx::configureInterfaces()
             FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
             for ( ; j!=j.end(); ++j ) 
             {
-                IPv4 *iaddr=IPv4::cast(*j);
-
-                output << "add_addr " << iaddr->getAddress().toString() << " "
-                       << iaddr->getNetmask().toString() << " "
+                Address *iaddr = Address::cast(*j);
+                output << "add_addr "
+                       << iaddr->getAddressPtr()->toString() << " "
+                       << iaddr->getNetmaskPtr()->toString() << " "
                        << iface->getName() << endl;
         
-                virtual_addresses.push_back(iaddr->getAddress());
+                virtual_addresses.push_back(*(iaddr->getAddressPtr()));
             }
         }
         output << endl;
