@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator_openbsd.cpp,v 1.2 2006/03/06 03:02:58 vkurland Exp $
+  $Id: OSConfigurator_openbsd.cpp 378 2008-07-18 03:36:35Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -63,6 +63,14 @@ void OSConfigurator_openbsd::processFirewallOptions()
 	output << "$SYSCTL -w net.inet.ip.forwarding=" << s << endl;
     }
 
+    s=options->getStr("openbsd_ipv6_forward");
+    if (!s.empty()) {
+        if (s=="1" || s=="On" || s=="on") s="1";
+        else                              s="0";
+
+	output << "$SYSCTL -w net.inet.ip6.forwarding=" << s << endl;
+    }
+
     s=options->getStr("openbsd_ip_sourceroute");
     if (!s.empty()) {
 	if (s!="0" && s!="1")
@@ -80,34 +88,40 @@ void OSConfigurator_openbsd::processFirewallOptions()
     }
 }
 
-void OSConfigurator_openbsd::addVirtualAddressForNAT(const Network *nw)
+void OSConfigurator_openbsd::addVirtualAddressForNAT(const Network*)
 {
 }
 
 void OSConfigurator_openbsd::addVirtualAddressForNAT(const Address *addr)
 {
     if (virtual_addresses.empty() || 
-	find(virtual_addresses.begin(),virtual_addresses.end(),addr->getAddress())==virtual_addresses.end()) 
+	find(virtual_addresses.begin(),
+             virtual_addresses.end(),
+             *(addr->getAddressPtr())) == virtual_addresses.end())
     {
-        IPv4 *iaddr=IPv4::cast( findAddressFor(addr, fw ) );
+        FWObject *iaddr = findAddressFor(addr, fw );
         if (iaddr!=NULL)
         {
-            Interface *iface=Interface::cast(iaddr->getParent());
+            Address *iaddr_addr = Address::cast(iaddr);
+            assert(iaddr_addr!=NULL);
+            Interface *iface = Interface::cast(iaddr->getParent());
             assert(iface!=NULL);
 
-            output << "add_addr " << addr->getAddress().toString() << " "
-                   << iaddr->getNetmask().toString() <<  " "
+            output << "add_addr " << addr->getAddressPtr()->toString() << " "
+                   << iaddr_addr->getNetmaskPtr()->toString() <<  " "
                    << iface->getName() << endl;
         
-            virtual_addresses.push_back(addr->getAddress());
+            virtual_addresses.push_back(*(addr->getAddressPtr()));
         } else
-            warning(_("Can not add virtual address ") + addr->getAddress().toString() );
+            warning(_("Can not add virtual address ") +
+                    addr->getAddressPtr()->toString() );
     }
 }
 
 #if 0
     if (virtual_addresses.empty() || 
-	find(virtual_addresses.begin(),virtual_addresses.end(),addr->getAddress())==virtual_addresses.end()) {
+	find(virtual_addresses.begin(),virtual_addresses.end(),
+             *(addr->getAddressPtr())) == virtual_addresses.end()) {
 
         FWObjectTypedChildIterator i=fw->findByType(Interface::TYPENAME);
         for ( ; i!=i.end(); ++i ) {
@@ -115,20 +129,22 @@ void OSConfigurator_openbsd::addVirtualAddressForNAT(const Address *addr)
 	    assert(iface);
 
 	    FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
-	    for ( ; j!=j.end(); ++j ) {
-		IPv4 *iaddr=IPv4::cast(*j);
-
-                IPNetwork n( iaddr->getAddress() , iaddr->getNetmask() );
-                if ( n.belongs( addr->getAddress() ) ) {
+	    for ( ; j!=j.end(); ++j )
+            {
+                Address *iaddr = Address::cast(*j);
+                if ( ipv4->belongs( *(addr->getAddressPtr()) ) )
+                {
                     output << "ifconfig " 
                            << iface->getName() << " "
-                           << addr->getAddress().toString() << " alias" << endl;
-                    virtual_addresses.push_back( addr->getAddress() );
+                           << addr->getAddressPtr()->toString()
+                           << " alias" << endl;
+                    virtual_addresses.push_back( *(addr->getAddressPtr()) );
                     return;
                 }
             }
 	}
-	warning(_("Can not add virtual address ") + addr->getAddress().toString() );
+	warning(_("Can not add virtual address ") +
+                addr->getAddressPtr()->toString() );
     }
 }
 #endif
@@ -211,13 +227,13 @@ void  OSConfigurator_openbsd::configureInterfaces()
             FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
             for ( ; j!=j.end(); ++j ) 
             {
-                IPv4 *iaddr=IPv4::cast(*j);
-
-                output << "add_addr " << iaddr->getAddress().toString() << " "
-                       << iaddr->getNetmask().toString() << " "
+                Address *iaddr = Address::cast(*j);
+                output << "add_addr "
+                       << iaddr->getAddressPtr()->toString() << " "
+                       << iaddr->getNetmaskPtr()->toString() << " "
                        << iface->getName() << endl;
         
-                virtual_addresses.push_back(iaddr->getAddress());
+                virtual_addresses.push_back(*(iaddr->getAddressPtr()));
             }
         }
         output << endl;
