@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipf.h,v 1.3 2006/05/16 23:00:17 vkurland Exp $
+  $Id: PolicyCompiler_ipf.h 312 2008-07-05 17:40:12Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -247,7 +247,9 @@ namespace fwcompiler {
 
 	PolicyCompiler_ipf(libfwbuilder::FWObjectDatabase *_db,
 			   const std::string &fwname,
-			   fwcompiler::OSConfigurator *_oscnf) : PolicyCompiler_pf(_db,fwname,_oscnf,NULL) {}
+                           bool ipv6_policy,
+			   fwcompiler::OSConfigurator *_oscnf) :
+        PolicyCompiler_pf(_db, fwname, ipv6_policy, _oscnf, NULL) {}
 
 
 	virtual int  prolog();
