@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pix.cpp,v 1.1 2008/03/06 06:49:00 vkurland Exp $
+  $Id: PolicyCompiler_pix.cpp 338 2008-07-07 18:29:33Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -66,9 +66,10 @@ string PolicyCompiler_pix::myPlatformName() { return "pix"; }
 
 PolicyCompiler_pix::PolicyCompiler_pix(FWObjectDatabase *_db,
                                        const std::string &fwname,
+                                       bool ipv6_policy,
                                        OSConfigurator *_oscnf,
                                        NATCompiler_pix *_natcmp) :
-    PolicyCompiler_cisco(_db,fwname,_oscnf) 
+    PolicyCompiler_cisco(_db, fwname, ipv6_policy, _oscnf) 
 {
     natcmp=_natcmp;
     resetinbound=false;
@@ -123,14 +124,14 @@ int PolicyCompiler_pix::prolog()
             {
                 if (netmask.find(".")!=string::npos)
                 {
-                    Netmask nm(netmask);
-                    nm.to32BitInt(); // to avoid warning abt unused var
+                    InetAddr nm(netmask);
+                    nm.isAny(); // to avoid warning abt unused var
                 } else
                 {
                     int nm_length;
                     istringstream  str(netmask);
                     str >> nm_length;
-                    Netmask nm(nm_length);
+                    InetAddr nm(nm_length);
                     netmask = nm.toString();
                 }
             } catch(FWException &ex)
@@ -141,8 +142,7 @@ int PolicyCompiler_pix::prolog()
 
         try
         {
-            IPAddress a(addr);
-            a.to32BitInt();
+            InetAddr(addr);
         } catch(FWException &ex)
         {
             abort("Invalid address for management subnet: '"+addr+"'");
@@ -606,7 +606,10 @@ bool PolicyCompiler_pix::CreateObjectGroups::processNext()
 
 void PolicyCompiler_pix::compile()
 {
-    cout << " Compiling policy for " << fw->getName() << " ..." <<  endl << flush;
+    cout << endl;
+    cout << " Compiling ruleset " << getSourceRuleSet()->getName();
+    if (ipv6) cout << ", IPv6";
+    cout <<  endl << flush;
 
     try 
     {
@@ -761,6 +764,7 @@ void PolicyCompiler_pix::compile()
             add( new pickACL_v6( "assign ACLs for v6" ));
 
         add( new SpecialServices( "check for special services" ));
+        add( new CheckForUnsupportedUserService("check for user service") );
         add( new checkForZeroAddr( "check for zero addresses" ));
         add( new checkVersionAndDynamicInterface(
                  "check for dynamic interfaces in policy rule and verify version of PIX OS"));
@@ -809,12 +813,16 @@ void PolicyCompiler_pix::compile()
 
 string PolicyCompiler_pix::printAccessGroupCmd(ciscoACL *acl)
 {
-    string dir;
-    if (acl->direction()=="in" || acl->direction()=="Inbound") dir="in";
-    if (acl->direction()=="out" || acl->direction()=="Outbound") dir="out";
-    return string("access-group ") + acl->workName() +
-        " " + dir + 
-        " interface " + acl->getInterface()->getLabel() + "\n";
+    if (getSourceRuleSet()->isTop())
+    {
+        string dir;
+        if (acl->direction()=="in" || acl->direction()=="Inbound") dir="in";
+        if (acl->direction()=="out" || acl->direction()=="Outbound") dir="out";
+        return string("access-group ") + acl->workName() +
+            " " + dir + 
+            " interface " + acl->getInterface()->getLabel() + "\n";
+    }
+    return "";
 }
 
 void PolicyCompiler_pix::epilog()
