@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: pix.cpp,v 1.2 2008/03/09 05:13:55 vkurland Exp $
+  $Id: pix.cpp 443 2008-08-03 16:47:47Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -23,9 +23,7 @@
 
 */
 
-#include "config.h"
-
-#include <qsettings.h>
+#include "../../config.h"
 
 #include <fstream>
 #include <iostream>
@@ -46,6 +44,7 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <assert.h>
+#include <cstring>
 
 #include "PolicyCompiler_pix.h"
 #include "NATCompiler_pix.h"
@@ -107,14 +106,21 @@ class UpgradePredicate: public XMLTools::UpgradePredicate
     
 
 class sort_by_net_zone {
+    string any_address_id;
     public:
-//    explicit sort_by_net_zone();
-    bool operator()(const FWObject *a, const FWObject *b) {
-	if (Interface::constcast(a) && Interface::constcast(b)) {
+    explicit sort_by_net_zone()
+    {
+        any_address_id = FWObjectDatabase::getStringId(
+            FWObjectDatabase::ANY_ADDRESS_ID);
+    }
+    bool operator()(const FWObject *a, const FWObject *b)
+    {
+	if (Interface::constcast(a) && Interface::constcast(b))
+        {
 	    string netzone_a=a->getStr("network_zone");
 	    string netzone_b=b->getStr("network_zone");
-	    if ( netzone_a==FWObjectDatabase::getAnyNetworkId()) return false;
-	    if ( netzone_b==FWObjectDatabase::getAnyNetworkId()) return true;
+	    if ( netzone_a==any_address_id) return false;
+	    if ( netzone_b==any_address_id) return true;
 	}
 	return false;
     }
@@ -291,7 +297,7 @@ int main(int argc, char * const * argv)
         if (only_print_inspection_code)
         {
             OSConfigurator_pix_os *oscnf=NULL;
-            oscnf=new OSConfigurator_pix_os(objdb , fwobjectname);
+            oscnf = new OSConfigurator_pix_os(objdb , fwobjectname, false);
             oscnf->prolog();
 
             cout << oscnf->getProtocolInspectionCommands();
@@ -390,7 +396,8 @@ int main(int argc, char * const * argv)
                     "Network zone definition is missing for interface "
                     +iface->getName()+" ("+iface->getLabel()+")");
             }
-            FWObject *netzone=objdb->findInIndex(netzone_id);
+            FWObject *netzone=objdb->findInIndex(
+                FWObjectDatabase::getIntId(netzone_id));
             if (netzone==NULL) 
             {
                 throw FWException(
@@ -433,7 +440,8 @@ int main(int argc, char * const * argv)
                 nz->add(*j);
             }
             iface->setStr("orig_netzone_id", netzone_id );
-            iface->setStr("network_zone", nz->getId() );
+            iface->setStr("network_zone",
+                          FWObjectDatabase::getStringId(nz->getId()) );
         }
 
 /*
@@ -490,14 +498,14 @@ int main(int argc, char * const * argv)
         if (user_name==NULL) 
             throw FWException("Can't figure out your user name, aborting");
 
-        Preprocessor* prep=new Preprocessor(objdb , fwobjectname);
+        Preprocessor* prep=new Preprocessor(objdb , fwobjectname, false);
         prep->compile();
 
 /*
  * Process firewall options, build OS network configuration script
  */
         OSConfigurator *oscnf=NULL;
-        oscnf=new OSConfigurator_pix_os(objdb , fwobjectname);
+        oscnf = new OSConfigurator_pix_os(objdb , fwobjectname, false);
 
         oscnf->prolog();
         oscnf->processFirewallOptions();
@@ -505,8 +513,8 @@ int main(int argc, char * const * argv)
 
 /* create compilers and run the whole thing */
 
-        NATCompiler_pix *n=new NATCompiler_pix( objdb ,
-                                                fwobjectname, oscnf );
+        NATCompiler_pix *n = new NATCompiler_pix( objdb ,
+                                                  fwobjectname, false, oscnf );
 
         if (test_mode) n->setTestMode();
         n->setDebugLevel( dl );
@@ -520,9 +528,10 @@ int main(int argc, char * const * argv)
             cout << " Nothing to compile in NAT \n" << flush;
 
 
-        PolicyCompiler_pix *c=new PolicyCompiler_pix( objdb ,
-                                                      fwobjectname ,
-                                                      oscnf , n);
+        PolicyCompiler_pix *c = new PolicyCompiler_pix( objdb ,
+                                                        fwobjectname ,
+                                                        false,
+                                                        oscnf , n);
 
         if (test_mode) c->setTestMode();
         c->setDebugLevel( dl );
@@ -605,8 +614,24 @@ int main(int argc, char * const * argv)
 
         ofile << oscnf->getCompiledScript();
         ofile << endl;
+
+        if (c->haveErrorsAndWarnings())
+        {
+            ofile << "! Policy compiler errors and warnings:"
+                    << endl;
+            ofile << c->getErrors("! ");
+        }
+
         ofile << c->getCompiledScript();
         ofile << endl;
+
+        if (n->haveErrorsAndWarnings())
+        {
+            ofile << "! NAT compiler errors and warnings:"
+                    << endl;
+            ofile << n->getErrors("! ");
+        }
+
         ofile << n->getCompiledScript();
         
         ofile << endl;
